@@ -12,8 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use olm_rs::errors::{OlmGroupSessionError, OlmSessionError};
-use ruma::{identifiers::Error as IdentifierError, DeviceId, UserId};
+use ruma::{IdParseError, OwnedDeviceId, OwnedRoomId, OwnedUserId};
 use serde_json::Error as SerdeError;
 use thiserror::Error;
 
@@ -34,13 +33,13 @@ pub enum OlmError {
     #[error(transparent)]
     JsonError(#[from] SerdeError),
 
-    /// The underlying Olm session operation returned an error.
-    #[error("can't finish Olm Session operation {0}")]
-    OlmSession(#[from] OlmSessionError),
+    /// The event could not have been decrypted.
+    #[error(transparent)]
+    Decryption(#[from] vodozemac::olm::DecryptionError),
 
-    /// The underlying group session operation returned an error.
-    #[error("can't finish Olm Session operation {0}")]
-    OlmGroupSession(#[from] OlmGroupSessionError),
+    /// The received room key couldn't be converted into a valid Megolm session.
+    #[error(transparent)]
+    SessionCreation(#[from] vodozemac::megolm::SessionKeyDecodeError),
 
     /// The storage layer returned an error.
     #[error("failed to read or write to the crypto store {0}")]
@@ -50,12 +49,12 @@ pub enum OlmError {
     #[error(
         "decryption failed likely because an Olm session from {0} with sender key {1} was wedged"
     )]
-    SessionWedged(UserId, String),
+    SessionWedged(OwnedUserId, String),
 
     /// An Olm message got replayed while the Olm ratchet has already moved
     /// forward.
     #[error("decryption failed because an Olm message from {0} with sender key {1} was replayed")]
-    ReplayedMessage(UserId, String),
+    ReplayedMessage(OwnedUserId, String),
 
     /// Encryption failed because the device does not have a valid Olm session
     /// with us.
@@ -77,19 +76,23 @@ pub enum MegolmError {
     #[error(transparent)]
     JsonError(#[from] SerdeError),
 
-    /// Decryption failed because the session needed to decrypt the event is
-    /// missing.
-    #[error("decryption failed because the session to decrypt the message is missing")]
-    MissingSession,
+    /// Decryption failed because we're missing the room key that was to encrypt
+    /// the event.
+    #[error("decryption failed because the room key is missing")]
+    MissingRoomKey,
 
-    /// The underlying group session operation returned an error.
-    #[error("can't finish Olm group session operation {0}")]
-    OlmGroupSession(#[from] OlmGroupSessionError),
+    /// The encrypted megolm message couldn't be decoded.
+    #[error(transparent)]
+    Decode(#[from] vodozemac::DecodeError),
 
     /// The room where a group session should be shared is not encrypted.
     #[error("The room where a group session should be shared is not encrypted")]
     EncryptionNotEnabled,
 
+    /// The event could not have been decrypted.
+    #[error(transparent)]
+    Decryption(#[from] vodozemac::megolm::DecryptionError),
+
     /// The storage layer returned an error.
     #[error(transparent)]
     Store(#[from] CryptoStoreError),
@@ -97,8 +100,8 @@ pub enum MegolmError {
 
 #[derive(Error, Debug)]
 pub enum EventError {
-    #[error("the Olm message has a unsupported type")]
-    UnsupportedOlmType,
+    #[error("the Olm message has a unsupported type, got {0}, expected 0 or 1")]
+    UnsupportedOlmType(u64),
 
     #[error("the Encrypted message has been encrypted with a unsupported algorithm.")]
     UnsupportedAlgorithm,
@@ -118,21 +121,23 @@ pub enum EventError {
     #[error("the Encrypted message is missing the field {0}")]
     MissingField(String),
 
-    #[error("the sender of the plaintext doesn't match the sender of the encrypted message.")]
-    MissmatchedSender,
+    #[error(
+        "the sender of the plaintext doesn't match the sender of the encrypted \
+        message, got {0}, expected {0}"
+    )]
+    MismatchedSender(OwnedUserId, OwnedUserId),
 
-    #[error("the keys of the message don't match the keys in our database.")]
-    MissmatchedKeys,
-}
+    #[error(
+        "the public that was part of the message doesn't match to the key we \
+        have stored, expected {0}, got {0}"
+    )]
+    MismatchedKeys(String, String),
 
-#[derive(Error, Debug)]
-pub enum SessionUnpicklingError {
-    /// The underlying Olm session operation returned an error.
-    #[error("can't finish Olm Session operation {0}")]
-    OlmSession(#[from] OlmSessionError),
-    /// The Session timestamp was invalid.
-    #[error("can't load session timestamps")]
-    SessionTimestampError,
+    #[error(
+        "the room id of the room key doesn't match the room id of the \
+        decrypted event: expected {0}, got {:1}"
+    )]
+    MismatchedRoom(OwnedRoomId, Option<OwnedRoomId>),
 }
 
 /// Error type describin different errors that happen when we check or create
@@ -145,7 +150,7 @@ pub enum SignatureError {
 
     /// The ID of the signing key isn't a valid key ID.
     #[error("the ID of the signing key is invalid")]
-    InvalidKeyId(#[from] IdentifierError),
+    InvalidKeyId(#[from] IdParseError),
 
     /// The signing key that should create or check a signature is missing.
     #[error("the signing key is missing from the object that signed the message")]
@@ -154,7 +159,7 @@ pub enum SignatureError {
     /// The user id of signing key differs from the user id that provided the
     /// signature.
     #[error("the user id of the signing key differs user id that provided the signature")]
-    UserIdMissmatch,
+    UserIdMismatch,
 
     /// The provided JSON value that was signed and the signature should be
     /// checked isn't a valid JSON object.
@@ -167,8 +172,12 @@ pub enum SignatureError {
     NoSignatureFound,
 
     /// The signature couldn't be verified.
-    #[error("the signature didn't match the provided key")]
-    VerificationError,
+    #[error(transparent)]
+    VerificationError(#[from] vodozemac::SignatureError),
+
+    /// The public key isn't a valid ed25519 key.
+    #[error(transparent)]
+    InvalidKey(#[from] vodozemac::KeyError),
 
     /// The signed object couldn't be deserialized.
     #[error(transparent)]
@@ -176,29 +185,33 @@ pub enum SignatureError {
 }
 
 #[derive(Error, Debug)]
-pub(crate) enum SessionCreationError {
+pub enum SessionCreationError {
     #[error(
         "Failed to create a new Olm session for {0} {1}, the requested \
         one-time key isn't a signed curve key"
     )]
-    OneTimeKeyNotSigned(UserId, Box<DeviceId>),
+    OneTimeKeyNotSigned(OwnedUserId, OwnedDeviceId),
     #[error(
         "Tried to create a new Olm session for {0} {1}, but the signed \
         one-time key is missing"
     )]
-    OneTimeKeyMissing(UserId, Box<DeviceId>),
+    OneTimeKeyMissing(OwnedUserId, OwnedDeviceId),
     #[error(
         "Tried to create a new Olm session for {0} {1}, but the one-time \
         key algorithm is unsupported"
     )]
-    OneTimeKeyUnknown(UserId, Box<DeviceId>),
+    OneTimeKeyUnknown(OwnedUserId, OwnedDeviceId),
     #[error("Failed to verify the one-time key signatures for {0} {1}: {2:?}")]
-    InvalidSignature(UserId, Box<DeviceId>, SignatureError),
+    InvalidSignature(OwnedUserId, OwnedDeviceId, SignatureError),
     #[error(
         "Tried to create an Olm session for {0} {1}, but the device is missing \
         a curve25519 key"
     )]
-    DeviceMissingCurveKey(UserId, Box<DeviceId>),
-    #[error("Error creating new Olm session for {0} {1}: {2:?}")]
-    OlmError(UserId, Box<DeviceId>, OlmSessionError),
+    DeviceMissingCurveKey(OwnedUserId, OwnedDeviceId),
+    #[error("Error deserializing the one-time key: {0}")]
+    InvalidJson(#[from] serde_json::Error),
+    #[error("The given curve25519 key is not a valid key")]
+    InvalidCurveKey(#[from] vodozemac::KeyError),
+    #[error(transparent)]
+    InboundCreation(#[from] vodozemac::olm::SessionCreationError),
 }
