@@ -18,26 +18,29 @@ use std::{
 };
 
 use aes::{
-    cipher::{generic_array::GenericArray, FromBlockCipher, NewBlockCipher, StreamCipher},
-    Aes256, Aes256Ctr,
+    cipher::{generic_array::GenericArray, KeyIvInit, StreamCipher},
+    Aes256,
 };
 use base64::DecodeError;
-use getrandom::getrandom;
-use ruma::events::room::{EncryptedFile, JsonWebKey, JsonWebKeyInit};
+use rand::{thread_rng, RngCore};
+use ruma::{
+    events::room::{EncryptedFile, JsonWebKey, JsonWebKeyInit},
+    serde::Base64,
+};
 use serde::{Deserialize, Serialize};
 use sha2::{Digest, Sha256};
 use thiserror::Error;
-use zeroize::Zeroizing;
-
-use crate::utilities::{decode, decode_url_safe, encode, encode_url_safe};
+use zeroize::Zeroize;
 
 const IV_SIZE: usize = 16;
 const KEY_SIZE: usize = 32;
 const VERSION: &str = "v2";
 
+type Aes256Ctr = ctr::Ctr128BE<Aes256>;
+
 /// A wrapper that transparently encrypts anything that implements `Read` as an
 /// Matrix attachment.
-pub struct AttachmentDecryptor<'a, R: 'a + Read> {
+pub struct AttachmentDecryptor<'a, R: Read> {
     inner: &'a mut R,
     expected_hash: Vec<u8>,
     sha: Sha256,
@@ -125,37 +128,45 @@ impl<'a, R: Read + 'a> AttachmentDecryptor<'a, R> {
     /// ```
     pub fn new(
         input: &'a mut R,
-        info: EncryptionInfo,
+        info: MediaEncryptionInfo,
     ) -> Result<AttachmentDecryptor<'a, R>, DecryptorError> {
         if info.version != VERSION {
             return Err(DecryptorError::UnknownVersion);
         }
 
-        let hash = decode(info.hashes.get("sha256").ok_or(DecryptorError::MissingHash)?)?;
-        let key = Zeroizing::from(decode_url_safe(info.web_key.k)?);
-        let iv = decode(info.iv)?;
+        let hash =
+            info.hashes.get("sha256").ok_or(DecryptorError::MissingHash)?.as_bytes().to_owned();
+        let mut key = info.web_key.k.into_inner();
+        let iv = info.iv.into_inner();
+
+        if key.len() != KEY_SIZE {
+            return Err(DecryptorError::KeyNonceLength);
+        }
+
+        let key_array = GenericArray::from_slice(&key);
         let iv = GenericArray::from_exact_iter(iv).ok_or(DecryptorError::KeyNonceLength)?;
 
         let sha = Sha256::default();
-        let aes = Aes256::new_from_slice(&key).map_err(|_| DecryptorError::KeyNonceLength)?;
-        let aes = Aes256Ctr::from_block_cipher(aes, &iv);
+
+        let aes = Aes256Ctr::new(key_array, &iv);
+        key.zeroize();
 
         Ok(AttachmentDecryptor { inner: input, expected_hash: hash, sha, aes })
     }
 }
 
 /// A wrapper that transparently encrypts anything that implements `Read`.
-pub struct AttachmentEncryptor<'a, R: Read + 'a> {
+pub struct AttachmentEncryptor<'a, R: Read + ?Sized> {
     finished: bool,
     inner: &'a mut R,
     web_key: JsonWebKey,
-    iv: String,
-    hashes: BTreeMap<String, String>,
+    iv: Base64,
+    hashes: BTreeMap<String, Base64>,
     aes: Aes256Ctr,
     sha: Sha256,
 }
 
-impl<'a, R: 'a + Read + std::fmt::Debug> std::fmt::Debug for AttachmentEncryptor<'a, R> {
+impl<'a, R: 'a + Read + std::fmt::Debug + ?Sized> std::fmt::Debug for AttachmentEncryptor<'a, R> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("AttachmentEncryptor")
             .field("inner", &self.inner)
@@ -164,13 +175,15 @@ impl<'a, R: 'a + Read + std::fmt::Debug> std::fmt::Debug for AttachmentEncryptor
     }
 }
 
-impl<'a, R: Read + 'a> Read for AttachmentEncryptor<'a, R> {
+impl<'a, R: Read + ?Sized + 'a> Read for AttachmentEncryptor<'a, R> {
     fn read(&mut self, buf: &mut [u8]) -> std::io::Result<usize> {
         let read_bytes = self.inner.read(buf)?;
 
         if read_bytes == 0 {
             let hash = self.sha.finalize_reset();
-            self.hashes.entry("sha256".to_owned()).or_insert_with(|| encode(hash));
+            self.hashes
+                .entry("sha256".to_owned())
+                .or_insert_with(|| Base64::new(hash.as_slice().to_owned()));
             Ok(0)
         } else {
             self.aes.apply_keystream(&mut buf[0..read_bytes]);
@@ -181,7 +194,7 @@ impl<'a, R: Read + 'a> Read for AttachmentEncryptor<'a, R> {
     }
 }
 
-impl<'a, R: Read + 'a> AttachmentEncryptor<'a, R> {
+impl<'a, R: Read + ?Sized + 'a> AttachmentEncryptor<'a, R> {
     /// Wrap the given reader encrypting all the data we read from it.
     ///
     /// After all the reads are done, and all the data is encrypted that we wish
@@ -211,27 +224,29 @@ impl<'a, R: Read + 'a> AttachmentEncryptor<'a, R> {
     /// let key = encryptor.finish();
     /// ```
     pub fn new(reader: &'a mut R) -> Self {
-        let mut key = Zeroizing::new([0u8; KEY_SIZE]);
-        let mut iv = Zeroizing::new([0u8; IV_SIZE]);
+        let mut key = [0u8; KEY_SIZE];
+        let mut iv = [0u8; IV_SIZE];
 
-        getrandom(&mut *key).expect("Can't generate randomness");
+        let mut rng = thread_rng();
+
+        rng.fill_bytes(&mut key);
         // Only populate the first 8 bytes with randomness, the rest is 0
         // initialized for the counter.
-        getrandom(&mut iv[0..8]).expect("Can't generate randomness");
+        rng.fill_bytes(&mut iv[0..8]);
 
         let web_key = JsonWebKey::from(JsonWebKeyInit {
             kty: "oct".to_owned(),
             key_ops: vec!["encrypt".to_owned(), "decrypt".to_owned()],
             alg: "A256CTR".to_owned(),
-            k: encode_url_safe(&*key),
+            k: Base64::new(key.to_vec()),
             ext: true,
         });
-        let encoded_iv = encode(&*iv);
-        let iv = GenericArray::from_slice(&*iv);
-        let key = GenericArray::from_slice(&*key);
+        let encoded_iv = Base64::new((iv).to_vec());
 
-        let aes = Aes256::new(key);
-        let aes = Aes256Ctr::from_block_cipher(aes, iv);
+        let key_array = &key.into();
+
+        let aes = Aes256Ctr::new(key_array, &iv.into());
+        key.zeroize();
 
         AttachmentEncryptor {
             finished: false,
@@ -245,12 +260,14 @@ impl<'a, R: Read + 'a> AttachmentEncryptor<'a, R> {
     }
 
     /// Consume the encryptor and get the encryption key.
-    pub fn finish(mut self) -> EncryptionInfo {
+    pub fn finish(mut self) -> MediaEncryptionInfo {
         let hash = self.sha.finalize();
-        self.hashes.entry("sha256".to_owned()).or_insert_with(|| encode(hash));
+        self.hashes
+            .entry("sha256".to_owned())
+            .or_insert_with(|| Base64::new(hash.as_slice().to_owned()));
 
-        EncryptionInfo {
-            version: VERSION.to_string(),
+        MediaEncryptionInfo {
+            version: VERSION.to_owned(),
             hashes: self.hashes,
             iv: self.iv,
             web_key: self.web_key,
@@ -261,38 +278,38 @@ impl<'a, R: Read + 'a> AttachmentEncryptor<'a, R> {
 /// Struct holding all the information that is needed to decrypt an encrypted
 /// file.
 #[derive(Debug, Serialize, Deserialize)]
-pub struct EncryptionInfo {
+pub struct MediaEncryptionInfo {
     #[serde(rename = "v")]
     /// The version of the encryption scheme.
     pub version: String,
     /// The web key that was used to encrypt the file.
     pub web_key: JsonWebKey,
     /// The initialization vector that was used to encrypt the file.
-    pub iv: String,
+    pub iv: Base64,
     /// The hashes that can be used to check the validity of the file.
-    pub hashes: BTreeMap<String, String>,
+    pub hashes: BTreeMap<String, Base64>,
 }
 
-impl From<EncryptedFile> for EncryptionInfo {
+impl From<EncryptedFile> for MediaEncryptionInfo {
     fn from(file: EncryptedFile) -> Self {
         Self { version: file.v, web_key: file.key, iv: file.iv, hashes: file.hashes }
     }
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use std::io::{Cursor, Read};
 
     use serde_json::json;
 
-    use super::{AttachmentDecryptor, AttachmentEncryptor, EncryptionInfo};
+    use super::{AttachmentDecryptor, AttachmentEncryptor, MediaEncryptionInfo};
 
     const EXAMPLE_DATA: &[u8] = &[
         179, 154, 118, 127, 186, 127, 110, 33, 203, 33, 33, 134, 67, 100, 173, 46, 235, 27, 215,
         172, 36, 26, 75, 47, 33, 160,
     ];
 
-    fn example_key() -> EncryptionInfo {
+    fn example_key() -> MediaEncryptionInfo {
         let info = json!({
             "v": "v2",
             "web_key": {
