@@ -1,5 +1,7 @@
 mod attachments;
 mod key_export;
 
-pub use attachments::{AttachmentDecryptor, AttachmentEncryptor, DecryptorError, EncryptionInfo};
+pub use attachments::{
+    AttachmentDecryptor, AttachmentEncryptor, DecryptorError, MediaEncryptionInfo,
+};
 pub use key_export::{decrypt_key_export, encrypt_key_export, KeyExportError};
