@@ -25,40 +25,37 @@ use std::{
 use atomic::Atomic;
 use matrix_sdk_common::locks::Mutex;
 use ruma::{
-    api::client::r0::keys::upload_signatures::Request as SignatureUploadRequest,
-    encryption::{DeviceKeys, SignedKey},
+    api::client::keys::upload_signatures::v3::Request as SignatureUploadRequest,
     events::{
-        forwarded_room_key::ForwardedRoomKeyToDeviceEventContent,
-        key::verification::VerificationMethod, room::encrypted::EncryptedToDeviceEventContent,
+        forwarded_room_key::ToDeviceForwardedRoomKeyEventContent,
+        key::verification::VerificationMethod, room::encrypted::ToDeviceRoomEncryptedEventContent,
         AnyToDeviceEventContent,
     },
-    DeviceId, DeviceIdBox, DeviceKeyAlgorithm, DeviceKeyId, EventEncryptionAlgorithm, UserId,
+    DeviceId, DeviceKeyAlgorithm, DeviceKeyId, EventEncryptionAlgorithm, OwnedDeviceId,
+    OwnedDeviceKeyId, OwnedUserId, UserId,
 };
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
 use serde_json::{json, Value};
 use tracing::warn;
+use vodozemac::{Curve25519PublicKey, Ed25519PublicKey};
 
 use super::{atomic_bool_deserializer, atomic_bool_serializer};
+#[cfg(any(test, feature = "testing"))]
+use crate::OlmMachine;
 use crate::{
     error::{EventError, OlmError, OlmResult, SignatureError},
     identities::{ReadOnlyOwnUserIdentity, ReadOnlyUserIdentities},
-    olm::{InboundGroupSession, Session, Utility},
+    olm::{InboundGroupSession, Session, VerifyJson},
     store::{Changes, CryptoStore, DeviceChanges, Result as StoreResult},
+    types::{DeviceKey, DeviceKeys, SignedKey},
     verification::VerificationMachine,
-    OutgoingVerificationRequest, Sas, ToDeviceRequest, VerificationRequest,
+    OutgoingVerificationRequest, ReadOnlyAccount, Sas, ToDeviceRequest, VerificationRequest,
 };
-#[cfg(test)]
-use crate::{OlmMachine, ReadOnlyAccount};
 
 /// A read-only version of a `Device`.
 #[derive(Clone, Serialize, Deserialize)]
 pub struct ReadOnlyDevice {
-    user_id: Arc<UserId>,
-    device_id: Arc<DeviceId>,
-    algorithms: Arc<[EventEncryptionAlgorithm]>,
-    keys: Arc<BTreeMap<DeviceKeyId, String>>,
-    pub(crate) signatures: Arc<BTreeMap<UserId, BTreeMap<DeviceKeyId, String>>>,
-    display_name: Arc<Option<String>>,
+    pub(crate) inner: Arc<DeviceKeys>,
     #[serde(
         serialize_with = "atomic_bool_serializer",
         deserialize_with = "atomic_bool_deserializer"
@@ -74,9 +71,9 @@ pub struct ReadOnlyDevice {
 impl std::fmt::Debug for ReadOnlyDevice {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("ReadOnlyDevice")
-            .field("user_id", self.user_id())
+            .field("user_id", &self.user_id())
             .field("device_id", &self.device_id())
-            .field("display_name", self.display_name())
+            .field("display_name", &self.display_name())
             .field("keys", self.keys())
             .field("deleted", &self.deleted.load(Ordering::SeqCst))
             .field("trust_state", &self.trust_state)
@@ -181,8 +178,8 @@ impl Device {
 
     /// Get the Olm sessions that belong to this device.
     pub(crate) async fn get_sessions(&self) -> StoreResult<Option<Arc<Mutex<Vec<Session>>>>> {
-        if let Some(k) = self.get_key(DeviceKeyAlgorithm::Curve25519) {
-            self.verification_machine.store.get_sessions(k).await
+        if let Some(k) = self.curve25519_key() {
+            self.verification_machine.store.get_sessions(&k.to_base64()).await
         } else {
             Ok(None)
         }
@@ -221,13 +218,14 @@ impl Device {
         if self.user_id() == self.verification_machine.own_user_id() {
             Ok(self
                 .verification_machine
+                .store
                 .private_identity
                 .lock()
                 .await
                 .sign_device(&self.inner)
                 .await?)
         } else {
-            Err(SignatureError::UserIdMissmatch)
+            Err(SignatureError::UserIdMismatch)
         }
     }
 
@@ -258,7 +256,7 @@ impl Device {
     pub(crate) async fn encrypt(
         &self,
         content: AnyToDeviceEventContent,
-    ) -> OlmResult<(Session, EncryptedToDeviceEventContent)> {
+    ) -> OlmResult<(Session, ToDeviceRoomEncryptedEventContent)> {
         self.inner.encrypt(self.verification_machine.store.inner(), content).await
     }
 
@@ -268,14 +266,14 @@ impl Device {
         &self,
         session: InboundGroupSession,
         message_index: Option<u32>,
-    ) -> OlmResult<(Session, EncryptedToDeviceEventContent)> {
+    ) -> OlmResult<(Session, ToDeviceRoomEncryptedEventContent)> {
         let export = if let Some(index) = message_index {
             session.export_at_index(index).await
         } else {
             session.export().await
         };
 
-        let content: ForwardedRoomKeyToDeviceEventContent = if let Ok(c) = export.try_into() {
+        let content: ToDeviceForwardedRoomKeyEventContent = if let Ok(c) = export.try_into() {
             c
         } else {
             // TODO remove this panic.
@@ -295,7 +293,7 @@ impl Device {
 /// A read only view over all devices belonging to a user.
 #[derive(Debug)]
 pub struct UserDevices {
-    pub(crate) inner: HashMap<DeviceIdBox, ReadOnlyDevice>,
+    pub(crate) inner: HashMap<OwnedDeviceId, ReadOnlyDevice>,
     pub(crate) verification_machine: VerificationMachine,
     pub(crate) own_identity: Option<ReadOnlyOwnUserIdentity>,
     pub(crate) device_owner_identity: Option<ReadOnlyUserIdentities>,
@@ -312,15 +310,31 @@ impl UserDevices {
         })
     }
 
+    fn own_user_id(&self) -> &UserId {
+        self.verification_machine.own_user_id()
+    }
+
+    fn own_device_id(&self) -> &DeviceId {
+        self.verification_machine.own_device_id()
+    }
+
     /// Returns true if there is at least one devices of this user that is
     /// considered to be verified, false otherwise.
+    ///
+    /// This won't consider your own device as verified, as your own device is
+    /// always implicitly verified.
     pub fn is_any_verified(&self) -> bool {
-        self.inner.values().any(|d| d.verified(&self.own_identity, &self.device_owner_identity))
+        self.inner
+            .values()
+            .filter(|d| {
+                !(d.user_id() == self.own_user_id() && d.device_id() == self.own_device_id())
+            })
+            .any(|d| d.verified(&self.own_identity, &self.device_owner_identity))
     }
 
     /// Iterator over all the device ids of the user devices.
-    pub fn keys(&self) -> impl Iterator<Item = &DeviceIdBox> {
-        self.inner.keys()
+    pub fn keys(&self) -> impl Iterator<Item = &DeviceId> {
+        self.inner.keys().map(Deref::deref)
     }
 
     /// Iterator over all the devices of the user devices.
@@ -360,56 +374,67 @@ impl From<i64> for LocalTrust {
 }
 
 impl ReadOnlyDevice {
-    /// Create a new Device.
-    pub fn new(
-        user_id: UserId,
-        device_id: Box<DeviceId>,
-        display_name: Option<String>,
-        trust_state: LocalTrust,
-        algorithms: Vec<EventEncryptionAlgorithm>,
-        keys: BTreeMap<DeviceKeyId, String>,
-        signatures: BTreeMap<UserId, BTreeMap<DeviceKeyId, String>>,
-    ) -> Self {
+    /// Create a new Device, this constructor skips signature verification of
+    /// the keys, `TryFrom` should be used for completely new devices we
+    /// receive.
+    pub fn new(device_keys: DeviceKeys, trust_state: LocalTrust) -> Self {
         Self {
-            user_id: Arc::new(user_id),
-            device_id: device_id.into(),
-            display_name: Arc::new(display_name),
+            inner: device_keys.into(),
             trust_state: Arc::new(Atomic::new(trust_state)),
-            signatures: Arc::new(signatures),
-            algorithms: algorithms.into(),
-            keys: Arc::new(keys),
             deleted: Arc::new(AtomicBool::new(false)),
         }
     }
 
     /// The user id of the device owner.
     pub fn user_id(&self) -> &UserId {
-        &self.user_id
+        &self.inner.user_id
     }
 
     /// The unique ID of the device.
     pub fn device_id(&self) -> &DeviceId {
-        &self.device_id
+        &self.inner.device_id
     }
 
     /// Get the human readable name of the device.
-    pub fn display_name(&self) -> &Option<String> {
-        &self.display_name
+    pub fn display_name(&self) -> Option<&str> {
+        self.inner.unsigned.device_display_name.as_deref()
     }
 
     /// Get the key of the given key algorithm belonging to this device.
-    pub fn get_key(&self, algorithm: DeviceKeyAlgorithm) -> Option<&String> {
-        self.keys.get(&DeviceKeyId::from_parts(algorithm, &self.device_id))
+    pub fn get_key(&self, algorithm: DeviceKeyAlgorithm) -> Option<&DeviceKey> {
+        self.inner.keys.get(&DeviceKeyId::from_parts(algorithm, self.device_id()))
+    }
+
+    /// Get the Curve25519 key of the given device.
+    pub fn curve25519_key(&self) -> Option<Curve25519PublicKey> {
+        self.get_key(DeviceKeyAlgorithm::Curve25519).and_then(|k| {
+            if let DeviceKey::Curve25519(k) = k {
+                Some(*k)
+            } else {
+                None
+            }
+        })
+    }
+
+    /// Get the Ed25519 key of the given device.
+    pub fn ed25519_key(&self) -> Option<Ed25519PublicKey> {
+        self.get_key(DeviceKeyAlgorithm::Ed25519).and_then(|k| {
+            if let DeviceKey::Ed25519(k) = k {
+                Some(*k)
+            } else {
+                None
+            }
+        })
     }
 
     /// Get a map containing all the device keys.
-    pub fn keys(&self) -> &BTreeMap<DeviceKeyId, String> {
-        &self.keys
+    pub fn keys(&self) -> &BTreeMap<OwnedDeviceKeyId, DeviceKey> {
+        &self.inner.keys
     }
 
     /// Get a map containing all the device signatures.
-    pub fn signatures(&self) -> &BTreeMap<UserId, BTreeMap<DeviceKeyId, String>> {
-        &self.signatures
+    pub fn signatures(&self) -> &BTreeMap<OwnedUserId, BTreeMap<OwnedDeviceKeyId, String>> {
+        &self.inner.signatures
     }
 
     /// Get the trust state of the device.
@@ -439,7 +464,7 @@ impl ReadOnlyDevice {
 
     /// Get the list of algorithms this device supports.
     pub fn algorithms(&self) -> &[EventEncryptionAlgorithm] {
-        &self.algorithms
+        &self.inner.algorithms
     }
 
     /// Is the device deleted.
@@ -488,21 +513,22 @@ impl ReadOnlyDevice {
         &self,
         store: &dyn CryptoStore,
         content: AnyToDeviceEventContent,
-    ) -> OlmResult<(Session, EncryptedToDeviceEventContent)> {
-        let sender_key = if let Some(k) = self.get_key(DeviceKeyAlgorithm::Curve25519) {
+    ) -> OlmResult<(Session, ToDeviceRoomEncryptedEventContent)> {
+        let sender_key = if let Some(k) = self.curve25519_key() {
             k
         } else {
             warn!(
-                "Trying to encrypt a Megolm session for user {} on device {}, \
-                but the device doesn't have a curve25519 key",
-                self.user_id(),
-                self.device_id()
+                user_id = %self.user_id(),
+                device_id = %self.device_id(),
+                "Trying to encrypt a Megolm session, but the device doesn't \
+                have a curve25519 key",
             );
             return Err(EventError::MissingSenderKey.into());
         };
 
-        let session = if let Some(s) = store.get_sessions(sender_key).await? {
-            let sessions = s.lock().await;
+        let session = if let Some(s) = store.get_sessions(&sender_key.to_base64()).await? {
+            let mut sessions = s.lock().await;
+            sessions.sort_by_key(|s| s.last_use_time);
             sessions.get(0).cloned()
         } else {
             None
@@ -528,39 +554,23 @@ impl ReadOnlyDevice {
     /// Update a device with a new device keys struct.
     pub(crate) fn update_device(&mut self, device_keys: &DeviceKeys) -> Result<(), SignatureError> {
         self.verify_device_keys(device_keys)?;
-
-        let display_name = Arc::new(device_keys.unsigned.device_display_name.clone());
-
-        self.algorithms = device_keys.algorithms.as_slice().into();
-        self.keys = Arc::new(device_keys.keys.clone());
-        self.signatures = Arc::new(device_keys.signatures.clone());
-        self.display_name = display_name;
+        self.inner = device_keys.clone().into();
 
         Ok(())
     }
 
-    fn is_signed_by_device(&self, json: &mut Value) -> Result<(), SignatureError> {
-        let signing_key =
-            self.get_key(DeviceKeyAlgorithm::Ed25519).ok_or(SignatureError::MissingSigningKey)?;
+    pub(crate) fn is_signed_by_device(&self, json: &mut Value) -> Result<(), SignatureError> {
+        let key = self.ed25519_key().ok_or(SignatureError::MissingSigningKey)?;
 
-        let utility = Utility::new();
-
-        utility.verify_json(
-            &self.user_id,
+        key.verify_json(
+            self.user_id(),
             &DeviceKeyId::from_parts(DeviceKeyAlgorithm::Ed25519, self.device_id()),
-            signing_key,
             json,
         )
     }
 
-    pub(crate) fn as_device_keys(&self) -> DeviceKeys {
-        DeviceKeys::new(
-            self.user_id().clone(),
-            self.device_id().into(),
-            self.algorithms().to_vec(),
-            self.keys().clone(),
-            self.signatures().to_owned(),
-        )
+    pub(crate) fn as_device_keys(&self) -> &DeviceKeys {
+        &self.inner
     }
 
     pub(crate) fn verify_device_keys(
@@ -583,15 +593,31 @@ impl ReadOnlyDevice {
         self.deleted.store(true, Ordering::Relaxed);
     }
 
-    #[cfg(test)]
+    #[cfg(any(test, feature = "testing"))]
+    #[allow(dead_code)]
+    /// Generate the Device from the reference of an OlmMachine.
+    ///
+    /// TESTING FACILITY ONLY, DO NOT USE OUTSIDE OF TESTS
     pub async fn from_machine(machine: &OlmMachine) -> ReadOnlyDevice {
         ReadOnlyDevice::from_account(machine.account()).await
     }
 
-    #[cfg(test)]
+    /// Create a `ReadOnlyDevice` from an `Account`
+    ///
+    /// We will have our own device in the store once we receive a keys/query
+    /// response, but this is useful to create it before we receive such a
+    /// response.
+    ///
+    /// It also makes it easier to check that the server doesn't lie about our
+    /// own device.
+    ///
+    /// *Don't* use this after we received a keys/query response, other
+    /// users/devices might add signatures to our own device, which can't be
+    /// replicated locally.
     pub async fn from_account(account: &ReadOnlyAccount) -> ReadOnlyDevice {
         let device_keys = account.device_keys().await;
-        ReadOnlyDevice::try_from(&device_keys).unwrap()
+        ReadOnlyDevice::try_from(&device_keys)
+            .expect("Creating a device from our own account should always succeed")
     }
 }
 
@@ -600,12 +626,7 @@ impl TryFrom<&DeviceKeys> for ReadOnlyDevice {
 
     fn try_from(device_keys: &DeviceKeys) -> Result<Self, Self::Error> {
         let device = Self {
-            user_id: Arc::new(device_keys.user_id.clone()),
-            device_id: device_keys.device_id.clone().into(),
-            algorithms: device_keys.algorithms.as_slice().into(),
-            signatures: Arc::new(device_keys.signatures.clone()),
-            keys: Arc::new(device_keys.keys.clone()),
-            display_name: Arc::new(device_keys.unsigned.device_display_name.clone()),
+            inner: device_keys.clone().into(),
             deleted: Arc::new(AtomicBool::new(false)),
             trust_state: Arc::new(Atomic::new(LocalTrust::Unset)),
         };
@@ -621,16 +642,16 @@ impl PartialEq for ReadOnlyDevice {
     }
 }
 
-#[cfg(test)]
-pub(crate) mod test {
-    use std::convert::TryFrom;
-
-    use ruma::{encryption::DeviceKeys, user_id, DeviceKeyAlgorithm};
+#[cfg(any(test, feature = "testing"))]
+pub(crate) mod testing {
+    //! Testing Facilities for Device Management
+    #![allow(dead_code)]
     use serde_json::json;
 
-    use crate::identities::{LocalTrust, ReadOnlyDevice};
+    use crate::{identities::ReadOnlyDevice, types::DeviceKeys};
 
-    fn device_keys() -> DeviceKeys {
+    /// Generate default DeviceKeys for tests
+    pub fn device_keys() -> DeviceKeys {
         let device_keys = json!({
           "algorithms": vec![
               "m.olm.v1.curve25519-aes-sha2",
@@ -655,10 +676,20 @@ pub(crate) mod test {
         serde_json::from_value(device_keys).unwrap()
     }
 
-    pub(crate) fn get_device() -> ReadOnlyDevice {
+    /// Generate default ReadOnlyDevice for tests
+    pub fn get_device() -> ReadOnlyDevice {
         let device_keys = device_keys();
         ReadOnlyDevice::try_from(&device_keys).unwrap()
     }
+}
+
+#[cfg(test)]
+pub(crate) mod tests {
+    use ruma::user_id;
+    use vodozemac::{Curve25519PublicKey, Ed25519PublicKey};
+
+    use super::testing::{device_keys, get_device};
+    use crate::identities::LocalTrust;
 
     #[test]
     fn create_a_device() {
@@ -667,18 +698,19 @@ pub(crate) mod test {
 
         let device = get_device();
 
-        assert_eq!(&user_id, device.user_id());
+        assert_eq!(user_id, device.user_id());
         assert_eq!(device_id, device.device_id());
-        assert_eq!(device.algorithms.len(), 2);
+        assert_eq!(device.algorithms().len(), 2);
         assert_eq!(LocalTrust::Unset, device.local_trust_state());
-        assert_eq!("Alice's mobile phone", device.display_name().as_ref().unwrap());
+        assert_eq!("Alice's mobile phone", device.display_name().unwrap());
         assert_eq!(
-            device.get_key(DeviceKeyAlgorithm::Curve25519).unwrap(),
-            "xfgbLIC5WAl1OIkpOzoxpCe8FsRDT6nch7NQsOb15nc"
+            device.curve25519_key().unwrap(),
+            Curve25519PublicKey::from_base64("xfgbLIC5WAl1OIkpOzoxpCe8FsRDT6nch7NQsOb15nc")
+                .unwrap(),
         );
         assert_eq!(
-            device.get_key(DeviceKeyAlgorithm::Ed25519).unwrap(),
-            "2/5LWJMow5zhJqakV88SIc7q/1pa8fmkfgAzx72w9G4"
+            device.ed25519_key().unwrap(),
+            Ed25519PublicKey::from_base64("2/5LWJMow5zhJqakV88SIc7q/1pa8fmkfgAzx72w9G4").unwrap(),
         );
     }
 
@@ -686,7 +718,7 @@ pub(crate) mod test {
     fn update_a_device() {
         let mut device = get_device();
 
-        assert_eq!("Alice's mobile phone", device.display_name().as_ref().unwrap());
+        assert_eq!("Alice's mobile phone", device.display_name().unwrap());
 
         let display_name = "Alice's work computer".to_owned();
 
