@@ -13,18 +13,19 @@
 // limitations under the License.
 
 use std::{
-    collections::{BTreeMap, HashSet},
+    collections::{BTreeMap, BTreeSet, HashSet},
     convert::TryFrom,
+    ops::Deref,
     sync::Arc,
 };
 
-use futures::future::join_all;
+use futures_util::future::join_all;
 use matrix_sdk_common::executor::spawn;
 use ruma::{
-    api::client::r0::keys::get_keys::Response as KeysQueryResponse, encryption::DeviceKeys,
-    DeviceId, DeviceIdBox, UserId,
+    api::client::keys::get_keys::v3::Response as KeysQueryResponse, serde::Raw, DeviceId,
+    OwnedDeviceId, OwnedUserId, UserId,
 };
-use tracing::{info, trace, warn};
+use tracing::{debug, info, trace, warn};
 
 use crate::{
     error::OlmResult,
@@ -35,6 +36,8 @@ use crate::{
     olm::PrivateCrossSigningIdentity,
     requests::KeysQueryRequest,
     store::{Changes, DeviceChanges, IdentityChanges, Result as StoreResult, Store},
+    types::{CrossSigningKey, DeviceKeys},
+    LocalTrust,
 };
 
 enum DeviceChange {
@@ -74,27 +77,59 @@ impl IdentityManager {
         &self,
         response: &KeysQueryResponse,
     ) -> OlmResult<(DeviceChanges, IdentityChanges)> {
-        let changed_devices =
-            self.handle_devices_from_key_query(response.device_keys.clone()).await?;
-        let (changed_identities, cross_signing_identity) =
-            self.handle_cross_singing_keys(response).await?;
+        debug!(
+            users = ?response.device_keys.keys().collect::<BTreeSet<_>>(),
+            failures = ?response.failures,
+            "Handling a keys query response"
+        );
+
+        let devices = self.handle_devices_from_key_query(response.device_keys.clone()).await?;
+        let (identities, cross_signing_identity) = self.handle_cross_singing_keys(response).await?;
 
         let changes = Changes {
-            identities: changed_identities.clone(),
-            devices: changed_devices.clone(),
+            identities: identities.clone(),
+            devices: devices.clone(),
             private_identity: cross_signing_identity,
             ..Default::default()
         };
 
         // TODO turn this into a single transaction.
         self.store.save_changes(changes).await?;
-        let updated_users: Vec<&UserId> = response.device_keys.keys().collect();
+        let updated_users: Vec<&UserId> = response.device_keys.keys().map(Deref::deref).collect();
 
         for user_id in updated_users {
             self.store.update_tracked_user(user_id, false).await?;
         }
 
-        Ok((changed_devices, changed_identities))
+        let changed_devices = devices.changed.iter().fold(BTreeMap::new(), |mut acc, d| {
+            acc.entry(d.user_id()).or_insert_with(BTreeSet::new).insert(d.device_id());
+            acc
+        });
+
+        let new_devices = devices.new.iter().fold(BTreeMap::new(), |mut acc, d| {
+            acc.entry(d.user_id()).or_insert_with(BTreeSet::new).insert(d.device_id());
+            acc
+        });
+
+        let deleted_devices = devices.deleted.iter().fold(BTreeMap::new(), |mut acc, d| {
+            acc.entry(d.user_id()).or_insert_with(BTreeSet::new).insert(d.device_id());
+            acc
+        });
+
+        let new_identities = identities.new.iter().map(|i| i.user_id()).collect::<BTreeSet<_>>();
+        let changed_identities =
+            identities.changed.iter().map(|i| i.user_id()).collect::<BTreeSet<_>>();
+
+        debug!(
+            ?new_devices,
+            ?changed_devices,
+            ?deleted_devices,
+            ?new_identities,
+            ?changed_identities,
+            "Finished handling of the keys/query response"
+        );
+
+        Ok((devices, identities))
     }
 
     async fn update_or_create_device(
@@ -107,11 +142,12 @@ impl IdentityManager {
         if let Some(mut device) = old_device {
             if let Err(e) = device.update_device(&device_keys) {
                 warn!(
-                    "Failed to update the device keys for {} {}: {:?}",
-                    device.user_id(),
-                    device.device_id(),
-                    e
+                    user_id = device.user_id().as_str(),
+                    device_id = device.device_id().as_str(),
+                    error = ?e,
+                    "Failed to update device keys",
                 );
+
                 Ok(DeviceChange::None)
             } else {
                 Ok(DeviceChange::Updated(device))
@@ -119,14 +155,46 @@ impl IdentityManager {
         } else {
             match ReadOnlyDevice::try_from(&device_keys) {
                 Ok(d) => {
-                    trace!("Adding a new device to the device store {:?}", d);
-                    Ok(DeviceChange::New(d))
+                    // If this is our own device, check that the server isn't
+                    // lying about our keys, also mark the device as locally
+                    // trusted.
+                    if d.user_id() == store.user_id() && d.device_id() == store.device_id() {
+                        let local_device_keys = store.account().unsigned_device_keys();
+
+                        if d.keys() == &local_device_keys.keys {
+                            d.set_trust_state(LocalTrust::Verified);
+
+                            trace!(
+                                user_id = d.user_id().as_str(),
+                                device_id = d.device_id().as_str(),
+                                keys = ?d.keys(),
+                                "Adding our own device to the device store, \
+                                marking it as locally verified",
+                            );
+
+                            Ok(DeviceChange::New(d))
+                        } else {
+                            Ok(DeviceChange::None)
+                        }
+                    } else {
+                        trace!(
+                            user_id = d.user_id().as_str(),
+                            device_id = d.device_id().as_str(),
+                            keys = ?d.keys(),
+                            "Adding a new device to the device store",
+                        );
+
+                        Ok(DeviceChange::New(d))
+                    }
                 }
                 Err(e) => {
                     warn!(
-                        "Failed to create a new device for {} {}: {:?}",
-                        device_keys.user_id, device_keys.device_id, e
+                        user_id = device_keys.user_id.as_str(),
+                        device_id = device_keys.device_id.as_str(),
+                        error = ?e,
+                        "Failed to create a new device",
                     );
+
                     Ok(DeviceChange::None)
                 }
             }
@@ -137,27 +205,40 @@ impl IdentityManager {
         store: Store,
         own_user_id: Arc<UserId>,
         own_device_id: Arc<DeviceId>,
-        user_id: UserId,
-        device_map: BTreeMap<DeviceIdBox, DeviceKeys>,
+        user_id: OwnedUserId,
+        device_map: BTreeMap<OwnedDeviceId, Raw<ruma::encryption::DeviceKeys>>,
     ) -> StoreResult<DeviceChanges> {
-        let own_device_id = (&*own_device_id).to_owned();
+        let own_device_id = (*own_device_id).to_owned();
 
         let mut changes = DeviceChanges::default();
 
-        let current_devices: HashSet<DeviceIdBox> = device_map.keys().cloned().collect();
+        let current_devices: HashSet<OwnedDeviceId> = device_map.keys().cloned().collect();
 
-        let tasks = device_map.into_iter().filter_map(|(device_id, device_keys)| {
-            if user_id != device_keys.user_id || device_id != device_keys.device_id {
+        let tasks = device_map.into_iter().filter_map(|(device_id, device_keys)| match device_keys
+            .deserialize_as::<DeviceKeys>(
+        ) {
+            Ok(device_keys) => {
+                if user_id != device_keys.user_id || device_id != device_keys.device_id {
+                    warn!(
+                        user_id = user_id.as_str(),
+                        device_id = device_id.as_str(),
+                        device_key_user = device_keys.user_id.as_str(),
+                        device_key_device_id = device_keys.device_id.as_str(),
+                        "Mismatch in the device keys payload",
+                    );
+                    None
+                } else {
+                    Some(spawn(Self::update_or_create_device(store.clone(), device_keys)))
+                }
+            }
+            Err(e) => {
                 warn!(
                     user_id = user_id.as_str(),
                     device_id = device_id.as_str(),
-                    device_key_user = device_keys.user_id.as_str(),
-                    device_key_device_id = device_keys.device_id.as_str(),
-                    "Mismatch in the device keys payload",
+                    error = ?e,
+                    "Device keys failed to deserialize",
                 );
                 None
-            } else {
-                Some(spawn(Self::update_or_create_device(store.clone(), device_keys)))
             }
         });
 
@@ -173,14 +254,18 @@ impl IdentityManager {
             }
         }
 
-        let current_devices: HashSet<&DeviceIdBox> = current_devices.iter().collect();
+        let current_devices: HashSet<&OwnedDeviceId> = current_devices.iter().collect();
         let stored_devices = store.get_readonly_devices_unfiltered(&user_id).await?;
-        let stored_devices_set: HashSet<&DeviceIdBox> = stored_devices.keys().collect();
+        let stored_devices_set: HashSet<&OwnedDeviceId> = stored_devices.keys().collect();
         let deleted_devices_set = stored_devices_set.difference(&current_devices);
 
         for device_id in deleted_devices_set {
             if user_id == *own_user_id && *device_id == &own_device_id {
-                warn!("Our own device has been deleted");
+                warn!(
+                    user_id = own_user_id.as_str(),
+                    device_id = own_device_id.as_str(),
+                    "Our own device has been deleted"
+                );
             } else if let Some(device) = stored_devices.get(*device_id) {
                 device.mark_as_deleted();
                 changes.deleted.push(device.clone());
@@ -201,7 +286,10 @@ impl IdentityManager {
     /// they are new, one of their properties has changed or they got deleted.
     async fn handle_devices_from_key_query(
         &self,
-        device_keys_map: BTreeMap<UserId, BTreeMap<DeviceIdBox, DeviceKeys>>,
+        device_keys_map: BTreeMap<
+            OwnedUserId,
+            BTreeMap<OwnedDeviceId, Raw<ruma::encryption::DeviceKeys>>,
+        >,
     ) -> StoreResult<DeviceChanges> {
         let mut changes = DeviceChanges::default();
 
@@ -241,100 +329,131 @@ impl IdentityManager {
         let mut changes = IdentityChanges::default();
         let mut changed_identity = None;
 
+        // TODO this is a bit chunky, refactor this into smaller methods.
+
         for (user_id, master_key) in &response.master_keys {
-            let master_key = MasterPubkey::from(master_key);
+            match master_key.deserialize_as::<CrossSigningKey>() {
+                Ok(master_key) => {
+                    let master_key = MasterPubkey::from(master_key);
 
-            let self_signing = if let Some(s) = response.self_signing_keys.get(user_id) {
-                SelfSigningPubkey::from(s)
-            } else {
-                warn!(
-                    user_id = user_id.as_str(),
-                    "A user identity didn't contain a self signing pubkey"
-                );
-                continue;
-            };
+                    let self_signing = if let Some(s) = response
+                        .self_signing_keys
+                        .get(user_id)
+                        .and_then(|k| k.deserialize_as::<CrossSigningKey>().ok())
+                    {
+                        SelfSigningPubkey::from(s)
+                    } else {
+                        warn!(
+                            user_id = user_id.as_str(),
+                            "A user identity didn't contain a self signing pubkey \
+                            or the key was invalid"
+                        );
+                        continue;
+                    };
 
-            let result = if let Some(mut i) = self.store.get_user_identity(user_id).await? {
-                match &mut i {
-                    ReadOnlyUserIdentities::Own(ref mut identity) => {
-                        let user_signing = if let Some(s) = response.user_signing_keys.get(user_id)
+                    let result = if let Some(mut i) = self.store.get_user_identity(user_id).await? {
+                        match &mut i {
+                            ReadOnlyUserIdentities::Own(ref mut identity) => {
+                                let user_signing = if let Some(s) = response
+                                    .user_signing_keys
+                                    .get(user_id)
+                                    .and_then(|k| k.deserialize_as::<CrossSigningKey>().ok())
+                                {
+                                    UserSigningPubkey::from(s)
+                                } else {
+                                    warn!(
+                                        user_id = user_id.as_str(),
+                                        "User identity for our own user didn't \
+                                        contain a user signing pubkey",
+                                    );
+                                    continue;
+                                };
+
+                                identity
+                                    .update(master_key, self_signing, user_signing)
+                                    .map(|_| (i, false))
+                            }
+                            ReadOnlyUserIdentities::Other(ref mut identity) => {
+                                identity.update(master_key, self_signing).map(|_| (i, false))
+                            }
+                        }
+                    } else if user_id == self.user_id() {
+                        if let Some(s) = response
+                            .user_signing_keys
+                            .get(user_id)
+                            .and_then(|k| k.deserialize_as::<CrossSigningKey>().ok())
                         {
-                            UserSigningPubkey::from(s)
+                            let user_signing = UserSigningPubkey::from(s);
+
+                            if master_key.user_id() != user_id
+                                || self_signing.user_id() != user_id
+                                || user_signing.user_id() != user_id
+                            {
+                                warn!(
+                                    user_id = user_id.as_str(),
+                                    "User ID mismatch in one of the cross signing keys",
+                                );
+                                continue;
+                            }
+
+                            ReadOnlyOwnUserIdentity::new(master_key, self_signing, user_signing)
+                                .map(|i| (ReadOnlyUserIdentities::Own(i), true))
                         } else {
                             warn!(
                                 user_id = user_id.as_str(),
-                                "User identity for our own user didn't \
-                                contain a user signing pubkey",
+                                "User identity for our own user didn't contain a \
+                                user signing pubkey or the key isn't valid",
                             );
                             continue;
-                        };
-
-                        identity.update(master_key, self_signing, user_signing).map(|_| (i, false))
-                    }
-                    ReadOnlyUserIdentities::Other(ref mut identity) => {
-                        identity.update(master_key, self_signing).map(|_| (i, false))
-                    }
-                }
-            } else if user_id == self.user_id() {
-                if let Some(s) = response.user_signing_keys.get(user_id) {
-                    let user_signing = UserSigningPubkey::from(s);
-
-                    if master_key.user_id() != user_id
-                        || self_signing.user_id() != user_id
-                        || user_signing.user_id() != user_id
-                    {
+                        }
+                    } else if master_key.user_id() != user_id || self_signing.user_id() != user_id {
                         warn!(
-                            user_id = user_id.as_str(),
+                            user = user_id.as_str(),
                             "User ID mismatch in one of the cross signing keys",
                         );
                         continue;
-                    }
-
-                    ReadOnlyOwnUserIdentity::new(master_key, self_signing, user_signing)
-                        .map(|i| (ReadOnlyUserIdentities::Own(i), true))
-                } else {
-                    warn!(
-                        user_id = user_id.as_str(),
-                        "User identity for our own user didn't contain a \
-                        user signing pubkey",
-                    );
-                    continue;
-                }
-            } else if master_key.user_id() != user_id || self_signing.user_id() != user_id {
-                warn!(user = user_id.as_str(), "User ID mismatch in one of the cross signing keys",);
-                continue;
-            } else {
-                ReadOnlyUserIdentity::new(master_key, self_signing)
-                    .map(|i| (ReadOnlyUserIdentities::Other(i), true))
-            };
-
-            match result {
-                Ok((i, new)) => {
-                    if let Some(identity) = i.own() {
-                        let private_identity = self.store.private_identity();
-                        let private_identity = private_identity.lock().await;
-
-                        let result = private_identity.clear_if_differs(identity).await;
-
-                        if result.any_cleared() {
-                            changed_identity = Some((&*private_identity).clone());
-                            info!(cleared =? result, "Removed some or all of our private cross signing keys");
-                        }
-                    }
-
-                    if new {
-                        trace!(user_id = user_id.as_str(), identity =? i, "Created new user identity");
-                        changes.new.push(i);
                     } else {
-                        trace!(user_id = user_id.as_str(), identity =? i, "Updated a user identity");
-                        changes.changed.push(i);
+                        ReadOnlyUserIdentity::new(master_key, self_signing)
+                            .map(|i| (ReadOnlyUserIdentities::Other(i), true))
+                    };
+
+                    match result {
+                        Ok((i, new)) => {
+                            if let Some(identity) = i.own() {
+                                let private_identity = self.store.private_identity();
+                                let private_identity = private_identity.lock().await;
+
+                                let result = private_identity.clear_if_differs(identity).await;
+
+                                if result.any_cleared() {
+                                    changed_identity = Some((*private_identity).clone());
+                                    info!(cleared = ?result, "Removed some or all of our private cross signing keys");
+                                }
+                            }
+
+                            if new {
+                                trace!(user_id = user_id.as_str(), identity = ?i, "Created new user identity");
+                                changes.new.push(i);
+                            } else {
+                                trace!(user_id = user_id.as_str(), identity = ?i, "Updated a user identity");
+                                changes.changed.push(i);
+                            }
+                        }
+                        Err(e) => {
+                            warn!(
+                                user_id = user_id.as_str(),
+                                error = ?e,
+                                "Couldn't update or create new user identity"
+                            );
+                            continue;
+                        }
                     }
                 }
                 Err(e) => {
                     warn!(
                         user_id = user_id.as_str(),
-                        error =? e,
-                        "Couldn't update or create new user identity for"
+                        error = ?e,
+                        "Couldn't update or create new user identity"
                     );
                     continue;
                 }
@@ -357,10 +476,22 @@ impl IdentityManager {
     pub async fn users_for_key_query(&self) -> Vec<KeysQueryRequest> {
         let users = self.store.users_for_key_query();
 
+        // We always want to track our own user, but in case we aren't in an encrypted
+        // room yet, we won't be tracking ourselves yet. This ensures we are always
+        // tracking ourselves.
+        //
+        // The check for emptiness is done first for performance.
+        let users = if users.is_empty() && !self.store.tracked_users().contains(self.user_id()) {
+            self.update_tracked_users([self.user_id()]).await;
+            self.store.users_for_key_query()
+        } else {
+            users
+        };
+
         if users.is_empty() {
             Vec::new()
         } else {
-            let users: Vec<UserId> = users.into_iter().collect();
+            let users: Vec<OwnedUserId> = users.into_iter().collect();
 
             users
                 .chunks(Self::MAX_KEY_QUERY_USERS)
@@ -412,42 +543,42 @@ impl IdentityManager {
     }
 }
 
-#[cfg(test)]
-pub(crate) mod test {
+#[cfg(any(test, feature = "testing"))]
+pub(crate) mod testing {
+    #![allow(dead_code)]
     use std::sync::Arc;
 
     use matrix_sdk_common::locks::Mutex;
-    use matrix_sdk_test::async_test;
     use ruma::{
-        api::{client::r0::keys::get_keys::Response as KeyQueryResponse, IncomingResponse},
-        user_id, DeviceIdBox, UserId,
+        api::{client::keys::get_keys::v3::Response as KeyQueryResponse, IncomingResponse},
+        device_id, user_id, DeviceId, UserId,
     };
     use serde_json::json;
 
     use crate::{
         identities::IdentityManager,
-        machine::test::response_from_file,
+        machine::testing::response_from_file,
         olm::{PrivateCrossSigningIdentity, ReadOnlyAccount},
         store::{CryptoStore, MemoryStore, Store},
         verification::VerificationMachine,
     };
 
-    fn user_id() -> UserId {
+    pub fn user_id() -> &'static UserId {
         user_id!("@example:localhost")
     }
 
-    fn other_user_id() -> UserId {
+    pub fn other_user_id() -> &'static UserId {
         user_id!("@example2:localhost")
     }
 
-    fn device_id() -> DeviceIdBox {
-        "WSKKLTJZCL".into()
+    pub fn device_id() -> &'static DeviceId {
+        device_id!("WSKKLTJZCL")
     }
 
-    fn manager() -> IdentityManager {
+    pub(crate) fn manager() -> IdentityManager {
         let identity = Arc::new(Mutex::new(PrivateCrossSigningIdentity::empty(user_id())));
-        let user_id = Arc::new(user_id());
-        let account = ReadOnlyAccount::new(&user_id, &device_id());
+        let user_id = Arc::from(user_id());
+        let account = ReadOnlyAccount::new(&user_id, device_id());
         let store: Arc<dyn CryptoStore> = Arc::new(MemoryStore::new());
         let verification = VerificationMachine::new(account, identity.clone(), store);
         let store =
@@ -455,7 +586,7 @@ pub(crate) mod test {
         IdentityManager::new(user_id, device_id().into(), store)
     }
 
-    pub(crate) fn other_key_query() -> KeyQueryResponse {
+    pub fn other_key_query() -> KeyQueryResponse {
         let data = response_from_file(&json!({
             "device_keys": {
                 "@example2:localhost": {
@@ -514,7 +645,7 @@ pub(crate) mod test {
             .expect("Can't parse the keys upload response")
     }
 
-    pub(crate) fn own_key_query() -> KeyQueryResponse {
+    pub fn own_key_query() -> KeyQueryResponse {
         let data = response_from_file(&json!({
           "device_keys": {
             "@example:localhost": {
@@ -614,32 +745,40 @@ pub(crate) mod test {
         KeyQueryResponse::try_from_http_response(data)
             .expect("Can't parse the keys upload response")
     }
+}
+
+#[cfg(test)]
+pub(crate) mod tests {
+    use matrix_sdk_test::async_test;
+    use ruma::device_id;
+
+    use super::testing::{manager, other_key_query, other_user_id};
 
     #[async_test]
     async fn test_manager_creation() {
         let manager = manager();
-        assert!(manager.users_for_key_query().await.is_empty())
+        assert!(manager.store.tracked_users().is_empty())
     }
 
     #[async_test]
     async fn test_manager_key_query_response() {
         let manager = manager();
         let other_user = other_user_id();
-        let devices = manager.store.get_user_devices(&other_user).await.unwrap();
+        let devices = manager.store.get_user_devices(other_user).await.unwrap();
         assert_eq!(devices.devices().count(), 0);
 
         manager.receive_keys_query_response(&other_key_query()).await.unwrap();
 
-        let devices = manager.store.get_user_devices(&other_user).await.unwrap();
+        let devices = manager.store.get_user_devices(other_user).await.unwrap();
         assert_eq!(devices.devices().count(), 1);
 
         let device = manager
             .store
-            .get_readonly_device(&other_user, "SKISMLNIMH".into())
+            .get_readonly_device(other_user, device_id!("SKISMLNIMH"))
             .await
             .unwrap()
             .unwrap();
-        let identity = manager.store.get_user_identity(&other_user).await.unwrap().unwrap();
+        let identity = manager.store.get_user_identity(other_user).await.unwrap().unwrap();
         let identity = identity.other().unwrap();
 
         assert!(identity.is_device_signed(&device).is_ok())
@@ -649,23 +788,38 @@ pub(crate) mod test {
     async fn test_manager_own_key_query_response() {
         let manager = manager();
         let other_user = other_user_id();
-        let devices = manager.store.get_user_devices(&other_user).await.unwrap();
+        let devices = manager.store.get_user_devices(other_user).await.unwrap();
         assert_eq!(devices.devices().count(), 0);
 
         manager.receive_keys_query_response(&other_key_query()).await.unwrap();
 
-        let devices = manager.store.get_user_devices(&other_user).await.unwrap();
+        let devices = manager.store.get_user_devices(other_user).await.unwrap();
         assert_eq!(devices.devices().count(), 1);
 
         let device = manager
             .store
-            .get_readonly_device(&other_user, "SKISMLNIMH".into())
+            .get_readonly_device(other_user, device_id!("SKISMLNIMH"))
             .await
             .unwrap()
             .unwrap();
-        let identity = manager.store.get_user_identity(&other_user).await.unwrap().unwrap();
+        let identity = manager.store.get_user_identity(other_user).await.unwrap().unwrap();
         let identity = identity.other().unwrap();
 
         assert!(identity.is_device_signed(&device).is_ok())
     }
+
+    #[async_test]
+    async fn no_tracked_users_key_query_request() {
+        let manager = manager();
+
+        assert!(manager.store.tracked_users().is_empty(), "No users are initially tracked");
+
+        let requests = manager.users_for_key_query().await;
+
+        assert!(!requests.is_empty(), "We query the keys for our own user");
+        assert!(
+            manager.store.tracked_users().contains(manager.user_id()),
+            "Our own user is now tracked"
+        );
+    }
 }
