@@ -23,22 +23,24 @@ use std::{
 };
 
 use ruma::{
-    api::client::r0::keys::upload_signatures::Request as SignatureUploadRequest,
-    encryption::{CrossSigningKey, DeviceKeys, KeyUsage},
+    api::client::keys::upload_signatures::v3::Request as SignatureUploadRequest,
+    encryption::KeyUsage,
     events::{
         key::verification::VerificationMethod, room::message::KeyVerificationRequestEventContent,
     },
-    DeviceIdBox, DeviceKeyId, EventId, RoomId, UserId,
+    DeviceKeyId, EventId, OwnedDeviceId, OwnedDeviceKeyId, OwnedUserId, RoomId, UserId,
 };
 use serde::{Deserialize, Serialize};
 use serde_json::to_value;
 use tracing::error;
+use vodozemac::Ed25519PublicKey;
 
 use super::{atomic_bool_deserializer, atomic_bool_serializer};
 use crate::{
     error::SignatureError,
-    olm::Utility,
+    olm::VerifyJson,
     store::{Changes, IdentityChanges},
+    types::{CrossSigningKey, DeviceKeys, SigningKey},
     verification::VerificationMachine,
     CryptoStoreError, OutgoingVerificationRequest, ReadOnlyDevice, VerificationRequest,
 };
@@ -122,7 +124,7 @@ impl OwnUserIdentity {
         };
 
         if let Err(e) = self.verification_machine.store.save_changes(changes).await {
-            error!(error =? e, "Couldn't store our own user identity after marking it as verified");
+            error!(error = ?e, "Couldn't store our own user identity after marking it as verified");
         }
 
         self.verification_machine.store.account.sign_master_key(self.master_key.clone()).await
@@ -165,7 +167,7 @@ impl OwnUserIdentity {
         &self,
         methods: Option<Vec<VerificationMethod>>,
     ) -> Result<(VerificationRequest, OutgoingVerificationRequest), CryptoStoreError> {
-        let devices: Vec<DeviceIdBox> = self
+        let devices: Vec<OwnedDeviceId> = self
             .verification_machine
             .store
             .get_user_devices(self.user_id())
@@ -225,16 +227,17 @@ impl UserIdentity {
     /// Returns a request that needs to be sent out for the user to be marked
     /// as verified.
     pub async fn verify(&self) -> Result<SignatureUploadRequest, SignatureError> {
-        if self.user_id() == self.verification_machine.own_user_id() {
+        if self.user_id() != self.verification_machine.own_user_id() {
             Ok(self
                 .verification_machine
+                .store
                 .private_identity
                 .lock()
                 .await
                 .sign_user(&self.inner)
                 .await?)
         } else {
-            Err(SignatureError::UserIdMissmatch)
+            Err(SignatureError::UserIdMismatch)
         }
     }
 
@@ -312,40 +315,19 @@ impl PartialEq for UserSigningPubkey {
 
 impl From<CrossSigningKey> for MasterPubkey {
     fn from(key: CrossSigningKey) -> Self {
-        Self(Arc::new(key))
+        Self(key.into())
     }
 }
 
 impl From<CrossSigningKey> for SelfSigningPubkey {
     fn from(key: CrossSigningKey) -> Self {
-        Self(Arc::new(key))
+        Self(key.into())
     }
 }
 
 impl From<CrossSigningKey> for UserSigningPubkey {
     fn from(key: CrossSigningKey) -> Self {
-        Self(Arc::new(key))
-    }
-}
-
-#[allow(clippy::from_over_into)]
-impl Into<CrossSigningKey> for MasterPubkey {
-    fn into(self) -> CrossSigningKey {
-        self.0.as_ref().clone()
-    }
-}
-
-#[allow(clippy::from_over_into)]
-impl Into<CrossSigningKey> for UserSigningPubkey {
-    fn into(self) -> CrossSigningKey {
-        self.0.as_ref().clone()
-    }
-}
-
-#[allow(clippy::from_over_into)]
-impl Into<CrossSigningKey> for SelfSigningPubkey {
-    fn into(self) -> CrossSigningKey {
-        self.0.as_ref().clone()
+        Self(key.into())
     }
 }
 
@@ -367,24 +349,6 @@ impl AsRef<CrossSigningKey> for UserSigningPubkey {
     }
 }
 
-impl From<&CrossSigningKey> for MasterPubkey {
-    fn from(key: &CrossSigningKey) -> Self {
-        Self(Arc::new(key.clone()))
-    }
-}
-
-impl From<&CrossSigningKey> for SelfSigningPubkey {
-    fn from(key: &CrossSigningKey) -> Self {
-        Self(Arc::new(key.clone()))
-    }
-}
-
-impl From<&CrossSigningKey> for UserSigningPubkey {
-    fn from(key: &CrossSigningKey) -> Self {
-        Self(Arc::new(key.clone()))
-    }
-}
-
 impl<'a> From<&'a SelfSigningPubkey> for CrossSigningSubKeys<'a> {
     fn from(key: &'a SelfSigningPubkey) -> Self {
         CrossSigningSubKeys::SelfSigning(key)
@@ -430,7 +394,7 @@ impl MasterPubkey {
     }
 
     /// Get the keys map of containing the master keys.
-    pub fn keys(&self) -> &BTreeMap<String, String> {
+    pub fn keys(&self) -> &BTreeMap<OwnedDeviceKeyId, SigningKey> {
         &self.0.keys
     }
 
@@ -440,7 +404,7 @@ impl MasterPubkey {
     }
 
     /// Get the signatures map of this cross signing key.
-    pub fn signatures(&self) -> &BTreeMap<UserId, BTreeMap<String, String>> {
+    pub fn signatures(&self) -> &BTreeMap<OwnedUserId, BTreeMap<OwnedDeviceKeyId, String>> {
         &self.0.signatures
     }
 
@@ -449,16 +413,20 @@ impl MasterPubkey {
     /// # Arguments
     ///
     /// * `key_id` - The id of the key that should be fetched.
-    pub fn get_key(&self, key_id: &DeviceKeyId) -> Option<&str> {
-        self.0.keys.get(key_id.as_str()).map(|k| k.as_str())
+    pub fn get_key(&self, key_id: &DeviceKeyId) -> Option<&SigningKey> {
+        self.0.keys.get(key_id)
     }
 
     /// Get the first available master key.
     ///
     /// There's usually only a single master key so this will usually fetch the
     /// only key.
-    pub fn get_first_key(&self) -> Option<&str> {
-        self.0.keys.values().map(|k| k.as_str()).next()
+    pub fn get_first_key(&self) -> Option<Ed25519PublicKey> {
+        if let Some(SigningKey::Ed25519(k)) = self.0.keys.values().next() {
+            Some(*k)
+        } else {
+            None
+        }
     }
 
     /// Check if the given cross signing sub-key is signed by the master key.
@@ -475,31 +443,34 @@ impl MasterPubkey {
     ) -> Result<(), SignatureError> {
         let (key_id, key) = self.0.keys.iter().next().ok_or(SignatureError::MissingSigningKey)?;
 
-        let key_id = DeviceKeyId::try_from(key_id.as_str())?;
+        let key_id = <&DeviceKeyId>::try_from(key_id.as_str())?;
 
         // FIXME `KeyUsage is missing PartialEq.
         // if self.0.usage.contains(&KeyUsage::Master) {
         //     return Err(SignatureError::MissingSigningKey);
         // }
-        let subkey: CrossSigningSubKeys = subkey.into();
+        let subkey: CrossSigningSubKeys<'_> = subkey.into();
 
-        if &self.0.user_id != subkey.user_id() {
-            return Err(SignatureError::UserIdMissmatch);
+        if self.0.user_id != subkey.user_id() {
+            return Err(SignatureError::UserIdMismatch);
         }
 
-        let utility = Utility::new();
-        utility.verify_json(
-            &self.0.user_id,
-            &key_id,
-            key,
-            &mut to_value(subkey.cross_signing_key()).map_err(|_| SignatureError::NotAnObject)?,
-        )
+        if let SigningKey::Ed25519(key) = key {
+            key.verify_json(
+                &self.0.user_id,
+                key_id,
+                &mut to_value(subkey.cross_signing_key())
+                    .map_err(|_| SignatureError::NotAnObject)?,
+            )
+        } else {
+            Err(SignatureError::UnsupportedAlgorithm)
+        }
     }
 }
 
 impl<'a> IntoIterator for &'a MasterPubkey {
-    type Item = (&'a String, &'a String);
-    type IntoIter = Iter<'a, String, String>;
+    type Item = (&'a OwnedDeviceKeyId, &'a SigningKey);
+    type IntoIter = Iter<'a, OwnedDeviceKeyId, SigningKey>;
 
     fn into_iter(self) -> Self::IntoIter {
         self.keys().iter()
@@ -513,7 +484,7 @@ impl UserSigningPubkey {
     }
 
     /// Get the keys map of containing the user signing keys.
-    pub fn keys(&self) -> &BTreeMap<String, String> {
+    pub fn keys(&self) -> &BTreeMap<OwnedDeviceKeyId, SigningKey> {
         &self.0.keys
     }
 
@@ -534,19 +505,21 @@ impl UserSigningPubkey {
 
         // TODO check that the usage is OK.
 
-        let utility = Utility::new();
-        utility.verify_json(
-            &self.0.user_id,
-            &DeviceKeyId::try_from(key_id.as_str())?,
-            key,
-            &mut to_value(&*master_key.0).map_err(|_| SignatureError::NotAnObject)?,
-        )
+        if let SigningKey::Ed25519(key) = key {
+            key.verify_json(
+                &self.0.user_id,
+                key_id.as_str().try_into()?,
+                &mut to_value(&master_key.0).map_err(|_| SignatureError::NotAnObject)?,
+            )
+        } else {
+            Err(SignatureError::UnsupportedAlgorithm)
+        }
     }
 }
 
 impl<'a> IntoIterator for &'a UserSigningPubkey {
-    type Item = (&'a String, &'a String);
-    type IntoIter = Iter<'a, String, String>;
+    type Item = (&'a OwnedDeviceKeyId, &'a SigningKey);
+    type IntoIter = Iter<'a, OwnedDeviceKeyId, SigningKey>;
 
     fn into_iter(self) -> Self::IntoIter {
         self.keys().iter()
@@ -560,7 +533,7 @@ impl SelfSigningPubkey {
     }
 
     /// Get the keys map of containing the self signing keys.
-    pub fn keys(&self) -> &BTreeMap<String, String> {
+    pub fn keys(&self) -> &BTreeMap<OwnedDeviceKeyId, SigningKey> {
         &self.0.keys
     }
 
@@ -570,14 +543,11 @@ impl SelfSigningPubkey {
 
         let mut device = to_value(device_keys)?;
 
-        let utility = Utility::new();
-
-        utility.verify_json(
-            &self.0.user_id,
-            &DeviceKeyId::try_from(key_id.as_str())?,
-            key,
-            &mut device,
-        )
+        if let SigningKey::Ed25519(key) = key {
+            key.verify_json(&self.0.user_id, key_id.as_str().try_into()?, &mut device)
+        } else {
+            Err(SignatureError::UnsupportedAlgorithm)
+        }
     }
 
     /// Check if the given device is signed by this self signing key.
@@ -589,13 +559,13 @@ impl SelfSigningPubkey {
     /// Returns an empty result if the signature check succeeded, otherwise a
     /// SignatureError indicating why the check failed.
     pub(crate) fn verify_device(&self, device: &ReadOnlyDevice) -> Result<(), SignatureError> {
-        self.verify_device_keys(device.as_device_keys())
+        self.verify_device_keys(device.as_device_keys().to_owned())
     }
 }
 
 impl<'a> IntoIterator for &'a SelfSigningPubkey {
-    type Item = (&'a String, &'a String);
-    type IntoIter = Iter<'a, String, String>;
+    type Item = (&'a OwnedDeviceKeyId, &'a SigningKey);
+    type IntoIter = Iter<'a, OwnedDeviceKeyId, SigningKey>;
 
     fn into_iter(self) -> Self::IntoIter {
         self.keys().iter()
@@ -718,7 +688,7 @@ impl ReadOnlyUserIdentity {
     ) -> Result<Self, SignatureError> {
         master_key.verify_subkey(&self_signing_key)?;
 
-        Ok(Self { user_id: Arc::new(master_key.0.user_id.clone()), master_key, self_signing_key })
+        Ok(Self { user_id: (&*master_key.0.user_id).into(), master_key, self_signing_key })
     }
 
     #[cfg(test)]
@@ -727,7 +697,7 @@ impl ReadOnlyUserIdentity {
         let self_signing_key =
             identity.self_signing_key.lock().await.as_ref().unwrap().public_key.clone();
 
-        Self { user_id: Arc::new(identity.user_id().clone()), master_key, self_signing_key }
+        Self { user_id: identity.user_id().into(), master_key, self_signing_key }
     }
 
     /// Get the user id of this identity.
@@ -781,7 +751,7 @@ impl ReadOnlyUserIdentity {
     /// SignatureError indicating why the check failed.
     pub(crate) fn is_device_signed(&self, device: &ReadOnlyDevice) -> Result<(), SignatureError> {
         if self.user_id() != device.user_id() {
-            return Err(SignatureError::UserIdMissmatch);
+            return Err(SignatureError::UserIdMismatch);
         }
 
         self.self_signing_key.verify_device(device)
@@ -831,7 +801,7 @@ impl ReadOnlyOwnUserIdentity {
         master_key.verify_subkey(&user_signing_key)?;
 
         Ok(Self {
-            user_id: Arc::new(master_key.0.user_id.clone()),
+            user_id: (&*master_key.0.user_id).into(),
             master_key,
             self_signing_key,
             user_signing_key,
@@ -889,7 +859,7 @@ impl ReadOnlyOwnUserIdentity {
     /// SignatureError indicating why the check failed.
     pub(crate) fn is_device_signed(&self, device: &ReadOnlyDevice) -> Result<(), SignatureError> {
         if self.user_id() != device.user_id() {
-            return Err(SignatureError::UserIdMissmatch);
+            return Err(SignatureError::UserIdMismatch);
         }
 
         self.self_signing_key.verify_device(device)
@@ -931,7 +901,7 @@ impl ReadOnlyOwnUserIdentity {
         self.user_signing_key = user_signing_key;
 
         if self.master_key != master_key {
-            self.verified.store(false, Ordering::SeqCst)
+            self.verified.store(false, Ordering::SeqCst);
         }
 
         self.master_key = master_key;
@@ -940,65 +910,96 @@ impl ReadOnlyOwnUserIdentity {
     }
 }
 
-#[cfg(test)]
-pub(crate) mod test {
-    use std::{convert::TryFrom, sync::Arc};
+#[cfg(any(test, feature = "testing"))]
+pub(crate) mod testing {
+    //! Testing Facilities
+    #![allow(dead_code)]
+    use ruma::{api::client::keys::get_keys::v3::Response as KeyQueryResponse, user_id};
 
-    use matrix_sdk_common::locks::Mutex;
-    use matrix_sdk_test::async_test;
-    use ruma::{api::client::r0::keys::get_keys::Response as KeyQueryResponse, user_id};
-
-    use super::{ReadOnlyOwnUserIdentity, ReadOnlyUserIdentities, ReadOnlyUserIdentity};
+    use super::{ReadOnlyOwnUserIdentity, ReadOnlyUserIdentity};
     use crate::{
         identities::{
-            manager::test::{other_key_query, own_key_query},
-            Device, ReadOnlyDevice,
+            manager::testing::{other_key_query, own_key_query},
+            ReadOnlyDevice,
         },
-        olm::{PrivateCrossSigningIdentity, ReadOnlyAccount},
-        store::MemoryStore,
-        verification::VerificationMachine,
+        types::CrossSigningKey,
     };
 
-    fn device(response: &KeyQueryResponse) -> (ReadOnlyDevice, ReadOnlyDevice) {
+    /// Generate test devices from KeyQueryResponse
+    pub fn device(response: &KeyQueryResponse) -> (ReadOnlyDevice, ReadOnlyDevice) {
         let mut devices = response.device_keys.values().next().unwrap().values();
-        let first = ReadOnlyDevice::try_from(devices.next().unwrap()).unwrap();
-        let second = ReadOnlyDevice::try_from(devices.next().unwrap()).unwrap();
+        let first =
+            ReadOnlyDevice::try_from(&devices.next().unwrap().deserialize_as().unwrap()).unwrap();
+        let second =
+            ReadOnlyDevice::try_from(&devices.next().unwrap().deserialize_as().unwrap()).unwrap();
         (first, second)
     }
 
-    fn own_identity(response: &KeyQueryResponse) -> ReadOnlyOwnUserIdentity {
+    /// Generate ReadOnlyOwnUserIdentity from KeyQueryResponse for testing
+    pub fn own_identity(response: &KeyQueryResponse) -> ReadOnlyOwnUserIdentity {
         let user_id = user_id!("@example:localhost");
 
-        let master_key = response.master_keys.get(&user_id).unwrap();
-        let user_signing = response.user_signing_keys.get(&user_id).unwrap();
-        let self_signing = response.self_signing_keys.get(&user_id).unwrap();
+        let master_key: CrossSigningKey =
+            response.master_keys.get(user_id).unwrap().deserialize_as().unwrap();
+        let user_signing: CrossSigningKey =
+            response.user_signing_keys.get(user_id).unwrap().deserialize_as().unwrap();
+        let self_signing: CrossSigningKey =
+            response.self_signing_keys.get(user_id).unwrap().deserialize_as().unwrap();
 
         ReadOnlyOwnUserIdentity::new(master_key.into(), self_signing.into(), user_signing.into())
             .unwrap()
     }
 
-    pub(crate) fn get_own_identity() -> ReadOnlyOwnUserIdentity {
+    /// Generate default own identity for tests
+    pub fn get_own_identity() -> ReadOnlyOwnUserIdentity {
         own_identity(&own_key_query())
     }
 
-    pub(crate) fn get_other_identity() -> ReadOnlyUserIdentity {
+    /// Generate default other identify for tests
+    pub fn get_other_identity() -> ReadOnlyUserIdentity {
         let user_id = user_id!("@example2:localhost");
         let response = other_key_query();
 
-        let master_key = response.master_keys.get(&user_id).unwrap();
-        let self_signing = response.self_signing_keys.get(&user_id).unwrap();
+        let master_key: CrossSigningKey =
+            response.master_keys.get(user_id).unwrap().deserialize_as().unwrap();
+        let self_signing: CrossSigningKey =
+            response.self_signing_keys.get(user_id).unwrap().deserialize_as().unwrap();
 
         ReadOnlyUserIdentity::new(master_key.into(), self_signing.into()).unwrap()
     }
+}
+
+#[cfg(test)]
+pub(crate) mod tests {
+    use std::sync::Arc;
+
+    use matrix_sdk_common::locks::Mutex;
+    use matrix_sdk_test::async_test;
+    use ruma::user_id;
+
+    use super::{
+        testing::{device, get_other_identity, get_own_identity},
+        ReadOnlyOwnUserIdentity, ReadOnlyUserIdentities,
+    };
+    use crate::{
+        identities::{manager::testing::own_key_query, Device},
+        olm::{PrivateCrossSigningIdentity, ReadOnlyAccount},
+        store::MemoryStore,
+        types::CrossSigningKey,
+        verification::VerificationMachine,
+    };
 
     #[test]
     fn own_identity_create() {
         let user_id = user_id!("@example:localhost");
         let response = own_key_query();
 
-        let master_key = response.master_keys.get(&user_id).unwrap();
-        let user_signing = response.user_signing_keys.get(&user_id).unwrap();
-        let self_signing = response.self_signing_keys.get(&user_id).unwrap();
+        let master_key: CrossSigningKey =
+            response.master_keys.get(user_id).unwrap().deserialize_as().unwrap();
+        let user_signing: CrossSigningKey =
+            response.user_signing_keys.get(user_id).unwrap().deserialize_as().unwrap();
+        let self_signing: CrossSigningKey =
+            response.self_signing_keys.get(user_id).unwrap().deserialize_as().unwrap();
 
         ReadOnlyOwnUserIdentity::new(master_key.into(), self_signing.into(), user_signing.into())
             .unwrap();
@@ -1019,7 +1020,7 @@ pub(crate) mod test {
         assert!(identity.is_device_signed(&second).is_ok());
 
         let private_identity =
-            Arc::new(Mutex::new(PrivateCrossSigningIdentity::empty(second.user_id().clone())));
+            Arc::new(Mutex::new(PrivateCrossSigningIdentity::empty(second.user_id())));
         let verification_machine = VerificationMachine::new(
             ReadOnlyAccount::new(second.user_id(), second.device_id()),
             private_identity,
@@ -1057,7 +1058,7 @@ pub(crate) mod test {
         let (_, device) = device(&response);
 
         let account = ReadOnlyAccount::new(device.user_id(), device.device_id());
-        let (identity, _, _) = PrivateCrossSigningIdentity::new_with_account(&account).await;
+        let (identity, _, _) = PrivateCrossSigningIdentity::with_account(&account).await;
 
         let id = Arc::new(Mutex::new(identity.clone()));
 
@@ -1078,10 +1079,10 @@ pub(crate) mod test {
 
         assert!(!device.verified());
 
-        let mut device_keys = device.as_device_keys();
+        let mut device_keys = device.as_device_keys().to_owned();
 
         identity.sign_device_keys(&mut device_keys).await.unwrap();
-        device.inner.signatures = Arc::new(device_keys.signatures);
+        device.inner.update_device(&device_keys).expect("Couldn't update newly signed device keys");
         assert!(device.verified());
     }
 }
