@@ -12,34 +12,42 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::{collections::BTreeMap, convert::TryFrom, fmt, mem, sync::Arc};
+#![warn(missing_docs)]
+
+use std::{
+    collections::BTreeMap,
+    fmt,
+    sync::{
+        atomic::{AtomicBool, Ordering::SeqCst},
+        Arc,
+    },
+};
 
 use matrix_sdk_common::locks::Mutex;
-pub use olm_rs::{
-    account::IdentityKeys,
-    session::{OlmMessage, PreKeyMessage},
-    utility::OlmUtility,
-};
-use olm_rs::{
-    errors::OlmGroupSessionError, inbound_group_session::OlmInboundGroupSession, PicklingMode,
-};
 use ruma::{
     events::{
-        forwarded_room_key::ForwardedRoomKeyToDeviceEventContent,
+        forwarded_room_key::ToDeviceForwardedRoomKeyEventContent,
         room::{
-            encrypted::{EncryptedEventContent, EncryptedEventScheme},
+            encrypted::{EncryptedEventScheme, OriginalSyncRoomEncryptedEvent},
             history_visibility::HistoryVisibility,
         },
-        AnySyncRoomEvent, SyncMessageEvent,
+        AnyRoomEvent,
     },
     serde::Raw,
-    DeviceKeyAlgorithm, EventEncryptionAlgorithm, RoomId,
+    DeviceKeyAlgorithm, EventEncryptionAlgorithm, OwnedRoomId, RoomId,
 };
 use serde::{Deserialize, Serialize};
 use serde_json::Value;
-use zeroize::Zeroizing;
+use vodozemac::{
+    megolm::{
+        DecryptedMessage, DecryptionError, ExportedSessionKey, InboundGroupSession as InnerSession,
+        InboundGroupSessionPickle, MegolmMessage, SessionKeyDecodeError,
+    },
+    PickleError,
+};
+use zeroize::Zeroize;
 
-use super::{ExportedGroupSessionKey, ExportedRoomKey, GroupSessionKey};
+use super::{BackedUpRoomKey, ExportedRoomKey, SessionKey};
 use crate::error::{EventError, MegolmResult};
 
 // TODO add creation times to the inbound group sessions so we can export
@@ -52,16 +60,20 @@ use crate::error::{EventError, MegolmResult};
 /// participants. Inbound group sessions are used to decrypt the room messages.
 #[derive(Clone)]
 pub struct InboundGroupSession {
-    inner: Arc<Mutex<OlmInboundGroupSession>>,
+    inner: Arc<Mutex<InnerSession>>,
     history_visibility: Arc<Option<HistoryVisibility>>,
-    session_id: Arc<str>,
+    /// The SessionId associated to this GroupSession
+    pub session_id: Arc<str>,
     first_known_index: u32,
-    pub(crate) sender_key: Arc<str>,
-    pub(crate) signing_keys: Arc<BTreeMap<DeviceKeyAlgorithm, String>>,
-    pub(crate) room_id: Arc<RoomId>,
+    /// The sender_key associated to this GroupSession
+    pub sender_key: Arc<str>,
+    /// Map of DeviceKeyAlgorithm to the public ed25519 key of the account
+    pub signing_keys: Arc<BTreeMap<DeviceKeyAlgorithm, String>>,
+    /// The Room this GroupSession belongs to
+    pub room_id: Arc<RoomId>,
     forwarding_chains: Arc<Vec<String>>,
     imported: bool,
-    backed_up: bool,
+    backed_up: Arc<AtomicBool>,
 }
 
 impl InboundGroupSession {
@@ -81,32 +93,32 @@ impl InboundGroupSession {
     ///
     /// * `session_key` - The private session key that is used to decrypt
     /// messages.
-    pub(crate) fn new(
+    pub fn new(
         sender_key: &str,
         signing_key: &str,
         room_id: &RoomId,
-        session_key: GroupSessionKey,
+        session_key: SessionKey,
         history_visibility: Option<HistoryVisibility>,
-    ) -> Result<Self, OlmGroupSessionError> {
-        let session = OlmInboundGroupSession::new(&session_key.0)?;
+    ) -> Self {
+        let session = InnerSession::new(&session_key);
         let session_id = session.session_id();
         let first_known_index = session.first_known_index();
 
         let mut keys: BTreeMap<DeviceKeyAlgorithm, String> = BTreeMap::new();
         keys.insert(DeviceKeyAlgorithm::Ed25519, signing_key.to_owned());
 
-        Ok(InboundGroupSession {
+        InboundGroupSession {
             inner: Arc::new(Mutex::new(session)),
             session_id: session_id.into(),
             history_visibility: history_visibility.into(),
             sender_key: sender_key.to_owned().into(),
             first_known_index,
             signing_keys: keys.into(),
-            room_id: room_id.clone().into(),
+            room_id: room_id.into(),
             forwarding_chains: Vec::new().into(),
             imported: false,
-            backed_up: false,
-        })
+            backed_up: AtomicBool::new(false).into(),
+        }
     }
 
     /// Create a InboundGroupSession from an exported version of the group
@@ -115,12 +127,25 @@ impl InboundGroupSession {
     /// Most notably this can be called with an `ExportedRoomKey` from a
     /// previous [`export()`] call.
     ///
-    ///
     /// [`export()`]: #method.export
-    pub fn from_export(
-        exported_session: impl Into<ExportedRoomKey>,
-    ) -> Result<Self, OlmGroupSessionError> {
-        Self::try_from(exported_session.into())
+    pub fn from_export(exported_session: ExportedRoomKey) -> Self {
+        Self::from(exported_session)
+    }
+
+    #[allow(dead_code)]
+    fn from_backup(room_id: &RoomId, backup: BackedUpRoomKey) -> Self {
+        let session = InnerSession::import(&backup.session_key);
+        let session_id = session.session_id();
+
+        Self::from_export(ExportedRoomKey {
+            algorithm: backup.algorithm,
+            room_id: room_id.to_owned(),
+            sender_key: backup.sender_key,
+            session_id,
+            session_key: backup.session_key,
+            sender_claimed_keys: backup.sender_claimed_keys,
+            forwarding_curve25519_key_chain: backup.forwarding_curve25519_key_chain,
+        })
     }
 
     /// Create a new inbound group session from a forwarded room key content.
@@ -132,13 +157,14 @@ impl InboundGroupSession {
     ///
     /// * `content` - A forwarded room key content that contains the session key
     /// to create the `InboundGroupSession`.
-    pub(crate) fn from_forwarded_key(
+    pub fn from_forwarded_key(
         sender_key: &str,
-        content: &mut ForwardedRoomKeyToDeviceEventContent,
-    ) -> Result<Self, OlmGroupSessionError> {
-        let key = Zeroizing::from(mem::take(&mut content.session_key));
+        content: &mut ToDeviceForwardedRoomKeyEventContent,
+    ) -> Result<Self, SessionKeyDecodeError> {
+        let key = ExportedSessionKey::from_base64(&content.session_key)?;
+        content.session_key.zeroize();
 
-        let session = OlmInboundGroupSession::import(&key)?;
+        let session = InnerSession::import(&key);
         let first_known_index = session.first_known_index();
         let mut forwarding_chains = content.forwarding_curve25519_key_chain.clone();
         forwarding_chains.push(sender_key.to_owned());
@@ -154,10 +180,10 @@ impl InboundGroupSession {
             first_known_index,
             history_visibility: None.into(),
             signing_keys: sender_claimed_key.into(),
-            room_id: content.room_id.clone().into(),
+            room_id: (&*content.room_id).into(),
             forwarding_chains: forwarding_chains.into(),
             imported: true,
-            backed_up: false,
+            backed_up: AtomicBool::new(false).into(),
         })
     }
 
@@ -167,17 +193,17 @@ impl InboundGroupSession {
     ///
     /// * `pickle_mode` - The mode that was used to pickle the group session,
     /// either an unencrypted mode or an encrypted using passphrase.
-    pub async fn pickle(&self, pickle_mode: PicklingMode) -> PickledInboundGroupSession {
-        let pickle = self.inner.lock().await.pickle(pickle_mode);
+    pub async fn pickle(&self) -> PickledInboundGroupSession {
+        let pickle = self.inner.lock().await.pickle();
 
         PickledInboundGroupSession {
-            pickle: InboundGroupSessionPickle::from(pickle),
+            pickle,
             sender_key: self.sender_key.to_string(),
-            signing_key: (&*self.signing_keys).clone(),
-            room_id: (&*self.room_id).clone(),
+            signing_key: (*self.signing_keys).clone(),
+            room_id: (*self.room_id).to_owned(),
             forwarding_chains: self.forwarding_key_chain().to_vec(),
             imported: self.imported,
-            backed_up: self.backed_up,
+            backed_up: self.backed_up(),
             history_visibility: self.history_visibility.as_ref().clone(),
         }
     }
@@ -195,6 +221,22 @@ impl InboundGroupSession {
         &self.sender_key
     }
 
+    /// Has the session been backed up to the server.
+    pub fn backed_up(&self) -> bool {
+        self.backed_up.load(SeqCst)
+    }
+
+    /// Reset the backup state of the inbound group session.
+    pub fn reset_backup_state(&self) {
+        self.backed_up.store(false, SeqCst)
+    }
+
+    /// For testing, allow to manually mark this GroupSession to have been
+    /// backed up
+    pub fn mark_as_backed_up(&self) {
+        self.backed_up.store(true, SeqCst)
+    }
+
     /// Get the map of signing keys this session was received from.
     pub fn signing_keys(&self) -> &BTreeMap<DeviceKeyAlgorithm, String> {
         &self.signing_keys
@@ -213,24 +255,23 @@ impl InboundGroupSession {
     pub async fn export_at_index(&self, message_index: u32) -> ExportedRoomKey {
         let message_index = std::cmp::max(self.first_known_index(), message_index);
 
-        let session_key = ExportedGroupSessionKey(
-            self.inner.lock().await.export(message_index).expect("Can't export session"),
-        );
+        let session_key =
+            self.inner.lock().await.export_at(message_index).expect("Can't export session");
 
         ExportedRoomKey {
             algorithm: EventEncryptionAlgorithm::MegolmV1AesSha2,
-            room_id: (&*self.room_id).clone(),
-            sender_key: (&*self.sender_key).to_owned(),
+            room_id: (*self.room_id).to_owned(),
+            sender_key: (*self.sender_key).to_owned(),
             session_id: self.session_id().to_owned(),
             forwarding_curve25519_key_chain: self.forwarding_key_chain().to_vec(),
-            sender_claimed_keys: (&*self.signing_keys).clone(),
+            sender_claimed_keys: (*self.signing_keys).clone(),
             session_key,
         }
     }
 
     /// Restore a Session from a previously pickled string.
     ///
-    /// Returns the restored group session or a `OlmGroupSessionError` if there
+    /// Returns the restored group session or a `UnpicklingError` if there
     /// was an error.
     ///
     /// # Arguments
@@ -239,11 +280,8 @@ impl InboundGroupSession {
     ///
     /// * `pickle_mode` - The mode that was used to pickle the session, either
     /// an unencrypted mode or an encrypted using passphrase.
-    pub fn from_pickle(
-        pickle: PickledInboundGroupSession,
-        pickle_mode: PicklingMode,
-    ) -> Result<Self, OlmGroupSessionError> {
-        let session = OlmInboundGroupSession::unpickle(pickle.pickle.0, pickle_mode)?;
+    pub fn from_pickle(pickle: PickledInboundGroupSession) -> Result<Self, PickleError> {
+        let session: InnerSession = pickle.pickle.into();
         let first_known_index = session.first_known_index();
         let session_id = session.session_id();
 
@@ -254,9 +292,9 @@ impl InboundGroupSession {
             history_visibility: pickle.history_visibility.into(),
             first_known_index,
             signing_keys: pickle.signing_key.into(),
-            room_id: pickle.room_id.into(),
+            room_id: (&*pickle.room_id).into(),
             forwarding_chains: pickle.forwarding_chains.into(),
-            backed_up: pickle.backed_up,
+            backed_up: AtomicBool::from(pickle.backed_up).into(),
             imported: pickle.imported,
         })
     }
@@ -278,7 +316,7 @@ impl InboundGroupSession {
 
     /// Decrypt the given ciphertext.
     ///
-    /// Returns the decrypted plaintext or an `OlmGroupSessionError` if
+    /// Returns the decrypted plaintext or an `DecryptionError` if
     /// decryption failed.
     ///
     /// # Arguments
@@ -286,28 +324,37 @@ impl InboundGroupSession {
     /// * `message` - The message that should be decrypted.
     pub(crate) async fn decrypt_helper(
         &self,
-        message: String,
-    ) -> Result<(String, u32), OlmGroupSessionError> {
+        message: &MegolmMessage,
+    ) -> Result<DecryptedMessage, DecryptionError> {
         self.inner.lock().await.decrypt(message)
     }
 
+    /// Export the inbound group session into a format that can be uploaded to
+    /// the server as a backup.
+    #[cfg(feature = "backups_v1")]
+    pub async fn to_backup(&self) -> BackedUpRoomKey {
+        self.export().await.into()
+    }
+
     /// Decrypt an event from a room timeline.
     ///
     /// # Arguments
     ///
     /// * `event` - The event that should be decrypted.
-    pub(crate) async fn decrypt(
+    pub async fn decrypt(
         &self,
-        event: &SyncMessageEvent<EncryptedEventContent>,
-    ) -> MegolmResult<(Raw<AnySyncRoomEvent>, u32)> {
+        event: &OriginalSyncRoomEncryptedEvent,
+    ) -> MegolmResult<(Raw<AnyRoomEvent>, u32)> {
         let content = match &event.content.scheme {
             EncryptedEventScheme::MegolmV1AesSha2(c) => c,
             _ => return Err(EventError::UnsupportedAlgorithm.into()),
         };
 
-        let (plaintext, message_index) = self.decrypt_helper(content.ciphertext.clone()).await?;
+        let message = MegolmMessage::from_base64(&content.ciphertext)?;
 
-        let mut decrypted_value = serde_json::from_str::<Value>(&plaintext)?;
+        let decrypted = self.decrypt_helper(&message).await?;
+
+        let mut decrypted_value = serde_json::from_str::<Value>(&decrypted.plaintext)?;
         let decrypted_object = decrypted_value.as_object_mut().ok_or(EventError::NotAnObject)?;
 
         let server_ts: i64 = event.origin_server_ts.0.into();
@@ -316,13 +363,23 @@ impl InboundGroupSession {
         decrypted_object.insert("event_id".to_owned(), event.event_id.to_string().into());
         decrypted_object.insert("origin_server_ts".to_owned(), server_ts.into());
 
+        let room_id = decrypted_object
+            .get("room_id")
+            .and_then(|r| r.as_str().and_then(|r| RoomId::parse(r).ok()));
+
+        // Check that we have a room id and that the event wasn't forwarded from
+        // another room.
+        if room_id.as_deref() != Some(self.room_id()) {
+            return Err(EventError::MismatchedRoom(self.room_id().to_owned(), room_id).into());
+        }
+
         decrypted_object.insert(
             "unsigned".to_owned(),
             serde_json::to_value(&event.unsigned).unwrap_or_default(),
         );
 
         if let Some(decrypted_content) =
-            decrypted_object.get_mut("content").map(|c| c.as_object_mut()).flatten()
+            decrypted_object.get_mut("content").and_then(|c| c.as_object_mut())
         {
             if !decrypted_content.contains_key("m.relates_to") {
                 let content = serde_json::to_value(&event.content)?;
@@ -332,7 +389,7 @@ impl InboundGroupSession {
             }
         }
 
-        Ok((serde_json::from_value::<Raw<AnySyncRoomEvent>>(decrypted_value)?, message_index))
+        Ok((serde_json::from_value::<Raw<AnyRoomEvent>>(decrypted_value)?, decrypted.message_index))
     }
 }
 
@@ -353,7 +410,8 @@ impl PartialEq for InboundGroupSession {
 ///
 /// Holds all the information that needs to be stored in a database to restore
 /// an InboundGroupSession.
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Serialize, Deserialize)]
+#[allow(missing_debug_implementations)]
 pub struct PickledInboundGroupSession {
     /// The pickle string holding the InboundGroupSession.
     pub pickle: InboundGroupSessionPickle,
@@ -362,7 +420,7 @@ pub struct PickledInboundGroupSession {
     /// The public ed25519 key of the account that sent us the session.
     pub signing_key: BTreeMap<DeviceKeyAlgorithm, String>,
     /// The id of the room that the session is used in.
-    pub room_id: RoomId,
+    pub room_id: OwnedRoomId,
     /// The list of claimed ed25519 that forwarded us this key. Will be None if
     /// we directly received this session.
     #[serde(default)]
@@ -377,42 +435,22 @@ pub struct PickledInboundGroupSession {
     pub history_visibility: Option<HistoryVisibility>,
 }
 
-/// The typed representation of a base64 encoded string of the GroupSession
-/// pickle.
-#[derive(Debug, Clone, Serialize, Deserialize)]
-pub struct InboundGroupSessionPickle(String);
-
-impl From<String> for InboundGroupSessionPickle {
-    fn from(pickle_string: String) -> Self {
-        InboundGroupSessionPickle(pickle_string)
-    }
-}
-
-impl InboundGroupSessionPickle {
-    /// Get the string representation of the pickle.
-    pub fn as_str(&self) -> &str {
-        &self.0
-    }
-}
-
-impl TryFrom<ExportedRoomKey> for InboundGroupSession {
-    type Error = OlmGroupSessionError;
-
-    fn try_from(key: ExportedRoomKey) -> Result<Self, Self::Error> {
-        let session = OlmInboundGroupSession::import(&key.session_key.0)?;
+impl From<ExportedRoomKey> for InboundGroupSession {
+    fn from(key: ExportedRoomKey) -> Self {
+        let session = InnerSession::import(&key.session_key);
         let first_known_index = session.first_known_index();
 
-        Ok(InboundGroupSession {
-            inner: Arc::new(Mutex::new(session)),
+        InboundGroupSession {
+            inner: Mutex::new(session).into(),
             session_id: key.session_id.into(),
             sender_key: key.sender_key.into(),
             history_visibility: None.into(),
             first_known_index,
-            signing_keys: Arc::new(key.sender_claimed_keys),
-            room_id: Arc::new(key.room_id),
-            forwarding_chains: Arc::new(key.forwarding_curve25519_key_chain),
+            signing_keys: key.sender_claimed_keys.into(),
+            room_id: (&*key.room_id).into(),
+            forwarding_chains: key.forwarding_curve25519_key_chain.into(),
             imported: true,
-            backed_up: false,
-        })
+            backed_up: AtomicBool::from(false).into(),
+        }
     }
 }
