@@ -16,43 +16,35 @@ use std::{collections::BTreeMap, convert::TryInto};
 
 use ruma::{
     events::forwarded_room_key::{
-        ForwardedRoomKeyToDeviceEventContent, ForwardedRoomKeyToDeviceEventContentInit,
+        ToDeviceForwardedRoomKeyEventContent, ToDeviceForwardedRoomKeyEventContentInit,
     },
-    DeviceKeyAlgorithm, EventEncryptionAlgorithm, RoomId,
+    DeviceKeyAlgorithm, EventEncryptionAlgorithm, OwnedRoomId,
 };
 use serde::{Deserialize, Serialize};
-use zeroize::Zeroize;
 
 mod inbound;
 mod outbound;
 
-pub use inbound::{InboundGroupSession, InboundGroupSessionPickle, PickledInboundGroupSession};
+pub use inbound::{InboundGroupSession, PickledInboundGroupSession};
+pub(crate) use outbound::ShareState;
 pub use outbound::{
-    EncryptionSettings, OutboundGroupSession, PickledOutboundGroupSession, ShareInfo, ShareState,
+    EncryptionSettings, GroupSession, OutboundGroupSession, PickledOutboundGroupSession, ShareInfo,
 };
+use vodozemac::megolm::SessionKeyDecodeError;
+pub use vodozemac::megolm::{ExportedSessionKey, SessionKey};
+use zeroize::Zeroize;
 
-/// The private session key of a group session.
-/// Can be used to create a new inbound group session.
-#[derive(Clone, Debug, Serialize, Deserialize, Zeroize)]
-#[zeroize(drop)]
-pub struct GroupSessionKey(pub String);
-
-/// The exported version of an private session key of a group session.
-/// Can be used to create a new inbound group session.
-#[derive(Clone, Debug, Serialize, Deserialize, PartialEq, Zeroize)]
-#[zeroize(drop)]
-pub struct ExportedGroupSessionKey(pub String);
-
-/// An exported version of a `InboundGroupSession`
+/// An exported version of an `InboundGroupSession`
 ///
 /// This can be used to share the `InboundGroupSession` in an exported file.
-#[derive(Clone, Debug, Deserialize, Serialize, PartialEq)]
+#[derive(Deserialize, Serialize)]
+#[allow(missing_debug_implementations)]
 pub struct ExportedRoomKey {
     /// The encryption algorithm that the session uses.
     pub algorithm: EventEncryptionAlgorithm,
 
     /// The room where the session is used.
-    pub room_id: RoomId,
+    pub room_id: OwnedRoomId,
 
     /// The Curve25519 key of the device which initiated the session originally.
     pub sender_key: String,
@@ -61,7 +53,32 @@ pub struct ExportedRoomKey {
     pub session_id: String,
 
     /// The key for the session.
-    pub session_key: ExportedGroupSessionKey,
+    pub session_key: ExportedSessionKey,
+
+    /// The Ed25519 key of the device which initiated the session originally.
+    #[serde(default)]
+    pub sender_claimed_keys: BTreeMap<DeviceKeyAlgorithm, String>,
+
+    /// Chain of Curve25519 keys through which this session was forwarded, via
+    /// m.forwarded_room_key events.
+    #[serde(default)]
+    pub forwarding_curve25519_key_chain: Vec<String>,
+}
+
+/// A backed up version of an `InboundGroupSession`
+///
+/// This can be used to backup the `InboundGroupSession` to the server.
+#[derive(Deserialize, Serialize)]
+#[allow(missing_debug_implementations)]
+pub struct BackedUpRoomKey {
+    /// The encryption algorithm that the session uses.
+    pub algorithm: EventEncryptionAlgorithm,
+
+    /// The Curve25519 key of the device which initiated the session originally.
+    pub sender_key: String,
+
+    /// The key for the session.
+    pub session_key: ExportedSessionKey,
 
     /// The Ed25519 key of the device which initiated the session originally.
     pub sender_claimed_keys: BTreeMap<DeviceKeyAlgorithm, String>,
@@ -71,7 +88,7 @@ pub struct ExportedRoomKey {
     pub forwarding_curve25519_key_chain: Vec<String>,
 }
 
-impl TryInto<ForwardedRoomKeyToDeviceEventContent> for ExportedRoomKey {
+impl TryInto<ToDeviceForwardedRoomKeyEventContent> for ExportedRoomKey {
     type Error = ();
 
     /// Convert an exported room key into a content for a forwarded room key
@@ -80,7 +97,7 @@ impl TryInto<ForwardedRoomKeyToDeviceEventContent> for ExportedRoomKey {
     /// This will fail if the exported room key has multiple sender claimed keys
     /// or if the algorithm of the claimed sender key isn't
     /// `DeviceKeyAlgorithm::Ed25519`.
-    fn try_into(self) -> Result<ForwardedRoomKeyToDeviceEventContent, Self::Error> {
+    fn try_into(self) -> Result<ToDeviceForwardedRoomKeyEventContent, Self::Error> {
         if self.sender_claimed_keys.len() != 1 {
             Err(())
         } else {
@@ -90,12 +107,12 @@ impl TryInto<ForwardedRoomKeyToDeviceEventContent> for ExportedRoomKey {
                 return Err(());
             }
 
-            Ok(ForwardedRoomKeyToDeviceEventContentInit {
+            Ok(ToDeviceForwardedRoomKeyEventContentInit {
                 algorithm: self.algorithm,
                 room_id: self.room_id,
                 sender_key: self.sender_key,
                 session_id: self.session_id,
-                session_key: self.session_key.0.clone(),
+                session_key: self.session_key.to_base64(),
                 sender_claimed_ed25519_key: claimed_key.to_owned(),
                 forwarding_curve25519_key_chain: self.forwarding_curve25519_key_chain,
             }
@@ -104,71 +121,40 @@ impl TryInto<ForwardedRoomKeyToDeviceEventContent> for ExportedRoomKey {
     }
 }
 
-impl From<ForwardedRoomKeyToDeviceEventContent> for ExportedRoomKey {
+impl From<ExportedRoomKey> for BackedUpRoomKey {
+    fn from(k: ExportedRoomKey) -> Self {
+        Self {
+            algorithm: k.algorithm,
+            sender_key: k.sender_key,
+            session_key: k.session_key,
+            sender_claimed_keys: k.sender_claimed_keys,
+            forwarding_curve25519_key_chain: k.forwarding_curve25519_key_chain,
+        }
+    }
+}
+
+impl TryFrom<ToDeviceForwardedRoomKeyEventContent> for ExportedRoomKey {
+    type Error = SessionKeyDecodeError;
+
     /// Convert the content of a forwarded room key into a exported room key.
-    fn from(forwarded_key: ForwardedRoomKeyToDeviceEventContent) -> Self {
+    fn try_from(
+        mut forwarded_key: ToDeviceForwardedRoomKeyEventContent,
+    ) -> Result<Self, Self::Error> {
         let mut sender_claimed_keys: BTreeMap<DeviceKeyAlgorithm, String> = BTreeMap::new();
         sender_claimed_keys
             .insert(DeviceKeyAlgorithm::Ed25519, forwarded_key.sender_claimed_ed25519_key);
 
-        Self {
+        let session_key = ExportedSessionKey::from_base64(&forwarded_key.session_key)?;
+        forwarded_key.session_key.zeroize();
+
+        Ok(Self {
             algorithm: forwarded_key.algorithm,
             room_id: forwarded_key.room_id,
             session_id: forwarded_key.session_id,
             forwarding_curve25519_key_chain: forwarded_key.forwarding_curve25519_key_chain,
             sender_claimed_keys,
             sender_key: forwarded_key.sender_key,
-            session_key: ExportedGroupSessionKey(forwarded_key.session_key),
-        }
-    }
-}
-
-#[cfg(test)]
-mod test {
-    use std::{
-        sync::Arc,
-        time::{Duration, Instant},
-    };
-
-    use ruma::{
-        events::{room::message::MessageEventContent, AnyMessageEventContent},
-        room_id, user_id,
-    };
-
-    use super::EncryptionSettings;
-    use crate::ReadOnlyAccount;
-
-    #[tokio::test]
-    #[cfg(target_os = "linux")]
-    async fn expiration() {
-        let settings = EncryptionSettings { rotation_period_msgs: 1, ..Default::default() };
-
-        let account = ReadOnlyAccount::new(&user_id!("@alice:example.org"), "DEVICEID".into());
-        let (session, _) = account
-            .create_group_session_pair(&room_id!("!test_room:example.org"), settings)
-            .await
-            .unwrap();
-
-        assert!(!session.expired());
-        let _ = session
-            .encrypt(AnyMessageEventContent::RoomMessage(MessageEventContent::text_plain(
-                "Test message",
-            )))
-            .await;
-        assert!(session.expired());
-
-        let settings = EncryptionSettings {
-            rotation_period: Duration::from_millis(100),
-            ..Default::default()
-        };
-
-        let (mut session, _) = account
-            .create_group_session_pair(&room_id!("!test_room:example.org"), settings)
-            .await
-            .unwrap();
-
-        assert!(!session.expired());
-        session.creation_time = Arc::new(Instant::now() - Duration::from_secs(60 * 60));
-        assert!(session.expired());
+            session_key,
+        })
     }
 }
