@@ -12,42 +12,44 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::{collections::BTreeMap, sync::Arc, time::Duration};
+use std::{collections::BTreeMap, iter, sync::Arc, time::Duration};
 
-use matrix_sdk_common::uuid::Uuid;
 use ruma::{
-    api::client::r0::{
+    api::client::{
+        backup::{add_backup_keys::v3::Response as KeysBackupResponse, RoomKeyBackup},
         keys::{
-            claim_keys::{Request as KeysClaimRequest, Response as KeysClaimResponse},
-            get_keys::Response as KeysQueryResponse,
-            upload_keys::{Request as KeysUploadRequest, Response as KeysUploadResponse},
-            upload_signatures::{
+            claim_keys::v3::{Request as KeysClaimRequest, Response as KeysClaimResponse},
+            get_keys::v3::Response as KeysQueryResponse,
+            upload_keys::v3::{Request as KeysUploadRequest, Response as KeysUploadResponse},
+            upload_signatures::v3::{
                 Request as SignatureUploadRequest, Response as SignatureUploadResponse,
             },
-            upload_signing_keys::Response as SigningKeysUploadResponse,
+            upload_signing_keys::v3::Response as SigningKeysUploadResponse,
         },
-        message::send_message_event::Response as RoomMessageResponse,
-        to_device::send_event_to_device::Response as ToDeviceResponse,
+        message::send_message_event::v3::Response as RoomMessageResponse,
+        to_device::send_event_to_device::v3::Response as ToDeviceResponse,
+    },
+    events::{
+        AnyMessageLikeEventContent, AnyToDeviceEventContent, EventContent, ToDeviceEventType,
     },
-    encryption::CrossSigningKey,
-    events::{AnyMessageEventContent, AnyToDeviceEventContent, EventContent, EventType},
     serde::Raw,
     to_device::DeviceIdOrAllDevices,
-    DeviceIdBox, RoomId, UserId,
+    OwnedDeviceId, OwnedRoomId, OwnedTransactionId, OwnedUserId, TransactionId, UserId,
 };
 use serde::{Deserialize, Serialize};
 
+use crate::types::CrossSigningKey;
+
 /// Customized version of
-/// `ruma_client_api::r0::to_device::send_event_to_device::Request`,
-/// using a UUID for the transaction ID.
+/// `ruma_client_api::to_device::send_event_to_device::v3::Request`
 #[derive(Clone, Debug, Deserialize, Serialize)]
 pub struct ToDeviceRequest {
     /// Type of event being sent to each device.
-    pub event_type: EventType,
+    pub event_type: ToDeviceEventType,
 
     /// A request identifier unique to the access token used to send the
     /// request.
-    pub txn_id: Uuid,
+    pub txn_id: OwnedTransactionId,
 
     /// A map of users to devices to a content for a message event to be
     /// sent to the user's device. Individual message events can be sent
@@ -55,7 +57,8 @@ pub struct ToDeviceRequest {
     /// The content's type for this field will be updated in a future
     /// release, until then you can create a value using
     /// `serde_json::value::to_raw_value`.
-    pub messages: BTreeMap<UserId, BTreeMap<DeviceIdOrAllDevices, Raw<AnyToDeviceEventContent>>>,
+    pub messages:
+        BTreeMap<OwnedUserId, BTreeMap<DeviceIdOrAllDevices, Raw<AnyToDeviceEventContent>>>,
 }
 
 impl ToDeviceRequest {
@@ -75,53 +78,48 @@ impl ToDeviceRequest {
         recipient_device: impl Into<DeviceIdOrAllDevices>,
         content: AnyToDeviceEventContent,
     ) -> Self {
-        Self::new_with_id(recipient, recipient_device, content, Uuid::new_v4())
+        Self::with_id(recipient, recipient_device, content, TransactionId::new())
     }
 
-    pub(crate) fn new_for_recipients(
+    pub(crate) fn for_recipients(
         recipient: &UserId,
-        recipient_devices: Vec<DeviceIdBox>,
+        recipient_devices: Vec<OwnedDeviceId>,
         content: AnyToDeviceEventContent,
-        txn_id: Uuid,
+        txn_id: OwnedTransactionId,
     ) -> Self {
-        let mut messages = BTreeMap::new();
-        let event_type = EventType::from(content.event_type());
-
         if recipient_devices.is_empty() {
             Self::new(recipient, DeviceIdOrAllDevices::AllDevices, content)
         } else {
+            let event_type = content.event_type();
             let device_messages = recipient_devices
                 .into_iter()
-                .map(|d| (DeviceIdOrAllDevices::DeviceId(d), Raw::from(content.clone())))
+                .map(|d| {
+                    let raw_content =
+                        Raw::new(&content).expect("Failed to serialize to-device event");
+                    (DeviceIdOrAllDevices::DeviceId(d), raw_content)
+                })
                 .collect();
 
-            messages.insert(recipient.clone(), device_messages);
-
+            let messages = iter::once((recipient.to_owned(), device_messages)).collect();
             ToDeviceRequest { event_type, txn_id, messages }
         }
     }
 
-    pub(crate) fn new_with_id(
+    pub(crate) fn with_id(
         recipient: &UserId,
         recipient_device: impl Into<DeviceIdOrAllDevices>,
         content: AnyToDeviceEventContent,
-        txn_id: Uuid,
+        txn_id: OwnedTransactionId,
     ) -> Self {
-        let mut messages = BTreeMap::new();
-        let mut user_messages = BTreeMap::new();
-        let event_type = EventType::from(content.event_type());
+        let event_type = content.event_type();
+        let raw_content = Raw::new(&content).expect("Failed to serialize to-device event");
 
-        user_messages.insert(recipient_device.into(), Raw::from(content));
-        messages.insert(recipient.clone(), user_messages);
+        let user_messages = iter::once((recipient_device.into(), raw_content)).collect();
+        let messages = iter::once((recipient.to_owned(), user_messages)).collect();
 
         ToDeviceRequest { event_type, txn_id, messages }
     }
 
-    /// Gets the transaction ID as a string.
-    pub fn txn_id_string(&self) -> String {
-        self.txn_id.to_string()
-    }
-
     /// Get the number of unique messages this request contains.
     ///
     /// *Note*: A single message may be sent to multiple devices, so this may or
@@ -149,7 +147,7 @@ pub struct UploadSigningKeysRequest {
 }
 
 /// Customized version of
-/// `ruma_client_api::r0::keys::get_keys::Request`, without any
+/// `ruma_client_api::keys::get_keys::v3::Request`, without any
 /// references.
 #[derive(Clone, Debug)]
 pub struct KeysQueryRequest {
@@ -159,7 +157,7 @@ pub struct KeysQueryRequest {
 
     /// The keys to be downloaded. An empty list indicates all devices for
     /// the corresponding user.
-    pub device_keys: BTreeMap<UserId, Vec<DeviceIdBox>>,
+    pub device_keys: BTreeMap<OwnedUserId, Vec<OwnedDeviceId>>,
 
     /// If the client is fetching keys as a result of a device update
     /// received in a sync request, this should be the 'since' token of that
@@ -170,7 +168,7 @@ pub struct KeysQueryRequest {
 }
 
 impl KeysQueryRequest {
-    pub(crate) fn new(device_keys: BTreeMap<UserId, Vec<DeviceIdBox>>) -> Self {
+    pub(crate) fn new(device_keys: BTreeMap<OwnedUserId, Vec<OwnedDeviceId>>) -> Self {
         Self { timeout: None, device_keys, token: None }
     }
 }
@@ -197,6 +195,8 @@ pub enum OutgoingRequests {
     /// A room message request, usually for sending in-room interactive
     /// verification events.
     RoomMessage(RoomMessageRequest),
+    /// A request that will back up a batch of room keys to the server.
+    KeysBackup(KeysBackupRequest),
 }
 
 #[cfg(test)]
@@ -215,6 +215,12 @@ impl From<KeysQueryRequest> for OutgoingRequests {
     }
 }
 
+impl From<KeysBackupRequest> for OutgoingRequests {
+    fn from(r: KeysBackupRequest) -> Self {
+        OutgoingRequests::KeysBackup(r)
+    }
+}
+
 impl From<KeysClaimRequest> for OutgoingRequests {
     fn from(r: KeysClaimRequest) -> Self {
         Self::KeysClaim(r)
@@ -247,13 +253,13 @@ impl From<SignatureUploadRequest> for OutgoingRequests {
 
 impl From<OutgoingVerificationRequest> for OutgoingRequest {
     fn from(r: OutgoingVerificationRequest) -> Self {
-        Self { request_id: r.request_id(), request: Arc::new(r.into()) }
+        Self { request_id: r.request_id().to_owned(), request: Arc::new(r.into()) }
     }
 }
 
 impl From<SignatureUploadRequest> for OutgoingRequest {
     fn from(r: SignatureUploadRequest) -> Self {
-        Self { request_id: Uuid::new_v4(), request: Arc::new(r.into()) }
+        Self { request_id: TransactionId::new(), request: Arc::new(r.into()) }
     }
 }
 
@@ -278,6 +284,8 @@ pub enum IncomingResponse<'a> {
     SignatureUpload(&'a SignatureUploadResponse),
     /// A room message response, usually for interactive verifications.
     RoomMessage(&'a RoomMessageResponse),
+    /// Response for the server-side room key backup request.
+    KeysBackup(&'a KeysBackupResponse),
 }
 
 impl<'a> From<&'a KeysUploadResponse> for IncomingResponse<'a> {
@@ -286,6 +294,12 @@ impl<'a> From<&'a KeysUploadResponse> for IncomingResponse<'a> {
     }
 }
 
+impl<'a> From<&'a KeysBackupResponse> for IncomingResponse<'a> {
+    fn from(response: &'a KeysBackupResponse) -> Self {
+        IncomingResponse::KeysBackup(response)
+    }
+}
+
 impl<'a> From<&'a KeysQueryResponse> for IncomingResponse<'a> {
     fn from(response: &'a KeysQueryResponse) -> Self {
         IncomingResponse::KeysQuery(response)
@@ -322,14 +336,14 @@ impl<'a> From<&'a SignatureUploadResponse> for IncomingResponse<'a> {
 pub struct OutgoingRequest {
     /// The unique id of a request, needs to be passed when receiving a
     /// response.
-    pub(crate) request_id: Uuid,
+    pub(crate) request_id: OwnedTransactionId,
     /// The underlying outgoing request.
     pub(crate) request: Arc<OutgoingRequests>,
 }
 
 impl OutgoingRequest {
     /// Get the unique id of this request.
-    pub fn request_id(&self) -> &Uuid {
+    pub fn request_id(&self) -> &TransactionId {
         &self.request_id
     }
 
@@ -343,17 +357,27 @@ impl OutgoingRequest {
 #[derive(Clone, Debug)]
 pub struct RoomMessageRequest {
     /// The room to send the event to.
-    pub room_id: RoomId,
+    pub room_id: OwnedRoomId,
 
     /// The transaction ID for this event.
     ///
     /// Clients should generate an ID unique across requests with the
     /// same access token; it will be used by the server to ensure
     /// idempotency of requests.
-    pub txn_id: Uuid,
+    pub txn_id: OwnedTransactionId,
 
     /// The event content to send.
-    pub content: AnyMessageEventContent,
+    pub content: AnyMessageLikeEventContent,
+}
+
+/// A request that will back up a batch of room keys to the server.
+#[derive(Clone, Debug)]
+pub struct KeysBackupRequest {
+    /// The backup version that these room keys should be part of.
+    pub version: String,
+    /// The map from room id to a backed up room key that we're going to upload
+    /// to the server.
+    pub rooms: BTreeMap<OwnedRoomId, RoomKeyBackup>,
 }
 
 /// An enum over the different outgoing verification based requests.
@@ -367,10 +391,10 @@ pub enum OutgoingVerificationRequest {
 
 impl OutgoingVerificationRequest {
     /// Get the unique id of this request.
-    pub fn request_id(&self) -> Uuid {
+    pub fn request_id(&self) -> &TransactionId {
         match self {
-            OutgoingVerificationRequest::ToDevice(t) => t.txn_id,
-            OutgoingVerificationRequest::InRoom(r) => r.txn_id,
+            OutgoingVerificationRequest::ToDevice(t) => &t.txn_id,
+            OutgoingVerificationRequest::InRoom(r) => &r.txn_id,
         }
     }
 }
