@@ -14,21 +14,24 @@
 
 use std::{
     collections::{BTreeMap, BTreeSet, HashMap, HashSet},
+    ops::Deref,
     sync::Arc,
 };
 
 use dashmap::DashMap;
-use futures::future::join_all;
-use matrix_sdk_common::{executor::spawn, uuid::Uuid};
+use futures_util::future::join_all;
+use matrix_sdk_common::executor::spawn;
 use ruma::{
     events::{
-        room::{encrypted::EncryptedEventContent, history_visibility::HistoryVisibility},
-        AnyMessageEventContent, AnyToDeviceEventContent, EventType,
+        room::{encrypted::RoomEncryptedEventContent, history_visibility::HistoryVisibility},
+        AnyToDeviceEventContent, ToDeviceEventType,
     },
     serde::Raw,
     to_device::DeviceIdOrAllDevices,
-    DeviceId, DeviceIdBox, RoomId, UserId,
+    DeviceId, OwnedDeviceId, OwnedRoomId, OwnedTransactionId, OwnedUserId, RoomId, TransactionId,
+    UserId,
 };
+use serde_json::Value;
 use tracing::{debug, info, trace};
 
 use crate::{
@@ -41,10 +44,10 @@ use crate::{
 #[derive(Clone, Debug)]
 pub(crate) struct GroupSessionCache {
     store: Store,
-    sessions: Arc<DashMap<RoomId, OutboundGroupSession>>,
+    sessions: Arc<DashMap<OwnedRoomId, OutboundGroupSession>>,
     /// A map from the request id to the group session that the request belongs
     /// to. Used to mark requests belonging to the session as shared.
-    sessions_being_shared: Arc<DashMap<Uuid, OutboundGroupSession>>,
+    sessions_being_shared: Arc<DashMap<OwnedTransactionId, OutboundGroupSession>>,
 }
 
 impl GroupSessionCache {
@@ -72,7 +75,7 @@ impl GroupSessionCache {
                 self.sessions_being_shared.insert(request_id, s.clone());
             }
 
-            self.sessions.insert(room_id.clone(), s.clone());
+            self.sessions.insert(room_id.to_owned(), s.clone());
 
             Ok(Some(s))
         } else {
@@ -135,7 +138,7 @@ impl GroupSessionManager {
         }
     }
 
-    pub async fn mark_request_as_sent(&self, request_id: &Uuid) -> StoreResult<()> {
+    pub async fn mark_request_as_sent(&self, request_id: &TransactionId) -> StoreResult<()> {
         if let Some((_, s)) = self.sessions.sessions_being_shared.remove(request_id) {
             s.mark_request_as_sent(request_id);
 
@@ -155,19 +158,14 @@ impl GroupSessionManager {
     pub async fn encrypt(
         &self,
         room_id: &RoomId,
-        content: AnyMessageEventContent,
-    ) -> MegolmResult<EncryptedEventContent> {
-        let session = if let Some(s) = self.sessions.get(room_id) {
-            s
-        } else {
-            panic!("Session wasn't created nor shared");
-        };
+        content: Value,
+        event_type: &str,
+    ) -> MegolmResult<RoomEncryptedEventContent> {
+        let session = self.sessions.get(room_id).expect("Session wasn't created nor shared");
 
-        if session.expired() {
-            panic!("Session expired");
-        }
+        assert!(!session.expired(), "Session expired");
 
-        let content = session.encrypt(content).await;
+        let content = session.encrypt(content, event_type).await;
 
         let mut changes = Changes::default();
         changes.outbound_group_sessions.push(session);
@@ -224,18 +222,26 @@ impl GroupSessionManager {
         devices: Vec<Device>,
         message_index: u32,
     ) -> OlmResult<(
-        Uuid,
+        OwnedTransactionId,
         ToDeviceRequest,
-        BTreeMap<UserId, BTreeMap<DeviceIdBox, ShareInfo>>,
+        BTreeMap<OwnedUserId, BTreeMap<OwnedDeviceId, ShareInfo>>,
         Vec<Session>,
     )> {
+        // Use a named type instead of a tuple with rather long type name
+        struct EncryptResult {
+            used_session: Option<Session>,
+            share_info: BTreeMap<OwnedUserId, BTreeMap<OwnedDeviceId, ShareInfo>>,
+            message:
+                BTreeMap<OwnedUserId, BTreeMap<DeviceIdOrAllDevices, Raw<AnyToDeviceEventContent>>>,
+        }
+
         let mut messages = BTreeMap::new();
         let mut changed_sessions = Vec::new();
         let mut share_infos = BTreeMap::new();
 
         let encrypt = |device: Device, content: AnyToDeviceEventContent| async move {
             let mut message = BTreeMap::new();
-            let mut share_infos = BTreeMap::new();
+            let mut share_info = BTreeMap::new();
 
             let encrypted = device.encrypt(content.clone()).await;
 
@@ -246,9 +252,10 @@ impl GroupSessionManager {
                         .or_insert_with(BTreeMap::new)
                         .insert(
                             DeviceIdOrAllDevices::DeviceId(device.device_id().into()),
-                            Raw::from(AnyToDeviceEventContent::RoomEncrypted(encrypted)),
+                            Raw::new(&AnyToDeviceEventContent::RoomEncrypted(encrypted))
+                                .expect("Failed to serialize encrypted event"),
                         );
-                    share_infos
+                    share_info
                         .entry(device.user_id().to_owned())
                         .or_insert_with(BTreeMap::new)
                         .insert(
@@ -267,7 +274,7 @@ impl GroupSessionManager {
                 Err(e) => return Err(e),
             };
 
-            Ok((used_session, share_infos, message))
+            Ok(EncryptResult { used_session, share_info, message })
         };
 
         let tasks: Vec<_> =
@@ -276,33 +283,36 @@ impl GroupSessionManager {
         let results = join_all(tasks).await;
 
         for result in results {
-            let (used_session, infos, message) = result.expect("Encryption task panicked")?;
+            let EncryptResult { used_session, share_info, message } =
+                result.expect("Encryption task panicked")?;
 
             if let Some(session) = used_session {
                 changed_sessions.push(session);
             }
 
-            for (user, device_messages) in message.into_iter() {
+            for (user, device_messages) in message {
                 messages.entry(user).or_insert_with(BTreeMap::new).extend(device_messages);
             }
 
-            for (user, infos) in infos.into_iter() {
+            for (user, infos) in share_info {
                 share_infos.entry(user).or_insert_with(BTreeMap::new).extend(infos);
             }
         }
 
-        let id = Uuid::new_v4();
-
-        let request =
-            ToDeviceRequest { event_type: EventType::RoomEncrypted, txn_id: id, messages };
+        let txn_id = TransactionId::new();
+        let request = ToDeviceRequest {
+            event_type: ToDeviceEventType::RoomEncrypted,
+            txn_id: txn_id.clone(),
+            messages,
+        };
 
         trace!(
             recipient_count = request.message_count(),
-            transaction_id = ?id,
+            transaction_id = ?txn_id,
             "Created a to-device request carrying a room_key"
         );
 
-        Ok((id, request, share_infos, changed_sessions))
+        Ok((txn_id, request, share_infos, changed_sessions))
     }
 
     /// Given a list of user and an outbound session, return the list of users
@@ -315,21 +325,23 @@ impl GroupSessionManager {
         users: impl Iterator<Item = &UserId>,
         history_visibility: HistoryVisibility,
         outbound: &OutboundGroupSession,
-    ) -> OlmResult<(bool, HashMap<UserId, Vec<Device>>)> {
+    ) -> OlmResult<(bool, HashMap<OwnedUserId, Vec<Device>>)> {
         let users: HashSet<&UserId> = users.collect();
-        let mut devices: HashMap<UserId, Vec<Device>> = HashMap::new();
+        let mut devices: HashMap<OwnedUserId, Vec<Device>> = HashMap::new();
 
-        debug!(
-            users = ?users,
-            history_visibility = ?history_visibility,
+        trace!(
+            ?users,
+            ?history_visibility,
             session_id = outbound.session_id(),
+            room_id = outbound.room_id().as_str(),
             "Calculating group session recipients"
         );
 
-        let users_shared_with: HashSet<UserId> =
+        let users_shared_with: HashSet<OwnedUserId> =
             outbound.shared_with_set.iter().map(|k| k.key().clone()).collect();
 
-        let users_shared_with: HashSet<&UserId> = users_shared_with.iter().collect();
+        let users_shared_with: HashSet<&UserId> =
+            users_shared_with.iter().map(Deref::deref).collect();
 
         // A user left if a user is missing from the set of users that should
         // get the session but is in the set of users that received the session.
@@ -347,7 +359,7 @@ impl GroupSessionManager {
         let mut should_rotate = user_left || visibility_changed;
 
         for user_id in users {
-            let user_devices = self.store.get_user_devices(user_id).await?;
+            let user_devices = self.store.get_user_devices_filtered(user_id).await?;
             let non_blacklisted_devices: Vec<Device> =
                 user_devices.devices().filter(|d| !d.is_blacklisted()).collect();
 
@@ -361,9 +373,8 @@ impl GroupSessionManager {
                     non_blacklisted_devices.iter().map(|d| d.device_id()).collect();
 
                 if let Some(shared) = outbound.shared_with_set.get(user_id) {
-                    #[allow(clippy::map_clone)]
                     // Devices that received this session
-                    let shared: HashSet<DeviceIdBox> =
+                    let shared: HashSet<OwnedDeviceId> =
                         shared.iter().map(|d| d.key().clone()).collect();
                     let shared: HashSet<&DeviceId> = shared.iter().map(|d| d.as_ref()).collect();
 
@@ -383,12 +394,13 @@ impl GroupSessionManager {
                 };
             }
 
-            devices.entry(user_id.clone()).or_insert_with(Vec::new).extend(non_blacklisted_devices);
+            devices.entry(user_id.to_owned()).or_default().extend(non_blacklisted_devices);
         }
 
-        debug!(
+        trace!(
             should_rotate = should_rotate,
             session_id = outbound.session_id(),
+            room_id = outbound.room_id().as_str(),
             "Done calculating group session recipients"
         );
 
@@ -400,13 +412,13 @@ impl GroupSessionManager {
         content: AnyToDeviceEventContent,
         outbound: OutboundGroupSession,
         message_index: u32,
-        being_shared: Arc<DashMap<Uuid, OutboundGroupSession>>,
+        being_shared: Arc<DashMap<OwnedTransactionId, OutboundGroupSession>>,
     ) -> OlmResult<Vec<Session>> {
         let (id, request, share_infos, used_sessions) =
             Self::encrypt_session_for(content.clone(), chunk, message_index).await?;
 
         if !request.messages.is_empty() {
-            outbound.add_request(id, request.into(), share_infos);
+            outbound.add_request(id.clone(), request.into(), share_infos);
             being_shared.insert(id, outbound.clone());
         }
 
@@ -434,7 +446,7 @@ impl GroupSessionManager {
         users: impl Iterator<Item = &UserId>,
         encryption_settings: impl Into<EncryptionSettings>,
     ) -> OlmResult<Vec<Arc<ToDeviceRequest>>> {
-        debug!(room_id = room_id.as_str(), "Checking if a room key needs to be shared",);
+        trace!(room_id = room_id.as_str(), "Checking if a room key needs to be shared",);
 
         let encryption_settings = encryption_settings.into();
         let history_visibility = encryption_settings.history_visibility.clone();
@@ -463,8 +475,8 @@ impl GroupSessionManager {
                 room_id = room_id.as_str(),
                 old_session_id = old_session_id,
                 session_id = outbound.session_id(),
-                "A user/device has left the group since we last sent a message, \
-                   rotating the outbound session.",
+                "A user or device has left the room since we last sent a \
+                message, rotating the room key.",
             );
 
             outbound
@@ -474,27 +486,27 @@ impl GroupSessionManager {
 
         let devices: Vec<Device> = devices
             .into_iter()
-            .map(|(_, d)| {
+            .flat_map(|(_, d)| {
                 d.into_iter()
                     .filter(|d| matches!(outbound.is_shared_with(d), ShareState::NotShared))
             })
-            .flatten()
             .collect();
 
         let key_content = outbound.as_content().await;
         let message_index = outbound.message_index().await;
 
         if !devices.is_empty() {
-            let users = devices.iter().fold(BTreeMap::new(), |mut acc, d| {
+            let recipients = devices.iter().fold(BTreeMap::new(), |mut acc, d| {
                 acc.entry(d.user_id()).or_insert_with(BTreeSet::new).insert(d.device_id());
                 acc
             });
 
             info!(
                 index = message_index,
-                users = ?users,
+                ?recipients,
                 room_id = room_id.as_str(),
-                "Sharing an outbound group session",
+                session_id = outbound.session_id(),
+                "Trying to encrypt a room key",
             );
         }
 
@@ -519,100 +531,127 @@ impl GroupSessionManager {
 
         let requests = outbound.pending_requests();
 
-        debug!(
-            room_id = room_id.as_str(),
-            session_id = outbound.session_id(),
-            request_count = requests.len(),
-            "Done generating to-device requests for a room key share"
-        );
-
         if requests.is_empty() {
-            debug!(
+            if !outbound.shared() {
+                debug!(
+                    room_id = room_id.as_str(),
+                    session_id = outbound.session_id(),
+                    "The room key doesn't need to be shared with anyone. Marking as shared."
+                );
+
+                outbound.mark_as_shared();
+                changes.outbound_group_sessions.push(outbound.clone());
+            }
+        } else {
+            let mut recipients: BTreeMap<&UserId, BTreeSet<&DeviceIdOrAllDevices>> =
+                BTreeMap::new();
+
+            for request in &requests {
+                for (user_id, device_map) in &request.messages {
+                    let devices = device_map.keys();
+                    recipients.entry(user_id).or_default().extend(devices)
+                }
+            }
+
+            let transaction_ids: Vec<_> = requests.iter().map(|r| r.txn_id.clone()).collect();
+
+            // TODO log the withheld reasons here as well.
+            info!(
                 room_id = room_id.as_str(),
                 session_id = outbound.session_id(),
-                "The outbound group session doesn't need to be shared with \
-                    anyone, marking as shared",
+                request_count = requests.len(),
+                ?transaction_ids,
+                ?recipients,
+                "Encrypted a room key and created to-device requests"
             );
-            outbound.mark_as_shared();
         }
 
-        let session_count = changes.sessions.len();
+        if !changes.is_empty() {
+            let session_count = changes.sessions.len();
 
-        self.store.save_changes(changes).await?;
+            self.store.save_changes(changes).await?;
 
-        debug!(
-            room_id = room_id.as_str(),
-            session_id = outbound.session_id(),
-            session_count = session_count,
-            "Stored the changed sessions after encrypting an room key"
-        );
+            trace!(
+                room_id = room_id.as_str(),
+                session_id = outbound.session_id(),
+                session_count = session_count,
+                "Stored the changed sessions after encrypting an room key"
+            );
+        }
 
         Ok(requests)
     }
 }
 
 #[cfg(test)]
-mod test {
-    use matrix_sdk_common::uuid::Uuid;
-    use matrix_sdk_test::response_from_file;
+mod tests {
+    use std::ops::Deref;
+
+    use matrix_sdk_test::{async_test, response_from_file};
     use ruma::{
         api::{
-            client::r0::keys::{claim_keys, get_keys},
+            client::keys::{claim_keys, get_keys},
             IncomingResponse,
         },
-        room_id, user_id, DeviceIdBox, UserId,
+        device_id,
+        events::room::history_visibility::HistoryVisibility,
+        room_id, user_id, DeviceId, TransactionId, UserId,
     };
     use serde_json::Value;
 
     use crate::{EncryptionSettings, OlmMachine};
 
-    fn alice_id() -> UserId {
+    fn alice_id() -> &'static UserId {
         user_id!("@alice:example.org")
     }
 
-    fn alice_device_id() -> DeviceIdBox {
-        "JLAFKJWSCS".into()
+    fn alice_device_id() -> &'static DeviceId {
+        device_id!("JLAFKJWSCS")
     }
 
-    fn keys_query_response() -> get_keys::Response {
-        let data = include_bytes!("../../benches/keys_query.json");
+    fn keys_query_response() -> get_keys::v3::Response {
+        let data = include_bytes!("../../../../benchmarks/benches/crypto_bench/keys_query.json");
         let data: Value = serde_json::from_slice(data).unwrap();
         let data = response_from_file(&data);
-        get_keys::Response::try_from_http_response(data)
+        get_keys::v3::Response::try_from_http_response(data)
             .expect("Can't parse the keys upload response")
     }
 
-    fn keys_claim_response() -> claim_keys::Response {
-        let data = include_bytes!("../../benches/keys_claim.json");
+    fn keys_claim_response() -> claim_keys::v3::Response {
+        let data = include_bytes!("../../../../benchmarks/benches/crypto_bench/keys_claim.json");
         let data: Value = serde_json::from_slice(data).unwrap();
         let data = response_from_file(&data);
-        claim_keys::Response::try_from_http_response(data)
+        claim_keys::v3::Response::try_from_http_response(data)
             .expect("Can't parse the keys upload response")
     }
 
-    async fn machine() -> OlmMachine {
+    async fn machine_with_user(user_id: &UserId, device_id: &DeviceId) -> OlmMachine {
         let keys_query = keys_query_response();
         let keys_claim = keys_claim_response();
-        let uuid = Uuid::new_v4();
+        let txn_id = TransactionId::new();
 
-        let machine = OlmMachine::new(&alice_id(), &alice_device_id());
+        let machine = OlmMachine::new(user_id, device_id).await;
 
-        machine.mark_request_as_sent(&uuid, &keys_query).await.unwrap();
-        machine.mark_request_as_sent(&uuid, &keys_claim).await.unwrap();
+        machine.mark_request_as_sent(&txn_id, &keys_query).await.unwrap();
+        machine.mark_request_as_sent(&txn_id, &keys_claim).await.unwrap();
 
         machine
     }
 
-    #[tokio::test]
+    async fn machine() -> OlmMachine {
+        machine_with_user(alice_id(), alice_device_id()).await
+    }
+
+    #[async_test]
     async fn test_sharing() {
         let machine = machine().await;
         let room_id = room_id!("!test:localhost");
         let keys_claim = keys_claim_response();
 
-        let users: Vec<_> = keys_claim.one_time_keys.keys().collect();
+        let users = keys_claim.one_time_keys.keys().map(Deref::deref);
 
         let requests = machine
-            .share_group_session(&room_id, users.clone().into_iter(), EncryptionSettings::default())
+            .share_group_session(room_id, users, EncryptionSettings::default())
             .await
             .unwrap();
 
@@ -623,4 +662,37 @@ mod test {
         // that all 148 valid sessions get an room key.
         assert_eq!(event_count, 148);
     }
+
+    #[async_test]
+    async fn key_recipient_collecting() {
+        // The user id comes from the fact that the keys_query.json file uses
+        // this one.
+        let user_id = user_id!("@example:localhost");
+        let device_id = device_id!("TESTDEVICE");
+        let room_id = room_id!("!test:localhost");
+
+        let machine = machine_with_user(user_id, device_id).await;
+
+        let (outbound, _) = machine
+            .group_session_manager
+            .get_or_create_outbound_session(room_id, EncryptionSettings::default())
+            .await
+            .expect("We should be able to create a new session");
+        let history_visibility = HistoryVisibility::Joined;
+
+        let users = [user_id].into_iter();
+
+        let (_, recipients) = machine
+            .group_session_manager
+            .collect_session_recipients(users, history_visibility, &outbound)
+            .await
+            .expect("We should be able to collect the session recipients");
+
+        assert!(!recipients.is_empty());
+
+        // Make sure that our own device isn't part of the recipients.
+        assert!(!recipients[user_id]
+            .iter()
+            .any(|d| d.user_id() == user_id && d.device_id() == device_id));
+    }
 }
