@@ -12,19 +12,23 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::{collections::BTreeMap, sync::Arc, time::Duration};
+use std::{
+    collections::{BTreeMap, BTreeSet},
+    sync::Arc,
+    time::Duration,
+};
 
 use dashmap::{DashMap, DashSet};
-use matrix_sdk_common::uuid::Uuid;
 use ruma::{
-    api::client::r0::keys::claim_keys::{
+    api::client::keys::claim_keys::v3::{
         Request as KeysClaimRequest, Response as KeysClaimResponse,
     },
     assign,
-    events::{dummy::DummyToDeviceEventContent, AnyToDeviceEventContent},
-    DeviceId, DeviceIdBox, DeviceKeyAlgorithm, UserId,
+    events::{dummy::ToDeviceDummyEventContent, AnyToDeviceEventContent},
+    DeviceId, DeviceKeyAlgorithm, EventEncryptionAlgorithm, OwnedDeviceId, OwnedTransactionId,
+    OwnedUserId, SecondsSinceUnixEpoch, TransactionId, UserId,
 };
-use tracing::{error, info, warn};
+use tracing::{debug, error, info, warn};
 
 use crate::{
     error::OlmResult,
@@ -43,10 +47,10 @@ pub(crate) struct SessionManager {
     /// Submodules can insert user/device pairs into this map and the
     /// user/device paris will be added to the list of users when
     /// [`get_missing_sessions`](#method.get_missing_sessions) is called.
-    users_for_key_claim: Arc<DashMap<UserId, DashSet<DeviceIdBox>>>,
-    wedged_devices: Arc<DashMap<UserId, DashSet<DeviceIdBox>>>,
+    users_for_key_claim: Arc<DashMap<OwnedUserId, DashSet<OwnedDeviceId>>>,
+    wedged_devices: Arc<DashMap<OwnedUserId, DashSet<OwnedDeviceId>>>,
     key_request_machine: GossipMachine,
-    outgoing_to_device_requests: Arc<DashMap<Uuid, OutgoingRequest>>,
+    outgoing_to_device_requests: Arc<DashMap<OwnedTransactionId, OutgoingRequest>>,
 }
 
 impl SessionManager {
@@ -55,7 +59,7 @@ impl SessionManager {
 
     pub fn new(
         account: Account,
-        users_for_key_claim: Arc<DashMap<UserId, DashSet<DeviceIdBox>>>,
+        users_for_key_claim: Arc<DashMap<OwnedUserId, DashSet<OwnedDeviceId>>>,
         key_request_machine: GossipMachine,
         store: Store,
     ) -> Self {
@@ -70,7 +74,7 @@ impl SessionManager {
     }
 
     /// Mark the outgoing request as sent.
-    pub fn mark_outgoing_request_as_sent(&self, id: &Uuid) {
+    pub fn mark_outgoing_request_as_sent(&self, id: &TransactionId) {
         self.outgoing_to_device_requests.remove(id);
     }
 
@@ -80,19 +84,33 @@ impl SessionManager {
 
             if let Some(sessions) = sessions {
                 let mut sessions = sessions.lock().await;
-                sessions.sort_by_key(|s| s.creation_time.clone());
+                sessions.sort_by_key(|s| s.creation_time);
 
                 let session = sessions.get(0);
 
                 if let Some(session) = session {
-                    if session.creation_time.elapsed() > Self::UNWEDGING_INTERVAL {
+                    info!(
+                        sender = sender.as_str(),
+                        sender_key = curve_key,
+                        "Marking session to be unwedged"
+                    );
+
+                    let creation_time = Duration::from_secs(session.creation_time.get().into());
+                    let now = Duration::from_secs(SecondsSinceUnixEpoch::now().get().into());
+
+                    let should_unwedge = now
+                        .checked_sub(creation_time)
+                        .map(|elapsed| elapsed > Self::UNWEDGING_INTERVAL)
+                        .unwrap_or(true);
+
+                    if should_unwedge {
                         self.users_for_key_claim
-                            .entry(device.user_id().clone())
-                            .or_insert_with(DashSet::new)
+                            .entry(device.user_id().to_owned())
+                            .or_default()
                             .insert(device.device_id().into());
                         self.wedged_devices
                             .entry(device.user_id().to_owned())
-                            .or_insert_with(DashSet::new)
+                            .or_default()
                             .insert(device.device_id().into());
                     }
                 }
@@ -114,9 +132,9 @@ impl SessionManager {
     ///
     /// If the device was wedged this will queue up a dummy to-device message.
     async fn check_if_unwedged(&self, user_id: &UserId, device_id: &DeviceId) -> OlmResult<()> {
-        if self.wedged_devices.get(user_id).map(|d| d.remove(device_id)).flatten().is_some() {
+        if self.wedged_devices.get(user_id).and_then(|d| d.remove(device_id)).is_some() {
             if let Some(device) = self.store.get_device(user_id, device_id).await? {
-                let content = AnyToDeviceEventContent::Dummy(DummyToDeviceEventContent::new());
+                let content = AnyToDeviceEventContent::Dummy(ToDeviceDummyEventContent::new());
                 let (_, content) = device.encrypt(content).await?;
 
                 let request = ToDeviceRequest::new(
@@ -126,11 +144,11 @@ impl SessionManager {
                 );
 
                 let request = OutgoingRequest {
-                    request_id: request.txn_id,
+                    request_id: request.txn_id.clone(),
                     request: Arc::new(request.into()),
                 };
 
-                self.outgoing_to_device_requests.insert(request.request_id, request);
+                self.outgoing_to_device_requests.insert(request.request_id.clone(), request);
             }
         }
 
@@ -167,34 +185,46 @@ impl SessionManager {
     pub async fn get_missing_sessions(
         &self,
         users: impl Iterator<Item = &UserId>,
-    ) -> StoreResult<Option<(Uuid, KeysClaimRequest)>> {
-        let mut missing = BTreeMap::new();
+    ) -> StoreResult<Option<(OwnedTransactionId, KeysClaimRequest)>> {
+        let mut missing: BTreeMap<_, BTreeMap<_, _>> = BTreeMap::new();
 
         // Add the list of devices that the user wishes to establish sessions
         // right now.
         for user_id in users {
-            let user_devices = self.store.get_readonly_devices(user_id).await?;
+            let user_devices = self.store.get_readonly_devices_filtered(user_id).await?;
 
-            for (device_id, device) in user_devices.into_iter() {
-                let sender_key = if let Some(k) = device.get_key(DeviceKeyAlgorithm::Curve25519) {
-                    k
+            for (device_id, device) in user_devices {
+                if !device.algorithms().contains(&EventEncryptionAlgorithm::OlmV1Curve25519AesSha2)
+                {
+                    warn!(
+                        user_id = device.user_id().as_str(),
+                        device_id = device.device_id().as_str(),
+                        algorithms = ?device.algorithms(),
+                        "Device doesn't support any of our 1-to-1 E2EE \
+                        algorithms, can't establish an Olm session"
+                    );
+                } else if let Some(sender_key) = device.curve25519_key() {
+                    let sessions = self.store.get_sessions(&sender_key.to_base64()).await?;
+
+                    let is_missing = if let Some(sessions) = sessions {
+                        sessions.lock().await.is_empty()
+                    } else {
+                        true
+                    };
+
+                    if is_missing {
+                        missing
+                            .entry(user_id.to_owned())
+                            .or_default()
+                            .insert(device_id, DeviceKeyAlgorithm::SignedCurve25519);
+                    }
                 } else {
-                    continue;
-                };
-
-                let sessions = self.store.get_sessions(sender_key).await?;
-
-                let is_missing = if let Some(sessions) = sessions {
-                    sessions.lock().await.is_empty()
-                } else {
-                    true
-                };
-
-                if is_missing {
-                    missing
-                        .entry(user_id.to_owned())
-                        .or_insert_with(BTreeMap::new)
-                        .insert(device_id, DeviceKeyAlgorithm::SignedCurve25519);
+                    warn!(
+                        user_id = device.user_id().as_str(),
+                        device_id = device.device_id().as_str(),
+                        "Device doesn't have a valid Curve25519 key, \
+                        can't establish an Olm session"
+                    );
                 }
             }
         }
@@ -207,7 +237,7 @@ impl SessionManager {
             for device_id in item.value().iter() {
                 missing
                     .entry(user.to_owned())
-                    .or_insert_with(BTreeMap::new)
+                    .or_default()
                     .insert(device_id.to_owned(), DeviceKeyAlgorithm::SignedCurve25519);
             }
         }
@@ -215,8 +245,10 @@ impl SessionManager {
         if missing.is_empty() {
             Ok(None)
         } else {
+            debug!(?missing, "Collected user/device pairs that are missing an Olm session");
+
             Ok(Some((
-                Uuid::new_v4(),
+                TransactionId::new(),
                 assign!(KeysClaimRequest::new(missing), {
                     timeout: Some(Self::KEY_CLAIM_TIMEOUT),
                 }),
@@ -231,9 +263,10 @@ impl SessionManager {
     ///
     /// * `response` - The response containing the claimed one-time keys.
     pub async fn receive_keys_claim_response(&self, response: &KeysClaimResponse) -> OlmResult<()> {
-        // TODO log the failures here
+        debug!(failures = ?response.failures, "Received a /keys/claim response");
 
         let mut changes = Changes::default();
+        let mut new_sessions: BTreeMap<&UserId, BTreeSet<&DeviceId>> = BTreeMap::new();
 
         for (user_id, user_devices) in &response.one_time_keys {
             for (device_id, key_map) in user_devices {
@@ -241,33 +274,38 @@ impl SessionManager {
                     Ok(Some(d)) => d,
                     Ok(None) => {
                         warn!(
-                            "Tried to create an Olm session for {} {}, but the device is unknown",
-                            user_id, device_id
+                            user_id = user_id.as_str(),
+                            device_id = device_id.as_str(),
+                            "Tried to create an Olm session but the device is \
+                            unknown",
                         );
                         continue;
                     }
                     Err(e) => {
                         warn!(
-                            "Tried to create an Olm session for {} {}, but \
-                            can't fetch the device from the store {:?}",
-                            user_id, device_id, e
+                            user_id = user_id.as_str(),
+                            device_id = device_id.as_str(),
+                            error = ?e,
+                            "Tried to create an Olm session, but we can't \
+                            fetch the device from the store",
                         );
                         continue;
                     }
                 };
 
-                info!("Creating outbound Session for {} {}", user_id, device_id);
-
                 let session = match self.account.create_outbound_session(device, key_map).await {
                     Ok(s) => s,
                     Err(e) => {
-                        warn!("Error creating new outbound session {:?}", e);
+                        warn!(
+                            user_id = user_id.as_str(),
+                            device_id = device_id.as_str(),
+                            error = ?e,
+                            "Error creating outbound session"
+                        );
                         continue;
                     }
                 };
 
-                changes.sessions.push(session);
-
                 self.key_request_machine.retry_keyshare(user_id, device_id);
 
                 if let Err(e) = self.check_if_unwedged(user_id, device_id).await {
@@ -276,11 +314,14 @@ impl SessionManager {
                         user_id, device_id, e
                     );
                 }
+
+                changes.sessions.push(session);
+                new_sessions.entry(user_id).or_default().insert(device_id);
             }
         }
 
-        // TODO turn this into a single save_changes() call.
         self.store.save_changes(changes).await?;
+        info!(sessions = ?new_sessions, "Established new Olm sessions");
 
         match self.key_request_machine.collect_incoming_key_requests().await {
             Ok(sessions) => {
@@ -290,7 +331,7 @@ impl SessionManager {
             // We don't propagate the error here since the next sync will retry
             // this.
             Err(e) => {
-                warn!(error =? e, "Error while trying to collect the incoming secret requests")
+                warn!(error = ?e, "Error while trying to collect the incoming secret requests")
             }
         }
 
@@ -299,15 +340,15 @@ impl SessionManager {
 }
 
 #[cfg(test)]
-mod test {
-    use std::{collections::BTreeMap, sync::Arc};
+mod tests {
+    use std::{collections::BTreeMap, iter, sync::Arc};
 
     use dashmap::DashMap;
     use matrix_sdk_common::locks::Mutex;
     use matrix_sdk_test::async_test;
     use ruma::{
-        api::client::r0::keys::claim_keys::Response as KeyClaimResponse, user_id, DeviceIdBox,
-        UserId,
+        api::client::keys::claim_keys::v3::Response as KeyClaimResponse, device_id, user_id,
+        DeviceId, UserId,
     };
 
     use super::SessionManager;
@@ -320,16 +361,16 @@ mod test {
         verification::VerificationMachine,
     };
 
-    fn user_id() -> UserId {
+    fn user_id() -> &'static UserId {
         user_id!("@example:localhost")
     }
 
-    fn device_id() -> DeviceIdBox {
-        "DEVICEID".into()
+    fn device_id() -> &'static DeviceId {
+        device_id!("DEVICEID")
     }
 
     fn bob_account() -> ReadOnlyAccount {
-        ReadOnlyAccount::new(&user_id!("@bob:localhost"), "BOBDEVICE".into())
+        ReadOnlyAccount::new(user_id!("@bob:localhost"), device_id!("BOBDEVICE"))
     }
 
     async fn session_manager() -> SessionManager {
@@ -337,14 +378,14 @@ mod test {
         let device_id = device_id();
 
         let users_for_key_claim = Arc::new(DashMap::new());
-        let account = ReadOnlyAccount::new(&user_id, &device_id);
+        let account = ReadOnlyAccount::new(user_id, device_id);
         let store: Arc<dyn CryptoStore> = Arc::new(MemoryStore::new());
         store.save_account(account.clone()).await.unwrap();
-        let identity = Arc::new(Mutex::new(PrivateCrossSigningIdentity::empty(user_id.clone())));
+        let identity = Arc::new(Mutex::new(PrivateCrossSigningIdentity::empty(user_id)));
         let verification =
             VerificationMachine::new(account.clone(), identity.clone(), store.clone());
 
-        let user_id = Arc::new(user_id);
+        let user_id: Arc<UserId> = user_id.into();
         let device_id = device_id.into();
 
         let store = Store::new(user_id.clone(), identity, store, verification);
@@ -373,33 +414,27 @@ mod test {
 
         manager.store.save_devices(&[bob_device]).await.unwrap();
 
-        let (_, request) = manager
-            .get_missing_sessions(&mut [bob.user_id().clone()].iter())
-            .await
-            .unwrap()
-            .unwrap();
+        let (_, request) =
+            manager.get_missing_sessions(iter::once(bob.user_id())).await.unwrap().unwrap();
 
         assert!(request.one_time_keys.contains_key(bob.user_id()));
 
         bob.generate_one_time_keys_helper(1).await;
-        let one_time = bob.signed_one_time_keys_helper().await.unwrap();
+        let one_time = bob.signed_one_time_keys().await;
+        assert!(!one_time.is_empty());
         bob.mark_keys_as_published().await;
 
         let mut one_time_keys = BTreeMap::new();
         one_time_keys
-            .entry(bob.user_id().clone())
+            .entry(bob.user_id().to_owned())
             .or_insert_with(BTreeMap::new)
-            .insert(bob.device_id().into(), one_time);
+            .insert(bob.device_id().to_owned(), one_time);
 
         let response = KeyClaimResponse::new(one_time_keys);
 
         manager.receive_keys_claim_response(&response).await.unwrap();
 
-        assert!(manager
-            .get_missing_sessions(&mut [bob.user_id().clone()].iter())
-            .await
-            .unwrap()
-            .is_none());
+        assert!(manager.get_missing_sessions(iter::once(bob.user_id())).await.unwrap().is_none());
     }
 
     // This test doesn't run on macos because we're modifying the session
@@ -407,50 +442,45 @@ mod test {
     #[async_test]
     #[cfg(target_os = "linux")]
     async fn session_unwedging() {
-        use matrix_sdk_common::instant::{Duration, Instant};
-        use ruma::DeviceKeyAlgorithm;
+        use matrix_sdk_common::instant::{Duration, SystemTime};
+        use ruma::{DeviceKeyAlgorithm, SecondsSinceUnixEpoch};
 
         let manager = session_manager().await;
         let bob = bob_account();
         let (_, mut session) = bob.create_session_for(&manager.account).await;
 
         let bob_device = ReadOnlyDevice::from_account(&bob).await;
-        session.creation_time = Arc::new(Instant::now() - Duration::from_secs(3601));
+        let time = SystemTime::now() - Duration::from_secs(3601);
+        session.creation_time = SecondsSinceUnixEpoch::from_system_time(time).unwrap();
 
         manager.store.save_devices(&[bob_device.clone()]).await.unwrap();
         manager.store.save_sessions(&[session]).await.unwrap();
 
-        assert!(manager
-            .get_missing_sessions(&mut [bob.user_id().clone()].iter())
-            .await
-            .unwrap()
-            .is_none());
+        assert!(manager.get_missing_sessions(iter::once(bob.user_id())).await.unwrap().is_none());
 
         let curve_key = bob_device.get_key(DeviceKeyAlgorithm::Curve25519).unwrap();
 
         assert!(!manager.users_for_key_claim.contains_key(bob.user_id()));
         assert!(!manager.is_device_wedged(&bob_device));
-        manager.mark_device_as_wedged(bob_device.user_id(), curve_key).await.unwrap();
+        manager.mark_device_as_wedged(bob_device.user_id(), &curve_key.to_base64()).await.unwrap();
         assert!(manager.is_device_wedged(&bob_device));
         assert!(manager.users_for_key_claim.contains_key(bob.user_id()));
 
-        let (_, request) = manager
-            .get_missing_sessions(&mut [bob.user_id().clone()].iter())
-            .await
-            .unwrap()
-            .unwrap();
+        let (_, request) =
+            manager.get_missing_sessions(iter::once(bob.user_id())).await.unwrap().unwrap();
 
         assert!(request.one_time_keys.contains_key(bob.user_id()));
 
         bob.generate_one_time_keys_helper(1).await;
-        let one_time = bob.signed_one_time_keys_helper().await.unwrap();
+        let one_time = bob.signed_one_time_keys().await;
+        assert!(!one_time.is_empty());
         bob.mark_keys_as_published().await;
 
         let mut one_time_keys = BTreeMap::new();
         one_time_keys
-            .entry(bob.user_id().clone())
+            .entry(bob.user_id().to_owned())
             .or_insert_with(BTreeMap::new)
-            .insert(bob.device_id().into(), one_time);
+            .insert(bob.device_id().to_owned(), one_time);
 
         let response = KeyClaimResponse::new(one_time_keys);
 
@@ -459,11 +489,7 @@ mod test {
         manager.receive_keys_claim_response(&response).await.unwrap();
 
         assert!(!manager.is_device_wedged(&bob_device));
-        assert!(manager
-            .get_missing_sessions(&mut [bob.user_id().clone()].iter())
-            .await
-            .unwrap()
-            .is_none());
+        assert!(manager.get_missing_sessions(iter::once(bob.user_id())).await.unwrap().is_none());
         assert!(!manager.outgoing_to_device_requests.is_empty())
     }
 }
