@@ -21,7 +21,7 @@ use std::{collections::HashMap, sync::Arc};
 
 use dashmap::DashMap;
 use matrix_sdk_common::locks::Mutex;
-use ruma::{DeviceId, DeviceIdBox, RoomId, UserId};
+use ruma::{DeviceId, OwnedDeviceId, OwnedRoomId, OwnedUserId, RoomId, UserId};
 
 use crate::{
     identities::ReadOnlyDevice,
@@ -45,11 +45,7 @@ impl SessionStore {
     /// Returns true if the session was added, false if the session was
     /// already in the store.
     pub async fn add(&self, session: Session) -> bool {
-        let sessions_lock = self
-            .entries
-            .entry(session.sender_key.to_string())
-            .or_insert_with(|| Arc::new(Mutex::new(Vec::new())));
-
+        let sessions_lock = self.entries.entry(session.sender_key.to_base64()).or_default();
         let mut sessions = sessions_lock.lock().await;
 
         if !sessions.contains(&session) {
@@ -62,7 +58,6 @@ impl SessionStore {
 
     /// Get all the sessions that belong to the given sender key.
     pub fn get(&self, sender_key: &str) -> Option<Arc<Mutex<Vec<Session>>>> {
-        #[allow(clippy::map_clone)]
         self.entries.get(sender_key).map(|s| s.clone())
     }
 
@@ -76,7 +71,7 @@ impl SessionStore {
 /// In-memory store that holds inbound group sessions.
 pub struct GroupSessionStore {
     #[allow(clippy::type_complexity)]
-    entries: Arc<DashMap<RoomId, HashMap<String, HashMap<String, InboundGroupSession>>>>,
+    entries: Arc<DashMap<OwnedRoomId, HashMap<String, HashMap<String, InboundGroupSession>>>>,
 }
 
 impl GroupSessionStore {
@@ -91,10 +86,10 @@ impl GroupSessionStore {
     /// already in the store.
     pub fn add(&self, session: InboundGroupSession) -> bool {
         self.entries
-            .entry((&*session.room_id).clone())
-            .or_insert_with(HashMap::new)
+            .entry((*session.room_id).to_owned())
+            .or_default()
             .entry(session.sender_key.to_string())
-            .or_insert_with(HashMap::new)
+            .or_default()
             .insert(session.session_id().to_owned(), session)
             .is_none()
     }
@@ -112,6 +107,11 @@ impl GroupSessionStore {
             .collect()
     }
 
+    /// Get the number of `InboundGroupSession`s we have.
+    pub fn count(&self) -> usize {
+        self.entries.iter().map(|d| d.value().values().map(|t| t.len()).sum::<usize>()).sum()
+    }
+
     /// Get a inbound group session from our store.
     ///
     /// # Arguments
@@ -135,7 +135,7 @@ impl GroupSessionStore {
 /// In-memory store holding the devices of users.
 #[derive(Clone, Debug, Default)]
 pub struct DeviceStore {
-    entries: Arc<DashMap<UserId, DashMap<Box<DeviceId>, ReadOnlyDevice>>>,
+    entries: Arc<DashMap<OwnedUserId, DashMap<OwnedDeviceId, ReadOnlyDevice>>>,
 }
 
 impl DeviceStore {
@@ -151,7 +151,7 @@ impl DeviceStore {
         let user_id = device.user_id();
         self.entries
             .entry(user_id.to_owned())
-            .or_insert_with(DashMap::new)
+            .or_default()
             .insert(device.device_id().into(), device)
             .is_none()
     }
@@ -170,10 +170,10 @@ impl DeviceStore {
     }
 
     /// Get a read-only view over all devices of the given user.
-    pub fn user_devices(&self, user_id: &UserId) -> HashMap<DeviceIdBox, ReadOnlyDevice> {
+    pub fn user_devices(&self, user_id: &UserId) -> HashMap<OwnedDeviceId, ReadOnlyDevice> {
         self.entries
-            .entry(user_id.clone())
-            .or_insert_with(DashMap::new)
+            .entry(user_id.to_owned())
+            .or_default()
             .iter()
             .map(|i| (i.key().to_owned(), i.value().clone()))
             .collect()
@@ -181,16 +181,17 @@ impl DeviceStore {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
+    use matrix_sdk_test::async_test;
     use ruma::room_id;
 
     use crate::{
-        identities::device::test::get_device,
-        olm::{test::get_account_and_session, InboundGroupSession},
+        identities::device::testing::get_device,
+        olm::{tests::get_account_and_session, InboundGroupSession},
         store::caches::{DeviceStore, GroupSessionStore, SessionStore},
     };
 
-    #[tokio::test]
+    #[async_test]
     async fn test_session_store() {
         let (_, session) = get_account_and_session().await;
 
@@ -199,7 +200,7 @@ mod test {
         assert!(store.add(session.clone()).await);
         assert!(!store.add(session.clone()).await);
 
-        let sessions = store.get(&session.sender_key).unwrap();
+        let sessions = store.get(&session.sender_key.to_base64()).unwrap();
         let sessions = sessions.lock().await;
 
         let loaded_session = &sessions[0];
@@ -207,14 +208,14 @@ mod test {
         assert_eq!(&session, loaded_session);
     }
 
-    #[tokio::test]
+    #[async_test]
     async fn test_session_store_bulk_storing() {
         let (_, session) = get_account_and_session().await;
 
         let store = SessionStore::new();
-        store.set_for_sender(&session.sender_key, vec![session.clone()]);
+        store.set_for_sender(&session.sender_key.to_base64(), vec![session.clone()]);
 
-        let sessions = store.get(&session.sender_key).unwrap();
+        let sessions = store.get(&session.sender_key.to_base64()).unwrap();
         let sessions = sessions.lock().await;
 
         let loaded_session = &sessions[0];
@@ -222,13 +223,12 @@ mod test {
         assert_eq!(&session, loaded_session);
     }
 
-    #[tokio::test]
+    #[async_test]
     async fn test_group_session_store() {
         let (account, _) = get_account_and_session().await;
         let room_id = room_id!("!test:localhost");
 
-        let (outbound, _) =
-            account.create_group_session_pair_with_defaults(&room_id).await.unwrap();
+        let (outbound, _) = account.create_group_session_pair_with_defaults(room_id).await;
 
         assert_eq!(0, outbound.message_index().await);
         assert!(!outbound.shared());
@@ -238,20 +238,19 @@ mod test {
         let inbound = InboundGroupSession::new(
             "test_key",
             "test_key",
-            &room_id,
+            room_id,
             outbound.session_key().await,
             None,
-        )
-        .unwrap();
+        );
 
         let store = GroupSessionStore::new();
         store.add(inbound.clone());
 
-        let loaded_session = store.get(&room_id, "test_key", outbound.session_id()).unwrap();
+        let loaded_session = store.get(room_id, "test_key", outbound.session_id()).unwrap();
         assert_eq!(inbound, loaded_session);
     }
 
-    #[tokio::test]
+    #[async_test]
     async fn test_device_store() {
         let device = get_device();
         let store = DeviceStore::new();
