@@ -17,13 +17,17 @@ use std::{
     sync::Arc,
 };
 
+use async_trait::async_trait;
 use dashmap::{DashMap, DashSet};
-use matrix_sdk_common::{async_trait, locks::Mutex, uuid::Uuid};
-use ruma::{DeviceId, DeviceIdBox, RoomId, UserId};
+use matrix_sdk_common::locks::Mutex;
+use ruma::{
+    DeviceId, OwnedDeviceId, OwnedTransactionId, OwnedUserId, RoomId, TransactionId, UserId,
+};
 
 use super::{
     caches::{DeviceStore, GroupSessionStore, SessionStore},
-    Changes, CryptoStore, InboundGroupSession, ReadOnlyAccount, Result, Session,
+    BackupKeys, Changes, CryptoStore, InboundGroupSession, ReadOnlyAccount, Result, RoomKeyCounts,
+    Session,
 };
 use crate::{
     gossiping::{GossipRequest, SecretInfo},
@@ -45,13 +49,13 @@ fn encode_key_info(info: &SecretInfo) -> String {
 pub struct MemoryStore {
     sessions: SessionStore,
     inbound_group_sessions: GroupSessionStore,
-    tracked_users: Arc<DashSet<UserId>>,
-    users_for_key_query: Arc<DashSet<UserId>>,
+    tracked_users: Arc<DashSet<OwnedUserId>>,
+    users_for_key_query: Arc<DashSet<OwnedUserId>>,
     olm_hashes: Arc<DashMap<String, DashSet<String>>>,
     devices: DeviceStore,
-    identities: Arc<DashMap<UserId, ReadOnlyUserIdentities>>,
-    outgoing_key_requests: Arc<DashMap<Uuid, GossipRequest>>,
-    key_requests_by_info: Arc<DashMap<String, Uuid>>,
+    identities: Arc<DashMap<OwnedUserId, ReadOnlyUserIdentities>>,
+    outgoing_key_requests: Arc<DashMap<OwnedTransactionId, GossipRequest>>,
+    key_requests_by_info: Arc<DashMap<String, OwnedTransactionId>>,
 }
 
 impl Default for MemoryStore {
@@ -76,26 +80,26 @@ impl MemoryStore {
         Self::default()
     }
 
-    pub(crate) async fn save_devices(&self, mut devices: Vec<ReadOnlyDevice>) {
-        for device in devices.drain(..) {
+    pub(crate) async fn save_devices(&self, devices: Vec<ReadOnlyDevice>) {
+        for device in devices {
             let _ = self.devices.add(device);
         }
     }
 
-    async fn delete_devices(&self, mut devices: Vec<ReadOnlyDevice>) {
-        for device in devices.drain(..) {
+    async fn delete_devices(&self, devices: Vec<ReadOnlyDevice>) {
+        for device in devices {
             let _ = self.devices.remove(device.user_id(), device.device_id());
         }
     }
 
-    async fn save_sessions(&self, mut sessions: Vec<Session>) {
-        for session in sessions.drain(..) {
+    async fn save_sessions(&self, sessions: Vec<Session>) {
+        for session in sessions {
             let _ = self.sessions.add(session.clone()).await;
         }
     }
 
-    async fn save_inbound_group_sessions(&self, mut sessions: Vec<InboundGroupSession>) {
-        for session in sessions.drain(..) {
+    async fn save_inbound_group_sessions(&self, sessions: Vec<InboundGroupSession>) {
+        for session in sessions {
             self.inbound_group_sessions.add(session);
         }
     }
@@ -116,7 +120,7 @@ impl CryptoStore for MemoryStore {
         Ok(None)
     }
 
-    async fn save_changes(&self, mut changes: Changes) -> Result<()> {
+    async fn save_changes(&self, changes: Changes) -> Result<()> {
         self.save_sessions(changes.sessions).await;
         self.save_inbound_group_sessions(changes.inbound_group_sessions).await;
 
@@ -124,22 +128,22 @@ impl CryptoStore for MemoryStore {
         self.save_devices(changes.devices.changed).await;
         self.delete_devices(changes.devices.deleted).await;
 
-        for identity in changes.identities.new.drain(..).chain(changes.identities.changed) {
+        for identity in changes.identities.new.into_iter().chain(changes.identities.changed) {
             let _ = self.identities.insert(identity.user_id().to_owned(), identity.clone());
         }
 
         for hash in changes.message_hashes {
             self.olm_hashes
                 .entry(hash.sender_key.to_owned())
-                .or_insert_with(DashSet::new)
+                .or_default()
                 .insert(hash.hash.clone());
         }
 
         for key_request in changes.key_requests {
-            let id = key_request.request_id;
+            let id = key_request.request_id.clone();
             let info_string = encode_key_info(&key_request.info);
 
-            self.outgoing_key_requests.insert(id, key_request);
+            self.outgoing_key_requests.insert(id.clone(), key_request);
             self.key_requests_by_info.insert(info_string, id);
         }
 
@@ -163,6 +167,34 @@ impl CryptoStore for MemoryStore {
         Ok(self.inbound_group_sessions.get_all())
     }
 
+    async fn inbound_group_session_counts(&self) -> Result<RoomKeyCounts> {
+        let backed_up =
+            self.get_inbound_group_sessions().await?.into_iter().filter(|s| s.backed_up()).count();
+
+        Ok(RoomKeyCounts { total: self.inbound_group_sessions.count(), backed_up })
+    }
+
+    async fn inbound_group_sessions_for_backup(
+        &self,
+        limit: usize,
+    ) -> Result<Vec<InboundGroupSession>> {
+        Ok(self
+            .get_inbound_group_sessions()
+            .await?
+            .into_iter()
+            .filter(|s| !s.backed_up())
+            .take(limit)
+            .collect())
+    }
+
+    async fn reset_backup_state(&self) -> Result<()> {
+        for session in self.get_inbound_group_sessions().await? {
+            session.reset_backup_state();
+        }
+
+        Ok(())
+    }
+
     async fn get_outbound_group_sessions(
         &self,
         _: &RoomId,
@@ -178,12 +210,11 @@ impl CryptoStore for MemoryStore {
         !self.users_for_key_query.is_empty()
     }
 
-    fn users_for_key_query(&self) -> HashSet<UserId> {
-        #[allow(clippy::map_clone)]
+    fn users_for_key_query(&self) -> HashSet<OwnedUserId> {
         self.users_for_key_query.iter().map(|u| u.clone()).collect()
     }
 
-    fn tracked_users(&self) -> HashSet<UserId> {
+    fn tracked_users(&self) -> HashSet<OwnedUserId> {
         self.tracked_users.iter().map(|u| u.to_owned()).collect()
     }
 
@@ -201,12 +232,12 @@ impl CryptoStore for MemoryStore {
         // The counter would top out at 2 since there won't be a race between 3
         // different key queries syncs.
         if dirty {
-            self.users_for_key_query.insert(user.clone());
+            self.users_for_key_query.insert(user.to_owned());
         } else {
             self.users_for_key_query.remove(user);
         }
 
-        Ok(self.tracked_users.insert(user.clone()))
+        Ok(self.tracked_users.insert(user.to_owned()))
     }
 
     async fn get_device(
@@ -220,12 +251,11 @@ impl CryptoStore for MemoryStore {
     async fn get_user_devices(
         &self,
         user_id: &UserId,
-    ) -> Result<HashMap<DeviceIdBox, ReadOnlyDevice>> {
+    ) -> Result<HashMap<OwnedDeviceId, ReadOnlyDevice>> {
         Ok(self.devices.user_devices(user_id))
     }
 
     async fn get_user_identity(&self, user_id: &UserId) -> Result<Option<ReadOnlyUserIdentities>> {
-        #[allow(clippy::map_clone)]
         Ok(self.identities.get(user_id).map(|i| i.clone()))
     }
 
@@ -233,15 +263,15 @@ impl CryptoStore for MemoryStore {
         Ok(self
             .olm_hashes
             .entry(message_hash.sender_key.to_owned())
-            .or_insert_with(DashSet::new)
+            .or_default()
             .contains(&message_hash.hash))
     }
 
     async fn get_outgoing_secret_requests(
         &self,
-        request_id: Uuid,
+        request_id: &TransactionId,
     ) -> Result<Option<GossipRequest>> {
-        Ok(self.outgoing_key_requests.get(&request_id).map(|r| r.clone()))
+        Ok(self.outgoing_key_requests.get(request_id).map(|r| r.clone()))
     }
 
     async fn get_secret_request_by_info(
@@ -253,7 +283,7 @@ impl CryptoStore for MemoryStore {
         Ok(self
             .key_requests_by_info
             .get(&key_info_string)
-            .and_then(|i| self.outgoing_key_requests.get(&i).map(|r| r.clone())))
+            .and_then(|i| self.outgoing_key_requests.get(&*i).map(|r| r.clone())))
     }
 
     async fn get_unsent_secret_requests(&self) -> Result<Vec<GossipRequest>> {
@@ -265,27 +295,32 @@ impl CryptoStore for MemoryStore {
             .collect())
     }
 
-    async fn delete_outgoing_secret_requests(&self, request_id: Uuid) -> Result<()> {
-        self.outgoing_key_requests.remove(&request_id).and_then(|(_, i)| {
+    async fn delete_outgoing_secret_requests(&self, request_id: &TransactionId) -> Result<()> {
+        self.outgoing_key_requests.remove(request_id).and_then(|(_, i)| {
             let key_info_string = encode_key_info(&i.info);
             self.key_requests_by_info.remove(&key_info_string)
         });
 
         Ok(())
     }
+
+    async fn load_backup_keys(&self) -> Result<BackupKeys> {
+        Ok(BackupKeys::default())
+    }
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
+    use matrix_sdk_test::async_test;
     use ruma::room_id;
 
     use crate::{
-        identities::device::test::get_device,
-        olm::{test::get_account_and_session, InboundGroupSession, OlmMessageHash},
+        identities::device::testing::get_device,
+        olm::{tests::get_account_and_session, InboundGroupSession, OlmMessageHash},
         store::{memorystore::MemoryStore, Changes, CryptoStore},
     };
 
-    #[tokio::test]
+    #[async_test]
     async fn test_session_store() {
         let (account, session) = get_account_and_session().await;
         let store = MemoryStore::new();
@@ -295,7 +330,7 @@ mod test {
 
         store.save_sessions(vec![session.clone()]).await;
 
-        let sessions = store.get_sessions(&session.sender_key).await.unwrap().unwrap();
+        let sessions = store.get_sessions(&session.sender_key.to_base64()).await.unwrap().unwrap();
         let sessions = sessions.lock().await;
 
         let loaded_session = &sessions[0];
@@ -303,34 +338,32 @@ mod test {
         assert_eq!(&session, loaded_session);
     }
 
-    #[tokio::test]
+    #[async_test]
     async fn test_group_session_store() {
         let (account, _) = get_account_and_session().await;
         let room_id = room_id!("!test:localhost");
 
-        let (outbound, _) =
-            account.create_group_session_pair_with_defaults(&room_id).await.unwrap();
+        let (outbound, _) = account.create_group_session_pair_with_defaults(room_id).await;
         let inbound = InboundGroupSession::new(
             "test_key",
             "test_key",
-            &room_id,
+            room_id,
             outbound.session_key().await,
             None,
-        )
-        .unwrap();
+        );
 
         let store = MemoryStore::new();
-        let _ = store.save_inbound_group_sessions(vec![inbound.clone()]).await;
+        store.save_inbound_group_sessions(vec![inbound.clone()]).await;
 
         let loaded_session = store
-            .get_inbound_group_session(&room_id, "test_key", outbound.session_id())
+            .get_inbound_group_session(room_id, "test_key", outbound.session_id())
             .await
             .unwrap()
             .unwrap();
         assert_eq!(inbound, loaded_session);
     }
 
-    #[tokio::test]
+    #[async_test]
     async fn test_device_store() {
         let device = get_device();
         let store = MemoryStore::new();
@@ -355,7 +388,7 @@ mod test {
         assert!(store.get_device(device.user_id(), device.device_id()).await.unwrap().is_none());
     }
 
-    #[tokio::test]
+    #[async_test]
     async fn test_tracked_users() {
         let device = get_device();
         let store = MemoryStore::new();
@@ -366,7 +399,7 @@ mod test {
         assert!(store.is_user_tracked(device.user_id()));
     }
 
-    #[tokio::test]
+    #[async_test]
     async fn test_message_hash() {
         let store = MemoryStore::new();
 
