@@ -26,12 +26,12 @@
 //! #     OlmMachine,
 //! #     store::MemoryStore,
 //! # };
-//! # use ruma::{user_id, DeviceIdBox};
+//! # use ruma::{device_id, user_id};
 //! # let user_id = user_id!("@example:localhost");
-//! # let device_id: DeviceIdBox = "TEST".into();
+//! # let device_id = device_id!("TEST");
 //! let store = Box::new(MemoryStore::new());
 //!
-//! let machine = OlmMachine::new_with_store(user_id, device_id, store);
+//! let machine = OlmMachine::with_store(user_id, device_id, store);
 //! ```
 //!
 //! [`OlmMachine`]: /matrix_sdk_crypto/struct.OlmMachine.html
@@ -39,9 +39,11 @@
 
 pub mod caches;
 mod memorystore;
-mod pickle_key;
-#[cfg(feature = "sled_cryptostore")]
-pub(crate) mod sled;
+
+#[cfg(any(test, feature = "testing"))]
+#[macro_use]
+#[allow(missing_docs)]
+pub mod integration_tests;
 
 use std::{
     collections::{HashMap, HashSet},
@@ -51,25 +53,21 @@ use std::{
     sync::Arc,
 };
 
-use base64::DecodeError;
-use matrix_sdk_common::{async_trait, locks::Mutex, uuid::Uuid, AsyncTraitDeps};
+use async_trait::async_trait;
+use matrix_sdk_common::{locks::Mutex, AsyncTraitDeps};
 pub use memorystore::MemoryStore;
-use olm_rs::errors::{OlmAccountError, OlmGroupSessionError, OlmSessionError};
-pub use pickle_key::{EncryptedPickleKey, PickleKey};
 use ruma::{
-    events::secret::request::SecretName, identifiers::Error as IdentifierValidationError, DeviceId,
-    DeviceIdBox, DeviceKeyAlgorithm, RoomId, UserId,
+    events::secret::request::SecretName, DeviceId, IdParseError, OwnedDeviceId, OwnedUserId,
+    RoomId, TransactionId, UserId,
 };
+use serde::{Deserialize, Serialize};
 use serde_json::Error as SerdeError;
 use thiserror::Error;
 use tracing::{info, warn};
+use vodozemac::Curve25519PublicKey;
 use zeroize::Zeroize;
 
-#[cfg(feature = "sled_cryptostore")]
-pub use self::sled::SledStore;
 use crate::{
-    error::SessionUnpicklingError,
-    gossiping::{GossipRequest, SecretInfo},
     identities::{
         user::{OwnUserIdentity, UserIdentities, UserIdentity},
         Device, ReadOnlyDevice, ReadOnlyUserIdentities, UserDevices,
@@ -78,6 +76,7 @@ use crate::{
         InboundGroupSession, OlmMessageHash, OutboundGroupSession, PrivateCrossSigningIdentity,
         ReadOnlyAccount, Session,
     },
+    utilities::encode,
     verification::VerificationMachine,
     CrossSigningStatus,
 };
@@ -85,6 +84,8 @@ use crate::{
 /// A `CryptoStore` specific result type.
 pub type Result<T, E = CryptoStoreError> = std::result::Result<T, E>;
 
+pub use crate::gossiping::{GossipRequest, SecretInfo};
+
 /// A wrapper for our CryptoStore trait object.
 ///
 /// This is needed because we want to have a generic interface so we can
@@ -92,27 +93,44 @@ pub type Result<T, E = CryptoStoreError> = std::result::Result<T, E>;
 /// generics don't mix let the CryptoStore store strings and this wrapper
 /// adds the generic interface on top.
 #[derive(Debug, Clone)]
-pub(crate) struct Store {
+pub struct Store {
     user_id: Arc<UserId>,
     identity: Arc<Mutex<PrivateCrossSigningIdentity>>,
     inner: Arc<dyn CryptoStore>,
     verification_machine: VerificationMachine,
 }
 
-#[derive(Clone, Debug, Default)]
+#[derive(Default, Debug)]
 #[allow(missing_docs)]
 pub struct Changes {
     pub account: Option<ReadOnlyAccount>,
     pub private_identity: Option<PrivateCrossSigningIdentity>,
+    pub backup_version: Option<String>,
+    pub recovery_key: Option<RecoveryKey>,
     pub sessions: Vec<Session>,
     pub message_hashes: Vec<OlmMessageHash>,
     pub inbound_group_sessions: Vec<InboundGroupSession>,
     pub outbound_group_sessions: Vec<OutboundGroupSession>,
-    pub identities: IdentityChanges,
     pub key_requests: Vec<GossipRequest>,
+    pub identities: IdentityChanges,
     pub devices: DeviceChanges,
 }
 
+impl Changes {
+    /// Are there any changes stored or is this an empty `Changes` struct
+    pub fn is_empty(&self) -> bool {
+        self.account.is_none()
+            && self.private_identity.is_none()
+            && self.sessions.is_empty()
+            && self.message_hashes.is_empty()
+            && self.inbound_group_sessions.is_empty()
+            && self.outbound_group_sessions.is_empty()
+            && self.key_requests.is_empty()
+            && self.identities.is_empty()
+            && self.devices.is_empty()
+    }
+}
+
 #[derive(Debug, Clone, Default)]
 #[allow(missing_docs)]
 pub struct IdentityChanges {
@@ -120,6 +138,12 @@ pub struct IdentityChanges {
     pub changed: Vec<ReadOnlyUserIdentities>,
 }
 
+impl IdentityChanges {
+    fn is_empty(&self) -> bool {
+        self.new.is_empty() && self.changed.is_empty()
+    }
+}
+
 #[derive(Debug, Clone, Default)]
 #[allow(missing_docs)]
 pub struct DeviceChanges {
@@ -128,6 +152,40 @@ pub struct DeviceChanges {
     pub deleted: Vec<ReadOnlyDevice>,
 }
 
+/// The private part of a backup key.
+#[derive(Zeroize, Deserialize, Serialize)]
+#[zeroize(drop)]
+#[serde(transparent)]
+pub struct RecoveryKey {
+    pub(crate) inner: Box<[u8; RecoveryKey::KEY_SIZE]>,
+}
+
+impl RecoveryKey {
+    /// The number of bytes the recovery key will hold.
+    pub const KEY_SIZE: usize = 32;
+
+    /// Create a new random recovery key.
+    pub fn new() -> Result<Self, rand::Error> {
+        let mut rng = rand::thread_rng();
+
+        let mut key = Box::new([0u8; Self::KEY_SIZE]);
+        rand::Fill::try_fill(key.as_mut_slice(), &mut rng)?;
+
+        Ok(Self { inner: key })
+    }
+
+    /// Export the `RecoveryKey` as a base64 encoded string.
+    pub fn to_base64(&self) -> String {
+        encode(self.inner.as_slice())
+    }
+}
+
+impl Debug for RecoveryKey {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("RecoveryKey").finish()
+    }
+}
+
 impl DeviceChanges {
     /// Merge the given `DeviceChanges` into this instance of `DeviceChanges`.
     pub fn extend(&mut self, other: DeviceChanges) {
@@ -135,6 +193,28 @@ impl DeviceChanges {
         self.changed.extend(other.changed);
         self.deleted.extend(other.deleted);
     }
+
+    fn is_empty(&self) -> bool {
+        self.new.is_empty() && self.changed.is_empty() && self.deleted.is_empty()
+    }
+}
+
+/// Struct holding info about how many room keys the store has.
+#[derive(Debug, Clone, Default)]
+pub struct RoomKeyCounts {
+    /// The total number of room keys the store has.
+    pub total: usize,
+    /// The number of backed up room keys the store has.
+    pub backed_up: usize,
+}
+
+/// Stored versions of the backup keys.
+#[derive(Default, Debug)]
+pub struct BackupKeys {
+    /// The recovery key, the one used to decrypt backed up room keys.
+    pub recovery_key: Option<RecoveryKey>,
+    /// The version that we are using for backups.
+    pub backup_version: Option<String>,
 }
 
 /// A struct containing private cross signing keys that can be backed up or
@@ -164,22 +244,23 @@ impl Debug for CrossSigningKeyExport {
 /// or the key backup key.
 #[derive(Debug, Error)]
 pub enum SecretImportError {
-    /// The seed for the private key wasn't valid base64.
+    /// The key that we tried to import was invalid.
     #[error(transparent)]
-    Base64(#[from] DecodeError),
+    Key(#[from] vodozemac::KeyError),
     /// The public key of the imported private key doesn't match to the public
     /// key that was uploaded to the server.
     #[error(
         "The public key of the imported private key doesn't match to the \
             public key that was uploaded to the server"
     )]
-    MissmatchedPublicKeys,
+    MismatchedPublicKeys,
     /// The new version of the identity couldn't be stored.
     #[error(transparent)]
     Store(#[from] CryptoStoreError),
 }
 
 impl Store {
+    /// Create a new Store
     pub fn new(
         user_id: Arc<UserId>,
         identity: Arc<Mutex<PrivateCrossSigningIdentity>>,
@@ -189,23 +270,33 @@ impl Store {
         Self { user_id, identity, inner: store, verification_machine }
     }
 
+    /// UserId associated with this store
     pub fn user_id(&self) -> &UserId {
         &self.user_id
     }
 
+    /// DeviceId associated with this store
     pub fn device_id(&self) -> &DeviceId {
         self.verification_machine.own_device_id()
     }
 
+    /// The Account associated with this store
+    pub fn account(&self) -> &ReadOnlyAccount {
+        &self.verification_machine.store.account
+    }
+
     #[cfg(test)]
+    /// test helper to reset the cross signing identity
     pub async fn reset_cross_signing_identity(&self) {
         self.identity.lock().await.reset().await;
     }
 
+    ///  PrivateCrossSigningIdentity associated with this store
     pub fn private_identity(&self) -> Arc<Mutex<PrivateCrossSigningIdentity>> {
         self.identity.clone()
     }
 
+    /// Save the given Sessions to the store
     pub async fn save_sessions(&self, sessions: &[Session]) -> Result<()> {
         let changes = Changes { sessions: sessions.to_vec(), ..Default::default() };
 
@@ -213,6 +304,7 @@ impl Store {
     }
 
     #[cfg(test)]
+    /// Testing helper to allo to save only a set of devices
     pub async fn save_devices(&self, devices: &[ReadOnlyDevice]) -> Result<()> {
         let changes = Changes {
             devices: DeviceChanges { changed: devices.to_vec(), ..Default::default() },
@@ -223,6 +315,7 @@ impl Store {
     }
 
     #[cfg(test)]
+    /// Testing helper to allo to save only a set of InboundGroupSession
     pub async fn save_inbound_group_sessions(
         &self,
         sessions: &[InboundGroupSession],
@@ -238,31 +331,25 @@ impl Store {
             .inner
             .get_device(self.user_id(), self.device_id())
             .await?
-            .and_then(|d| d.display_name().to_owned()))
+            .and_then(|d| d.display_name().map(|d| d.to_owned())))
     }
 
-    /// Get the read-only version of all the devices that the given user has.
-    ///
-    /// *Note*: This doesn't return our own device.
+    /// Get the read-only device associated with `device_id` for `user_id`
     pub async fn get_readonly_device(
         &self,
         user_id: &UserId,
         device_id: &DeviceId,
     ) -> Result<Option<ReadOnlyDevice>> {
-        if user_id == self.user_id() && device_id == self.device_id() {
-            Ok(None)
-        } else {
-            self.inner.get_device(user_id, device_id).await
-        }
+        self.inner.get_device(user_id, device_id).await
     }
 
     /// Get the read-only version of all the devices that the given user has.
     ///
     /// *Note*: This doesn't return our own device.
-    pub async fn get_readonly_devices(
+    pub async fn get_readonly_devices_filtered(
         &self,
         user_id: &UserId,
-    ) -> Result<HashMap<DeviceIdBox, ReadOnlyDevice>> {
+    ) -> Result<HashMap<OwnedDeviceId, ReadOnlyDevice>> {
         self.inner.get_user_devices(user_id).await.map(|mut d| {
             if user_id == self.user_id() {
                 d.remove(self.device_id());
@@ -277,7 +364,7 @@ impl Store {
     pub async fn get_readonly_devices_unfiltered(
         &self,
         user_id: &UserId,
-    ) -> Result<HashMap<DeviceIdBox, ReadOnlyDevice>> {
+    ) -> Result<HashMap<OwnedDeviceId, ReadOnlyDevice>> {
         self.inner.get_user_devices(user_id).await
     }
 
@@ -289,23 +376,36 @@ impl Store {
         user_id: &UserId,
         curve_key: &str,
     ) -> Result<Option<Device>> {
-        self.get_user_devices(user_id).await.map(|d| {
-            d.devices().find(|d| {
-                d.get_key(DeviceKeyAlgorithm::Curve25519).map_or(false, |k| k == curve_key)
-            })
+        if let Ok(curve_key) = Curve25519PublicKey::from_base64(curve_key) {
+            self.get_user_devices(user_id)
+                .await
+                .map(|d| d.devices().find(|d| d.curve25519_key().map_or(false, |k| k == curve_key)))
+        } else {
+            Ok(None)
+        }
+    }
+
+    /// Get all devices associated with the given `user_id`
+    ///
+    /// *Note*: This doesn't return our own device.
+    pub async fn get_user_devices_filtered(&self, user_id: &UserId) -> Result<UserDevices> {
+        self.get_user_devices(user_id).await.map(|mut d| {
+            if user_id == self.user_id() {
+                d.inner.remove(self.device_id());
+            }
+            d
         })
     }
 
+    /// Get all devices associated with the given `user_id`
+    ///
+    /// *Note*: This does also return our own device.
     pub async fn get_user_devices(&self, user_id: &UserId) -> Result<UserDevices> {
-        let mut devices = self.inner.get_user_devices(user_id).await?;
-
-        if user_id == self.user_id() {
-            devices.remove(self.device_id());
-        }
+        let devices = self.get_readonly_devices_unfiltered(user_id).await?;
 
         let own_identity =
-            self.inner.get_user_identity(&self.user_id).await?.map(|i| i.own().cloned()).flatten();
-        let device_owner_identity = self.inner.get_user_identity(user_id).await.ok().flatten();
+            self.inner.get_user_identity(&self.user_id).await?.and_then(|i| i.own().cloned());
+        let device_owner_identity = self.inner.get_user_identity(user_id).await?;
 
         Ok(UserDevices {
             inner: devices,
@@ -315,31 +415,25 @@ impl Store {
         })
     }
 
+    /// Get a Device copy associated with `device_id` for `user_id`
     pub async fn get_device(
         &self,
         user_id: &UserId,
         device_id: &DeviceId,
     ) -> Result<Option<Device>> {
-        if user_id == self.user_id() && device_id == self.device_id() {
-            Ok(None)
-        } else {
-            let own_identity = self
-                .inner
-                .get_user_identity(&self.user_id)
-                .await?
-                .map(|i| i.own().cloned())
-                .flatten();
-            let device_owner_identity = self.inner.get_user_identity(user_id).await?;
+        let own_identity =
+            self.inner.get_user_identity(&self.user_id).await?.and_then(|i| i.own().cloned());
+        let device_owner_identity = self.inner.get_user_identity(user_id).await?;
 
-            Ok(self.inner.get_device(user_id, device_id).await?.map(|d| Device {
-                inner: d,
-                verification_machine: self.verification_machine.clone(),
-                own_identity,
-                device_owner_identity,
-            }))
-        }
+        Ok(self.inner.get_device(user_id, device_id).await?.map(|d| Device {
+            inner: d,
+            verification_machine: self.verification_machine.clone(),
+            own_identity,
+            device_owner_identity,
+        }))
     }
 
+    ///  Get the Identity of `user_id`
     pub async fn get_identity(&self, user_id: &UserId) -> Result<Option<UserIdentities>> {
         // let own_identity =
         // self.inner.get_user_identity(self.user_id()).await?.and_then(|i| i.own());
@@ -387,14 +481,26 @@ impl Store {
             | SecretName::CrossSigningSelfSigningKey => {
                 self.identity.lock().await.export_secret(secret_name).await
             }
-            SecretName::RecoveryKey => None,
+            SecretName::RecoveryKey => {
+                #[cfg(feature = "backups_v1")]
+                if let Some(key) = self.load_backup_keys().await.unwrap().recovery_key {
+                    let exported = key.to_base64();
+                    Some(exported)
+                } else {
+                    None
+                }
+
+                #[cfg(not(feature = "backups_v1"))]
+                None
+            }
             name => {
-                warn!(secret =? name, "Unknown secret was requested");
+                warn!(secret = ?name, "Unknown secret was requested");
                 None
             }
         }
     }
 
+    /// Import the Cross Signing Keys
     pub async fn import_cross_signing_keys(
         &self,
         export: CrossSigningKeyExport,
@@ -413,7 +519,7 @@ impl Store {
                 .await?;
 
             let status = identity.status().await;
-            info!(status =? status, "Successfully imported the private cross signing keys");
+            info!(?status, "Successfully imported the private cross signing keys");
 
             let changes =
                 Changes { private_identity: Some(identity.clone()), ..Default::default() };
@@ -424,6 +530,7 @@ impl Store {
         Ok(self.identity.lock().await.status().await)
     }
 
+    /// Import the given `secret` named `secret_name` into the keystore.
     pub async fn import_secret(
         &self,
         secret_name: &SecretName,
@@ -452,9 +559,14 @@ impl Store {
                     self.save_changes(changes).await?;
                 }
             }
-            SecretName::RecoveryKey => (),
+            SecretName::RecoveryKey => {
+                // We don't import the recovery key here since we'll want to
+                // check if the public key matches to the latest version on the
+                // server. We instead leave the key in the event and let the
+                // user import it later.
+            }
             name => {
-                warn!(secret =? name, "Tried to import an unknown secret");
+                warn!(secret = ?name, "Tried to import an unknown secret");
             }
         }
 
@@ -470,50 +582,61 @@ impl Deref for Store {
     }
 }
 
-#[derive(Debug, Error)]
 /// The crypto store's error type.
+#[derive(Debug, Error)]
 pub enum CryptoStoreError {
     /// The account that owns the sessions, group sessions, and devices wasn't
     /// found.
     #[error("can't save/load sessions or group sessions in the store before an account is stored")]
     AccountUnset,
 
-    /// Error in the internal database
-    #[cfg(feature = "sled_cryptostore")]
-    #[error(transparent)]
-    Database(#[from] sled::Error),
-
     /// An IO error occurred.
     #[error(transparent)]
     Io(#[from] IoError),
 
-    /// The underlying Olm Account operation returned an error.
-    #[error(transparent)]
-    OlmAccount(#[from] OlmAccountError),
-
-    /// The underlying Olm session operation returned an error.
-    #[error(transparent)]
-    OlmSession(#[from] OlmSessionError),
-
-    /// The underlying Olm group session operation returned an error.
-    #[error(transparent)]
-    OlmGroupSession(#[from] OlmGroupSessionError),
-
-    /// A session time-stamp couldn't be loaded.
-    #[error(transparent)]
-    SessionUnpickling(#[from] SessionUnpicklingError),
-
     /// Failed to decrypt an pickled object.
     #[error("An object failed to be decrypted while unpickling")]
     UnpicklingError,
 
+    /// Failed to decrypt an pickled object.
+    #[error(transparent)]
+    Pickle(#[from] vodozemac::PickleError),
+
+    /// The received room key couldn't be converted into a valid Megolm session.
+    #[error(transparent)]
+    SessionCreation(#[from] vodozemac::megolm::SessionKeyDecodeError),
+
     /// A Matrix identifier failed to be validated.
     #[error(transparent)]
-    IdentifierValidation(#[from] IdentifierValidationError),
+    IdentifierValidation(#[from] IdParseError),
 
     /// The store failed to (de)serialize a data type.
     #[error(transparent)]
     Serialization(#[from] SerdeError),
+
+    /// The database format has changed in a backwards incompatible way.
+    #[error(
+        "The database format changed in an incompatible way, current \
+        version: {0}, latest version: {1}"
+    )]
+    UnsupportedDatabaseVersion(usize, usize),
+
+    /// A problem with the underlying database backend
+    #[error(transparent)]
+    Backend(Box<dyn std::error::Error + Send + Sync>),
+}
+
+impl CryptoStoreError {
+    /// Create a new [`Backend`][Self::Backend] error.
+    ///
+    /// Shorthand for `StoreError::Backend(Box::new(error))`.
+    #[inline]
+    pub fn backend<E>(error: E) -> Self
+    where
+        E: std::error::Error + Send + Sync + 'static,
+    {
+        Self::Backend(Box::new(error))
+    }
 }
 
 /// Trait abstracting a store that the `OlmMachine` uses to store cryptographic
@@ -566,6 +689,22 @@ pub trait CryptoStore: AsyncTraitDeps {
     /// Get all the inbound group sessions we have stored.
     async fn get_inbound_group_sessions(&self) -> Result<Vec<InboundGroupSession>>;
 
+    /// Get the number inbound group sessions we have and how many of them are
+    /// backed up.
+    async fn inbound_group_session_counts(&self) -> Result<RoomKeyCounts>;
+
+    /// Get all the inbound group sessions we have not backed up yet.
+    async fn inbound_group_sessions_for_backup(
+        &self,
+        limit: usize,
+    ) -> Result<Vec<InboundGroupSession>>;
+
+    /// Reset the backup state of all the stored inbound group sessions.
+    async fn reset_backup_state(&self) -> Result<()>;
+
+    /// Get the backup keys we have stored.
+    async fn load_backup_keys(&self) -> Result<BackupKeys>;
+
     /// Get the outbound group sessions we have stored that is used for the
     /// given room.
     async fn get_outbound_group_sessions(
@@ -581,10 +720,10 @@ pub trait CryptoStore: AsyncTraitDeps {
 
     /// Set of users that we need to query keys for. This is a subset of
     /// the tracked users.
-    fn users_for_key_query(&self) -> HashSet<UserId>;
+    fn users_for_key_query(&self) -> HashSet<OwnedUserId>;
 
     /// Get all tracked users we know about.
-    fn tracked_users(&self) -> HashSet<UserId>;
+    fn tracked_users(&self) -> HashSet<OwnedUserId>;
 
     /// Add an user for tracking.
     ///
@@ -618,7 +757,7 @@ pub trait CryptoStore: AsyncTraitDeps {
     async fn get_user_devices(
         &self,
         user_id: &UserId,
-    ) -> Result<HashMap<DeviceIdBox, ReadOnlyDevice>>;
+    ) -> Result<HashMap<OwnedDeviceId, ReadOnlyDevice>>;
 
     /// Get the user identity that is attached to the given user id.
     ///
@@ -637,8 +776,10 @@ pub trait CryptoStore: AsyncTraitDeps {
     ///
     /// * `request_id` - The unique request id that identifies this outgoing
     /// secret request.
-    async fn get_outgoing_secret_requests(&self, request_id: Uuid)
-        -> Result<Option<GossipRequest>>;
+    async fn get_outgoing_secret_requests(
+        &self,
+        request_id: &TransactionId,
+    ) -> Result<Option<GossipRequest>>;
 
     /// Get an outgoing key request that we created that matches the given
     /// requested key info.
@@ -661,5 +802,5 @@ pub trait CryptoStore: AsyncTraitDeps {
     ///
     /// * `request_id` - The unique request id that identifies this outgoing key
     /// request.
-    async fn delete_outgoing_secret_requests(&self, request_id: Uuid) -> Result<()>;
+    async fn delete_outgoing_secret_requests(&self, request_id: &TransactionId) -> Result<()>;
 }
