@@ -15,8 +15,7 @@
 use std::sync::Arc;
 
 use dashmap::DashMap;
-use matrix_sdk_common::uuid::Uuid;
-use ruma::{DeviceId, UserId};
+use ruma::{DeviceId, OwnedTransactionId, OwnedUserId, TransactionId, UserId};
 use tracing::trace;
 
 use super::{event_enums::OutgoingContent, Sas, Verification};
@@ -26,8 +25,8 @@ use crate::{OutgoingRequest, OutgoingVerificationRequest, RoomMessageRequest, To
 
 #[derive(Clone, Debug)]
 pub struct VerificationCache {
-    verification: Arc<DashMap<UserId, DashMap<String, Verification>>>,
-    outgoing_requests: Arc<DashMap<Uuid, OutgoingRequest>>,
+    verification: Arc<DashMap<OwnedUserId, DashMap<String, Verification>>>,
+    outgoing_requests: Arc<DashMap<OwnedTransactionId, OutgoingRequest>>,
 }
 
 impl VerificationCache {
@@ -46,7 +45,7 @@ impl VerificationCache {
 
         self.verification
             .entry(verification.other_user().to_owned())
-            .or_insert_with(DashMap::new)
+            .or_default()
             .insert(verification.flow_id().to_owned(), verification);
     }
 
@@ -55,13 +54,11 @@ impl VerificationCache {
     }
 
     #[cfg(feature = "qrcode")]
-    #[cfg_attr(feature = "docs", doc(cfg(qrcode)))]
     pub fn insert_qr(&self, qr: QrVerification) {
         self.insert(qr)
     }
 
     #[cfg(feature = "qrcode")]
-    #[cfg_attr(feature = "docs", doc(cfg(qrcode)))]
     pub fn get_qr(&self, sender: &UserId, flow_id: &str) -> Option<QrVerification> {
         self.get(sender, flow_id).and_then(|v| {
             if let Verification::QrV1(qr) = v {
@@ -121,12 +118,14 @@ impl VerificationCache {
 
     pub fn add_request(&self, request: OutgoingRequest) {
         trace!("Adding an outgoing verification request {:?}", request);
-        self.outgoing_requests.insert(request.request_id, request);
+        self.outgoing_requests.insert(request.request_id.clone(), request);
     }
 
     pub fn add_verification_request(&self, request: OutgoingVerificationRequest) {
-        let request =
-            OutgoingRequest { request_id: request.request_id(), request: Arc::new(request.into()) };
+        let request = OutgoingRequest {
+            request_id: request.request_id().to_owned(),
+            request: Arc::new(request.into()),
+        };
         self.add_request(request);
     }
 
@@ -139,21 +138,25 @@ impl VerificationCache {
         match content {
             OutgoingContent::ToDevice(c) => {
                 let request = ToDeviceRequest::new(recipient, recipient_device.to_owned(), c);
-                let request_id = request.txn_id;
+                let request_id = request.txn_id.clone();
 
-                let request = OutgoingRequest { request_id, request: Arc::new(request.into()) };
+                let request = OutgoingRequest {
+                    request_id: request_id.clone(),
+                    request: Arc::new(request.into()),
+                };
 
                 self.outgoing_requests.insert(request_id, request);
             }
 
             OutgoingContent::Room(r, c) => {
-                let request_id = Uuid::new_v4();
+                let request_id = TransactionId::new();
 
                 let request = OutgoingRequest {
                     request: Arc::new(
-                        RoomMessageRequest { room_id: r, txn_id: request_id, content: c }.into(),
+                        RoomMessageRequest { room_id: r, txn_id: request_id.clone(), content: c }
+                            .into(),
                     ),
-                    request_id,
+                    request_id: request_id.clone(),
                 };
 
                 self.outgoing_requests.insert(request_id, request);
@@ -161,7 +164,7 @@ impl VerificationCache {
         }
     }
 
-    pub fn mark_request_as_sent(&self, uuid: &Uuid) {
-        self.outgoing_requests.remove(uuid);
+    pub fn mark_request_as_sent(&self, txn_id: &TransactionId) {
+        self.outgoing_requests.remove(txn_id);
     }
 }
