@@ -20,28 +20,38 @@ use std::{
 use ruma::{
     events::{
         key::verification::{
-            accept::{AcceptEventContent, AcceptMethod, AcceptToDeviceEventContent},
-            cancel::{CancelCode, CancelEventContent, CancelToDeviceEventContent},
-            done::{DoneEventContent, DoneToDeviceEventContent},
-            key::{KeyEventContent, KeyToDeviceEventContent},
-            mac::{MacEventContent, MacToDeviceEventContent},
-            ready::{ReadyEventContent, ReadyToDeviceEventContent},
-            request::RequestToDeviceEventContent,
-            start::{StartEventContent, StartMethod, StartToDeviceEventContent},
+            accept::{
+                AcceptMethod, KeyVerificationAcceptEventContent,
+                ToDeviceKeyVerificationAcceptEventContent,
+            },
+            cancel::{
+                CancelCode, KeyVerificationCancelEventContent,
+                ToDeviceKeyVerificationCancelEventContent,
+            },
+            done::{KeyVerificationDoneEventContent, ToDeviceKeyVerificationDoneEventContent},
+            key::{KeyVerificationKeyEventContent, ToDeviceKeyVerificationKeyEventContent},
+            mac::{KeyVerificationMacEventContent, ToDeviceKeyVerificationMacEventContent},
+            ready::{KeyVerificationReadyEventContent, ToDeviceKeyVerificationReadyEventContent},
+            request::ToDeviceKeyVerificationRequestEventContent,
+            start::{
+                KeyVerificationStartEventContent, StartMethod,
+                ToDeviceKeyVerificationStartEventContent,
+            },
             VerificationMethod,
         },
         room::message::{KeyVerificationRequestEventContent, MessageType},
-        AnyMessageEvent, AnyMessageEventContent, AnyToDeviceEvent, AnyToDeviceEventContent,
+        AnyMessageLikeEvent, AnyMessageLikeEventContent, AnyToDeviceEvent, AnyToDeviceEventContent,
+        MessageLikeEvent,
     },
-    serde::CanonicalJsonValue,
-    DeviceId, MilliSecondsSinceUnixEpoch, RoomId, UserId,
+    serde::{Base64, CanonicalJsonValue},
+    DeviceId, MilliSecondsSinceUnixEpoch, OwnedRoomId, UserId,
 };
 
 use super::FlowId;
 
 #[derive(Debug)]
 pub enum AnyEvent<'a> {
-    Room(&'a AnyMessageEvent),
+    Room(&'a AnyMessageLikeEvent),
     ToDevice(&'a AnyToDeviceEvent),
 }
 
@@ -53,11 +63,11 @@ impl AnyEvent<'_> {
         }
     }
 
-    pub fn timestamp(&self) -> Option<&MilliSecondsSinceUnixEpoch> {
+    pub fn timestamp(&self) -> Option<MilliSecondsSinceUnixEpoch> {
         match self {
             AnyEvent::Room(e) => Some(e.origin_server_ts()),
             AnyEvent::ToDevice(e) => match e {
-                AnyToDeviceEvent::KeyVerificationRequest(e) => Some(&e.content.timestamp),
+                AnyToDeviceEvent::KeyVerificationRequest(e) => Some(e.content.timestamp),
                 _ => None,
             },
         }
@@ -67,50 +77,40 @@ impl AnyEvent<'_> {
         matches!(self, AnyEvent::Room(_))
     }
 
-    pub fn verification_content(&self) -> Option<AnyVerificationContent> {
+    pub fn verification_content(&self) -> Option<AnyVerificationContent<'_>> {
         match self {
             AnyEvent::Room(e) => match e {
-                AnyMessageEvent::CallAnswer(_)
-                | AnyMessageEvent::CallInvite(_)
-                | AnyMessageEvent::CallHangup(_)
-                | AnyMessageEvent::CallCandidates(_)
-                | AnyMessageEvent::Reaction(_)
-                | AnyMessageEvent::RoomEncrypted(_)
-                | AnyMessageEvent::RoomMessageFeedback(_)
-                | AnyMessageEvent::RoomRedaction(_)
-                | AnyMessageEvent::Sticker(_) => None,
-                AnyMessageEvent::RoomMessage(m) => {
+                AnyMessageLikeEvent::RoomMessage(MessageLikeEvent::Original(m)) => {
                     if let MessageType::VerificationRequest(v) = &m.content.msgtype {
                         Some(RequestContent::from(v).into())
                     } else {
                         None
                     }
                 }
-                AnyMessageEvent::KeyVerificationReady(e) => {
+                AnyMessageLikeEvent::KeyVerificationReady(MessageLikeEvent::Original(e)) => {
                     Some(ReadyContent::from(&e.content).into())
                 }
-                AnyMessageEvent::KeyVerificationStart(e) => {
+                AnyMessageLikeEvent::KeyVerificationStart(MessageLikeEvent::Original(e)) => {
                     Some(StartContent::from(&e.content).into())
                 }
-                AnyMessageEvent::KeyVerificationCancel(e) => {
+                AnyMessageLikeEvent::KeyVerificationCancel(MessageLikeEvent::Original(e)) => {
                     Some(CancelContent::from(&e.content).into())
                 }
-                AnyMessageEvent::KeyVerificationAccept(e) => {
+                AnyMessageLikeEvent::KeyVerificationAccept(MessageLikeEvent::Original(e)) => {
                     Some(AcceptContent::from(&e.content).into())
                 }
-                AnyMessageEvent::KeyVerificationKey(e) => Some(KeyContent::from(&e.content).into()),
-                AnyMessageEvent::KeyVerificationMac(e) => Some(MacContent::from(&e.content).into()),
-                AnyMessageEvent::KeyVerificationDone(e) => {
+                AnyMessageLikeEvent::KeyVerificationKey(MessageLikeEvent::Original(e)) => {
+                    Some(KeyContent::from(&e.content).into())
+                }
+                AnyMessageLikeEvent::KeyVerificationMac(MessageLikeEvent::Original(e)) => {
+                    Some(MacContent::from(&e.content).into())
+                }
+                AnyMessageLikeEvent::KeyVerificationDone(MessageLikeEvent::Original(e)) => {
                     Some(DoneContent::from(&e.content).into())
                 }
                 _ => None,
             },
             AnyEvent::ToDevice(e) => match e {
-                AnyToDeviceEvent::Dummy(_)
-                | AnyToDeviceEvent::RoomKey(_)
-                | AnyToDeviceEvent::RoomKeyRequest(_)
-                | AnyToDeviceEvent::ForwardedRoomKey(_)
-                | AnyToDeviceEvent::RoomEncrypted(_) => None,
                 AnyToDeviceEvent::KeyVerificationRequest(e) => {
                     Some(RequestContent::from(&e.content).into())
                 }
@@ -141,8 +141,8 @@ impl AnyEvent<'_> {
     }
 }
 
-impl<'a> From<&'a AnyMessageEvent> for AnyEvent<'a> {
-    fn from(e: &'a AnyMessageEvent) -> Self {
+impl<'a> From<&'a AnyMessageLikeEvent> for AnyEvent<'a> {
+    fn from(e: &'a AnyMessageLikeEvent) -> Self {
         Self::Room(e)
     }
 }
@@ -164,41 +164,34 @@ impl TryFrom<&AnyEvent<'_>> for FlowId {
     }
 }
 
-impl TryFrom<&AnyMessageEvent> for FlowId {
+impl TryFrom<&AnyMessageLikeEvent> for FlowId {
     type Error = ();
 
-    fn try_from(value: &AnyMessageEvent) -> Result<Self, Self::Error> {
+    fn try_from(value: &AnyMessageLikeEvent) -> Result<Self, Self::Error> {
         match value {
-            AnyMessageEvent::CallAnswer(_)
-            | AnyMessageEvent::CallInvite(_)
-            | AnyMessageEvent::CallHangup(_)
-            | AnyMessageEvent::CallCandidates(_)
-            | AnyMessageEvent::Reaction(_)
-            | AnyMessageEvent::RoomEncrypted(_)
-            | AnyMessageEvent::RoomMessageFeedback(_)
-            | AnyMessageEvent::RoomRedaction(_)
-            | AnyMessageEvent::Sticker(_) => Err(()),
-            AnyMessageEvent::KeyVerificationReady(e) => {
-                Ok(FlowId::from((&e.room_id, &e.content.relates_to.event_id)))
+            AnyMessageLikeEvent::KeyVerificationReady(MessageLikeEvent::Original(e)) => {
+                Ok(FlowId::from((&*e.room_id, &*e.content.relates_to.event_id)))
             }
-            AnyMessageEvent::RoomMessage(e) => Ok(FlowId::from((&e.room_id, &e.event_id))),
-            AnyMessageEvent::KeyVerificationStart(e) => {
-                Ok(FlowId::from((&e.room_id, &e.content.relates_to.event_id)))
+            AnyMessageLikeEvent::RoomMessage(MessageLikeEvent::Original(e)) => {
+                Ok(FlowId::from((&*e.room_id, &*e.event_id)))
             }
-            AnyMessageEvent::KeyVerificationCancel(e) => {
-                Ok(FlowId::from((&e.room_id, &e.content.relates_to.event_id)))
+            AnyMessageLikeEvent::KeyVerificationStart(MessageLikeEvent::Original(e)) => {
+                Ok(FlowId::from((&*e.room_id, &*e.content.relates_to.event_id)))
             }
-            AnyMessageEvent::KeyVerificationAccept(e) => {
-                Ok(FlowId::from((&e.room_id, &e.content.relates_to.event_id)))
+            AnyMessageLikeEvent::KeyVerificationCancel(MessageLikeEvent::Original(e)) => {
+                Ok(FlowId::from((&*e.room_id, &*e.content.relates_to.event_id)))
             }
-            AnyMessageEvent::KeyVerificationKey(e) => {
-                Ok(FlowId::from((&e.room_id, &e.content.relates_to.event_id)))
+            AnyMessageLikeEvent::KeyVerificationAccept(MessageLikeEvent::Original(e)) => {
+                Ok(FlowId::from((&*e.room_id, &*e.content.relates_to.event_id)))
             }
-            AnyMessageEvent::KeyVerificationMac(e) => {
-                Ok(FlowId::from((&e.room_id, &e.content.relates_to.event_id)))
+            AnyMessageLikeEvent::KeyVerificationKey(MessageLikeEvent::Original(e)) => {
+                Ok(FlowId::from((&*e.room_id, &*e.content.relates_to.event_id)))
             }
-            AnyMessageEvent::KeyVerificationDone(e) => {
-                Ok(FlowId::from((&e.room_id, &e.content.relates_to.event_id)))
+            AnyMessageLikeEvent::KeyVerificationMac(MessageLikeEvent::Original(e)) => {
+                Ok(FlowId::from((&*e.room_id, &*e.content.relates_to.event_id)))
+            }
+            AnyMessageLikeEvent::KeyVerificationDone(MessageLikeEvent::Original(e)) => {
+                Ok(FlowId::from((&*e.room_id, &*e.content.relates_to.event_id)))
             }
             _ => Err(()),
         }
@@ -210,11 +203,6 @@ impl TryFrom<&AnyToDeviceEvent> for FlowId {
 
     fn try_from(value: &AnyToDeviceEvent) -> Result<Self, Self::Error> {
         match value {
-            AnyToDeviceEvent::Dummy(_)
-            | AnyToDeviceEvent::RoomKey(_)
-            | AnyToDeviceEvent::RoomKeyRequest(_)
-            | AnyToDeviceEvent::ForwardedRoomKey(_)
-            | AnyToDeviceEvent::RoomEncrypted(_) => Err(()),
             AnyToDeviceEvent::KeyVerificationRequest(e) => {
                 Ok(FlowId::from(e.content.transaction_id.to_owned()))
             }
@@ -258,11 +246,12 @@ pub enum AnyVerificationContent<'a> {
 
 #[derive(Debug)]
 pub enum RequestContent<'a> {
-    ToDevice(&'a RequestToDeviceEventContent),
+    ToDevice(&'a ToDeviceKeyVerificationRequestEventContent),
     Room(&'a KeyVerificationRequestEventContent),
 }
 
 impl RequestContent<'_> {
+    #[allow(clippy::wrong_self_convention)]
     pub fn from_device(&self) -> &DeviceId {
         match self {
             Self::ToDevice(t) => &t.from_device,
@@ -280,11 +269,12 @@ impl RequestContent<'_> {
 
 #[derive(Debug)]
 pub enum ReadyContent<'a> {
-    ToDevice(&'a ReadyToDeviceEventContent),
-    Room(&'a ReadyEventContent),
+    ToDevice(&'a ToDeviceKeyVerificationReadyEventContent),
+    Room(&'a KeyVerificationReadyEventContent),
 }
 
 impl ReadyContent<'_> {
+    #[allow(clippy::wrong_self_convention)]
     pub fn from_device(&self) -> &DeviceId {
         match self {
             Self::ToDevice(t) => &t.from_device,
@@ -329,20 +319,20 @@ from_for_enum!(AcceptContent, Accept, AnyVerificationContent);
 from_for_enum!(KeyContent, Key, AnyVerificationContent);
 from_for_enum!(MacContent, Mac, AnyVerificationContent);
 
-from_borrow_for_enum!(RequestToDeviceEventContent, ToDevice, RequestContent);
+from_borrow_for_enum!(ToDeviceKeyVerificationRequestEventContent, ToDevice, RequestContent);
 from_borrow_for_enum!(KeyVerificationRequestEventContent, Room, RequestContent);
-from_borrow_for_enum!(ReadyEventContent, Room, ReadyContent);
-from_borrow_for_enum!(ReadyToDeviceEventContent, ToDevice, ReadyContent);
-from_borrow_for_enum!(StartEventContent, Room, StartContent);
-from_borrow_for_enum!(StartToDeviceEventContent, ToDevice, StartContent);
-from_borrow_for_enum!(AcceptEventContent, Room, AcceptContent);
-from_borrow_for_enum!(AcceptToDeviceEventContent, ToDevice, AcceptContent);
-from_borrow_for_enum!(KeyEventContent, Room, KeyContent);
-from_borrow_for_enum!(KeyToDeviceEventContent, ToDevice, KeyContent);
-from_borrow_for_enum!(MacEventContent, Room, MacContent);
-from_borrow_for_enum!(MacToDeviceEventContent, ToDevice, MacContent);
-from_borrow_for_enum!(CancelEventContent, Room, CancelContent);
-from_borrow_for_enum!(CancelToDeviceEventContent, ToDevice, CancelContent);
+from_borrow_for_enum!(KeyVerificationReadyEventContent, Room, ReadyContent);
+from_borrow_for_enum!(ToDeviceKeyVerificationReadyEventContent, ToDevice, ReadyContent);
+from_borrow_for_enum!(KeyVerificationStartEventContent, Room, StartContent);
+from_borrow_for_enum!(ToDeviceKeyVerificationStartEventContent, ToDevice, StartContent);
+from_borrow_for_enum!(KeyVerificationAcceptEventContent, Room, AcceptContent);
+from_borrow_for_enum!(ToDeviceKeyVerificationAcceptEventContent, ToDevice, AcceptContent);
+from_borrow_for_enum!(KeyVerificationKeyEventContent, Room, KeyContent);
+from_borrow_for_enum!(ToDeviceKeyVerificationKeyEventContent, ToDevice, KeyContent);
+from_borrow_for_enum!(KeyVerificationMacEventContent, Room, MacContent);
+from_borrow_for_enum!(ToDeviceKeyVerificationMacEventContent, ToDevice, MacContent);
+from_borrow_for_enum!(KeyVerificationCancelEventContent, Room, CancelContent);
+from_borrow_for_enum!(ToDeviceKeyVerificationCancelEventContent, ToDevice, CancelContent);
 
 macro_rules! try_from_outgoing_content {
     ($type: ident, $enum_variant: ident) => {
@@ -352,7 +342,7 @@ macro_rules! try_from_outgoing_content {
             fn try_from(value: &'a OutgoingContent) -> Result<Self, Self::Error> {
                 match value {
                     OutgoingContent::Room(_, c) => {
-                        if let AnyMessageEventContent::$enum_variant(c) = c {
+                        if let AnyMessageLikeEventContent::$enum_variant(c) = c {
                             Ok(Self::Room(c))
                         } else {
                             Err(())
@@ -385,7 +375,7 @@ impl<'a> TryFrom<&'a OutgoingContent> for RequestContent<'a> {
     fn try_from(value: &'a OutgoingContent) -> Result<Self, Self::Error> {
         match value {
             OutgoingContent::Room(_, c) => {
-                if let AnyMessageEventContent::RoomMessage(m) = c {
+                if let AnyMessageLikeEventContent::RoomMessage(m) = c {
                     if let MessageType::VerificationRequest(c) = &m.msgtype {
                         Ok(Self::Room(c))
                     } else {
@@ -408,11 +398,12 @@ impl<'a> TryFrom<&'a OutgoingContent> for RequestContent<'a> {
 
 #[derive(Debug)]
 pub enum StartContent<'a> {
-    ToDevice(&'a StartToDeviceEventContent),
-    Room(&'a StartEventContent),
+    ToDevice(&'a ToDeviceKeyVerificationStartEventContent),
+    Room(&'a KeyVerificationStartEventContent),
 }
 
 impl<'a> StartContent<'a> {
+    #[allow(clippy::wrong_self_convention)]
     pub fn from_device(&self) -> &DeviceId {
         match self {
             Self::ToDevice(c) => &c.from_device,
@@ -422,7 +413,7 @@ impl<'a> StartContent<'a> {
 
     pub fn flow_id(&self) -> &str {
         match self {
-            Self::ToDevice(c) => &c.transaction_id,
+            Self::ToDevice(c) => c.transaction_id.as_str(),
             Self::Room(c) => c.relates_to.event_id.as_str(),
         }
     }
@@ -455,18 +446,18 @@ impl<'a> From<&'a OwnedStartContent> for StartContent<'a> {
 
 #[derive(Debug)]
 pub enum DoneContent<'a> {
-    ToDevice(&'a DoneToDeviceEventContent),
-    Room(&'a DoneEventContent),
+    ToDevice(&'a ToDeviceKeyVerificationDoneEventContent),
+    Room(&'a KeyVerificationDoneEventContent),
 }
 
-impl<'a> From<&'a DoneEventContent> for DoneContent<'a> {
-    fn from(c: &'a DoneEventContent) -> Self {
+impl<'a> From<&'a KeyVerificationDoneEventContent> for DoneContent<'a> {
+    fn from(c: &'a KeyVerificationDoneEventContent) -> Self {
         Self::Room(c)
     }
 }
 
-impl<'a> From<&'a DoneToDeviceEventContent> for DoneContent<'a> {
-    fn from(c: &'a DoneToDeviceEventContent) -> Self {
+impl<'a> From<&'a ToDeviceKeyVerificationDoneEventContent> for DoneContent<'a> {
+    fn from(c: &'a ToDeviceKeyVerificationDoneEventContent) -> Self {
         Self::ToDevice(c)
     }
 }
@@ -474,7 +465,7 @@ impl<'a> From<&'a DoneToDeviceEventContent> for DoneContent<'a> {
 impl<'a> DoneContent<'a> {
     pub fn flow_id(&self) -> &str {
         match self {
-            Self::ToDevice(c) => &c.transaction_id,
+            Self::ToDevice(c) => c.transaction_id.as_str(),
             Self::Room(c) => c.relates_to.event_id.as_str(),
         }
     }
@@ -482,14 +473,14 @@ impl<'a> DoneContent<'a> {
 
 #[derive(Clone, Debug)]
 pub enum AcceptContent<'a> {
-    ToDevice(&'a AcceptToDeviceEventContent),
-    Room(&'a AcceptEventContent),
+    ToDevice(&'a ToDeviceKeyVerificationAcceptEventContent),
+    Room(&'a KeyVerificationAcceptEventContent),
 }
 
 impl AcceptContent<'_> {
     pub fn flow_id(&self) -> &str {
         match self {
-            Self::ToDevice(c) => &c.transaction_id,
+            Self::ToDevice(c) => c.transaction_id.as_str(),
             Self::Room(c) => c.relates_to.event_id.as_str(),
         }
     }
@@ -513,19 +504,19 @@ impl<'a> From<&'a OwnedAcceptContent> for AcceptContent<'a> {
 
 #[derive(Clone, Debug)]
 pub enum KeyContent<'a> {
-    ToDevice(&'a KeyToDeviceEventContent),
-    Room(&'a KeyEventContent),
+    ToDevice(&'a ToDeviceKeyVerificationKeyEventContent),
+    Room(&'a KeyVerificationKeyEventContent),
 }
 
 impl KeyContent<'_> {
     pub fn flow_id(&self) -> &str {
         match self {
-            Self::ToDevice(c) => &c.transaction_id,
+            Self::ToDevice(c) => c.transaction_id.as_str(),
             Self::Room(c) => c.relates_to.event_id.as_str(),
         }
     }
 
-    pub fn public_key(&self) -> &str {
+    pub fn public_key(&self) -> &Base64 {
         match self {
             Self::ToDevice(c) => &c.key,
             Self::Room(c) => &c.key,
@@ -535,26 +526,26 @@ impl KeyContent<'_> {
 
 #[derive(Clone, Debug)]
 pub enum MacContent<'a> {
-    ToDevice(&'a MacToDeviceEventContent),
-    Room(&'a MacEventContent),
+    ToDevice(&'a ToDeviceKeyVerificationMacEventContent),
+    Room(&'a KeyVerificationMacEventContent),
 }
 
 impl MacContent<'_> {
     pub fn flow_id(&self) -> &str {
         match self {
-            Self::ToDevice(c) => &c.transaction_id,
+            Self::ToDevice(c) => c.transaction_id.as_str(),
             Self::Room(c) => c.relates_to.event_id.as_str(),
         }
     }
 
-    pub fn mac(&self) -> &BTreeMap<String, String> {
+    pub fn mac(&self) -> &BTreeMap<String, Base64> {
         match self {
             Self::ToDevice(c) => &c.mac,
             Self::Room(c) => &c.mac,
         }
     }
 
-    pub fn keys(&self) -> &str {
+    pub fn keys(&self) -> &Base64 {
         match self {
             Self::ToDevice(c) => &c.keys,
             Self::Room(c) => &c.keys,
@@ -564,8 +555,8 @@ impl MacContent<'_> {
 
 #[derive(Clone, Debug)]
 pub enum CancelContent<'a> {
-    ToDevice(&'a CancelToDeviceEventContent),
-    Room(&'a CancelEventContent),
+    ToDevice(&'a ToDeviceKeyVerificationCancelEventContent),
+    Room(&'a KeyVerificationCancelEventContent),
 }
 
 impl CancelContent<'_> {
@@ -579,8 +570,8 @@ impl CancelContent<'_> {
 
 #[derive(Clone, Debug)]
 pub enum OwnedStartContent {
-    ToDevice(StartToDeviceEventContent),
-    Room(RoomId, StartEventContent),
+    ToDevice(ToDeviceKeyVerificationStartEventContent),
+    Room(OwnedRoomId, KeyVerificationStartEventContent),
 }
 
 impl OwnedStartContent {
@@ -622,32 +613,32 @@ impl OwnedStartContent {
     }
 }
 
-impl From<(RoomId, StartEventContent)> for OwnedStartContent {
-    fn from(tuple: (RoomId, StartEventContent)) -> Self {
+impl From<(OwnedRoomId, KeyVerificationStartEventContent)> for OwnedStartContent {
+    fn from(tuple: (OwnedRoomId, KeyVerificationStartEventContent)) -> Self {
         Self::Room(tuple.0, tuple.1)
     }
 }
 
-impl From<StartToDeviceEventContent> for OwnedStartContent {
-    fn from(content: StartToDeviceEventContent) -> Self {
+impl From<ToDeviceKeyVerificationStartEventContent> for OwnedStartContent {
+    fn from(content: ToDeviceKeyVerificationStartEventContent) -> Self {
         Self::ToDevice(content)
     }
 }
 
 #[derive(Clone, Debug)]
 pub enum OwnedAcceptContent {
-    ToDevice(AcceptToDeviceEventContent),
-    Room(RoomId, AcceptEventContent),
+    ToDevice(ToDeviceKeyVerificationAcceptEventContent),
+    Room(OwnedRoomId, KeyVerificationAcceptEventContent),
 }
 
-impl From<AcceptToDeviceEventContent> for OwnedAcceptContent {
-    fn from(content: AcceptToDeviceEventContent) -> Self {
+impl From<ToDeviceKeyVerificationAcceptEventContent> for OwnedAcceptContent {
+    fn from(content: ToDeviceKeyVerificationAcceptEventContent) -> Self {
         Self::ToDevice(content)
     }
 }
 
-impl From<(RoomId, AcceptEventContent)> for OwnedAcceptContent {
-    fn from(content: (RoomId, AcceptEventContent)) -> Self {
+impl From<(OwnedRoomId, KeyVerificationAcceptEventContent)> for OwnedAcceptContent {
+    fn from(content: (OwnedRoomId, KeyVerificationAcceptEventContent)) -> Self {
         Self::Room(content.0, content.1)
     }
 }
@@ -663,7 +654,7 @@ impl OwnedAcceptContent {
 
 #[derive(Clone, Debug)]
 pub enum OutgoingContent {
-    Room(RoomId, AnyMessageEventContent),
+    Room(OwnedRoomId, AnyMessageLikeEventContent),
     ToDevice(AnyToDeviceEventContent),
 }
 
@@ -671,7 +662,7 @@ impl From<OwnedStartContent> for OutgoingContent {
     fn from(content: OwnedStartContent) -> Self {
         match content {
             OwnedStartContent::Room(r, c) => {
-                (r, AnyMessageEventContent::KeyVerificationStart(c)).into()
+                (r, AnyMessageLikeEventContent::KeyVerificationStart(c)).into()
             }
             OwnedStartContent::ToDevice(c) => {
                 AnyToDeviceEventContent::KeyVerificationStart(c).into()
@@ -686,8 +677,8 @@ impl From<AnyToDeviceEventContent> for OutgoingContent {
     }
 }
 
-impl From<(RoomId, AnyMessageEventContent)> for OutgoingContent {
-    fn from(content: (RoomId, AnyMessageEventContent)) -> Self {
+impl From<(OwnedRoomId, AnyMessageLikeEventContent)> for OutgoingContent {
+    fn from(content: (OwnedRoomId, AnyMessageLikeEventContent)) -> Self {
         OutgoingContent::Room(content.0, content.1)
     }
 }
@@ -715,7 +706,7 @@ impl TryFrom<ToDeviceRequest> for OutgoingContent {
     type Error = String;
 
     fn try_from(request: ToDeviceRequest) -> Result<Self, Self::Error> {
-        use ruma::events::EventType;
+        use ruma::events::ToDeviceEventType;
         use serde_json::Value;
 
         let json: Value = serde_json::from_str(
@@ -730,30 +721,40 @@ impl TryFrom<ToDeviceRequest> for OutgoingContent {
         .map_err(|e| e.to_string())?;
 
         let content = match request.event_type {
-            EventType::KeyVerificationStart => AnyToDeviceEventContent::KeyVerificationStart(
+            ToDeviceEventType::KeyVerificationStart => {
+                AnyToDeviceEventContent::KeyVerificationStart(
+                    serde_json::from_value(json).map_err(|e| e.to_string())?,
+                )
+            }
+            ToDeviceEventType::KeyVerificationKey => AnyToDeviceEventContent::KeyVerificationKey(
                 serde_json::from_value(json).map_err(|e| e.to_string())?,
             ),
-            EventType::KeyVerificationKey => AnyToDeviceEventContent::KeyVerificationKey(
+            ToDeviceEventType::KeyVerificationAccept => {
+                AnyToDeviceEventContent::KeyVerificationAccept(
+                    serde_json::from_value(json).map_err(|e| e.to_string())?,
+                )
+            }
+            ToDeviceEventType::KeyVerificationMac => AnyToDeviceEventContent::KeyVerificationMac(
                 serde_json::from_value(json).map_err(|e| e.to_string())?,
             ),
-            EventType::KeyVerificationAccept => AnyToDeviceEventContent::KeyVerificationAccept(
-                serde_json::from_value(json).map_err(|e| e.to_string())?,
-            ),
-            EventType::KeyVerificationMac => AnyToDeviceEventContent::KeyVerificationMac(
-                serde_json::from_value(json).map_err(|e| e.to_string())?,
-            ),
-            EventType::KeyVerificationCancel => AnyToDeviceEventContent::KeyVerificationCancel(
-                serde_json::from_value(json).map_err(|e| e.to_string())?,
-            ),
-            EventType::KeyVerificationReady => AnyToDeviceEventContent::KeyVerificationReady(
-                serde_json::from_value(json).map_err(|e| e.to_string())?,
-            ),
-            EventType::KeyVerificationDone => AnyToDeviceEventContent::KeyVerificationDone(
-                serde_json::from_value(json).map_err(|e| e.to_string())?,
-            ),
-            EventType::KeyVerificationRequest => AnyToDeviceEventContent::KeyVerificationRequest(
+            ToDeviceEventType::KeyVerificationCancel => {
+                AnyToDeviceEventContent::KeyVerificationCancel(
+                    serde_json::from_value(json).map_err(|e| e.to_string())?,
+                )
+            }
+            ToDeviceEventType::KeyVerificationReady => {
+                AnyToDeviceEventContent::KeyVerificationReady(
+                    serde_json::from_value(json).map_err(|e| e.to_string())?,
+                )
+            }
+            ToDeviceEventType::KeyVerificationDone => AnyToDeviceEventContent::KeyVerificationDone(
                 serde_json::from_value(json).map_err(|e| e.to_string())?,
             ),
+            ToDeviceEventType::KeyVerificationRequest => {
+                AnyToDeviceEventContent::KeyVerificationRequest(
+                    serde_json::from_value(json).map_err(|e| e.to_string())?,
+                )
+            }
             e => return Err(format!("Unsupported event type {}", e)),
         };
 
@@ -766,11 +767,12 @@ impl TryFrom<OutgoingRequest> for OutgoingContent {
 
     fn try_from(value: OutgoingRequest) -> Result<Self, Self::Error> {
         match value.request() {
-            crate::OutgoingRequests::KeysUpload(_) => Err("Invalid request type".to_owned()),
-            crate::OutgoingRequests::KeysQuery(_) => Err("Invalid request type".to_owned()),
+            crate::OutgoingRequests::KeysUpload(_)
+            | crate::OutgoingRequests::KeysQuery(_)
+            | crate::OutgoingRequests::KeysBackup(_)
+            | crate::OutgoingRequests::SignatureUpload(_)
+            | crate::OutgoingRequests::KeysClaim(_) => Err("Invalid request type".to_owned()),
             crate::OutgoingRequests::ToDeviceRequest(r) => Self::try_from(r.clone()),
-            crate::OutgoingRequests::SignatureUpload(_) => Err("Invalid request type".to_owned()),
-            crate::OutgoingRequests::KeysClaim(_) => Err("Invalid request type".to_owned()),
             crate::OutgoingRequests::RoomMessage(r) => Ok(Self::from(r.clone())),
         }
     }
