@@ -18,14 +18,15 @@ use std::{
 };
 
 use dashmap::DashMap;
-use matrix_sdk_common::{locks::Mutex, uuid::Uuid};
+use matrix_sdk_common::locks::Mutex;
 use ruma::{
     events::{
         key::verification::VerificationMethod, AnyToDeviceEvent, AnyToDeviceEventContent,
         ToDeviceEvent,
     },
     serde::Raw,
-    DeviceId, DeviceIdBox, EventId, MilliSecondsSinceUnixEpoch, RoomId, UserId,
+    uint, DeviceId, EventId, MilliSecondsSinceUnixEpoch, OwnedDeviceId, OwnedUserId, RoomId,
+    SecondsSinceUnixEpoch, TransactionId, UInt, UserId,
 };
 use tracing::{info, trace, warn};
 
@@ -46,10 +47,9 @@ use crate::{
 
 #[derive(Clone, Debug)]
 pub struct VerificationMachine {
-    pub(crate) private_identity: Arc<Mutex<PrivateCrossSigningIdentity>>,
     pub(crate) store: VerificationStore,
     verifications: VerificationCache,
-    requests: Arc<DashMap<UserId, DashMap<String, VerificationRequest>>>,
+    requests: Arc<DashMap<OwnedUserId, DashMap<String, VerificationRequest>>>,
 }
 
 impl VerificationMachine {
@@ -59,8 +59,7 @@ impl VerificationMachine {
         store: Arc<dyn CryptoStore>,
     ) -> Self {
         Self {
-            private_identity: identity,
-            store: VerificationStore { account, inner: store },
+            store: VerificationStore { account, private_identity: identity, inner: store },
             verifications: VerificationCache::new(),
             requests: Default::default(),
         }
@@ -77,14 +76,13 @@ impl VerificationMachine {
     pub(crate) async fn request_to_device_verification(
         &self,
         user_id: &UserId,
-        recipient_devices: Vec<DeviceIdBox>,
+        recipient_devices: Vec<OwnedDeviceId>,
         methods: Option<Vec<VerificationMethod>>,
     ) -> (VerificationRequest, OutgoingVerificationRequest) {
-        let flow_id = FlowId::from(Uuid::new_v4().to_string());
+        let flow_id = FlowId::from(TransactionId::new());
 
         let verification = VerificationRequest::new(
             self.verifications.clone(),
-            self.private_identity.lock().await.clone(),
             self.store.clone(),
             flow_id,
             user_id,
@@ -110,7 +108,6 @@ impl VerificationMachine {
 
         let request = VerificationRequest::new(
             self.verifications.clone(),
-            self.private_identity.lock().await.clone(),
             self.store.clone(),
             flow_id,
             identity.user_id(),
@@ -127,25 +124,12 @@ impl VerificationMachine {
         &self,
         device: ReadOnlyDevice,
     ) -> Result<(Sas, OutgoingVerificationRequest), CryptoStoreError> {
-        let identity = self.store.get_user_identity(device.user_id()).await?;
-        let own_identity =
-            self.store.get_user_identity(self.own_user_id()).await?.and_then(|i| i.into_own());
-        let private_identity = self.private_identity.lock().await.clone();
-
-        let (sas, content) = Sas::start(
-            private_identity,
-            device.clone(),
-            self.store.clone(),
-            own_identity,
-            identity,
-            None,
-            true,
-            None,
-        );
+        let identities = self.store.get_identities(device.clone()).await?;
+        let (sas, content) = Sas::start(identities, TransactionId::new(), true, None);
 
         let request = match content {
             OutgoingContent::Room(r, c) => {
-                RoomMessageRequest { room_id: r, txn_id: Uuid::new_v4(), content: c }.into()
+                RoomMessageRequest { room_id: r, txn_id: TransactionId::new(), content: c }.into()
             }
             OutgoingContent::ToDevice(c) => {
                 let request =
@@ -178,7 +162,7 @@ impl VerificationMachine {
     fn insert_request(&self, request: VerificationRequest) {
         self.requests
             .entry(request.other_user().to_owned())
-            .or_insert_with(DashMap::new)
+            .or_default()
             .insert(request.flow_id().as_str().to_owned(), request);
     }
 
@@ -190,10 +174,7 @@ impl VerificationMachine {
         self.verifications.get_sas(user_id, flow_id)
     }
 
-    #[cfg(not(target_arch = "wasm32"))]
-    fn is_timestamp_valid(timestamp: &MilliSecondsSinceUnixEpoch) -> bool {
-        use ruma::{uint, UInt};
-
+    fn is_timestamp_valid(timestamp: MilliSecondsSinceUnixEpoch) -> bool {
         // The event should be ignored if the event is older than 10 minutes
         let old_timestamp_threshold: UInt = uint!(600);
         // The event should be ignored if the event is 5 minutes or more into the
@@ -201,21 +182,12 @@ impl VerificationMachine {
         let timestamp_threshold: UInt = uint!(300);
 
         let timestamp = timestamp.as_secs();
-        let now = MilliSecondsSinceUnixEpoch::now().as_secs();
+        let now = SecondsSinceUnixEpoch::now().get();
 
         !(now.saturating_sub(timestamp) > old_timestamp_threshold
             || timestamp.saturating_sub(now) > timestamp_threshold)
     }
 
-    #[cfg(target_arch = "wasm32")]
-    fn is_timestamp_valid(timestamp: &MilliSecondsSinceUnixEpoch) -> bool {
-        // TODO the non-wasm method with the same name uses
-        // `MilliSecondsSinceUnixEpoch::now()` which internally uses
-        // `SystemTime::now()` this panics under WASM, thus we're returning here
-        // true for now.
-        true
-    }
-
     fn queue_up_content(
         &self,
         recipient: &UserId,
@@ -225,8 +197,8 @@ impl VerificationMachine {
         self.verifications.queue_up_content(recipient, recipient_device, content)
     }
 
-    pub fn mark_request_as_sent(&self, uuid: &Uuid) {
-        self.verifications.mark_request_as_sent(uuid);
+    pub fn mark_request_as_sent(&self, txn_id: &TransactionId) {
+        self.verifications.mark_request_as_sent(txn_id);
     }
 
     pub fn outgoing_messages(&self) -> Vec<OutgoingRequest> {
@@ -260,7 +232,11 @@ impl VerificationMachine {
             {
                 let event = ToDeviceEvent { content, sender: self.own_user_id().to_owned() };
 
-                events.push(AnyToDeviceEvent::KeyVerificationCancel(event).into());
+                events.push(
+                    Raw::new(&event)
+                        .expect("Failed to serialize m.key_verification.cancel event")
+                        .cast(),
+                );
             }
 
             self.verifications.add_verification_request(request)
@@ -303,6 +279,7 @@ impl VerificationMachine {
     ) -> Result<(), CryptoStoreError> {
         let event = event.into();
 
+        #[allow(clippy::question_mark)]
         let flow_id = if let Ok(flow_id) = FlowId::try_from(&event) {
             flow_id
         } else {
@@ -343,7 +320,6 @@ impl VerificationMachine {
                             if !event_sent_from_us(&event, r.from_device()) {
                                 let request = VerificationRequest::from_request(
                                     self.verifications.clone(),
-                                    self.private_identity.lock().await.clone(),
                                     self.store.clone(),
                                     event.sender(),
                                     flow_id,
@@ -362,7 +338,7 @@ impl VerificationMachine {
                             trace!(
                                 sender = event.sender().as_str(),
                                 from_device = r.from_device().as_str(),
-                                timestamp =? timestamp,
+                                ?timestamp,
                                 "The received verification request was too old or too far into the future",
                             );
                         }
@@ -414,25 +390,9 @@ impl VerificationMachine {
                         if let Some(device) =
                             self.store.get_device(event.sender(), c.from_device()).await?
                         {
-                            let private_identity = self.private_identity.lock().await.clone();
-                            let own_identity = self
-                                .store
-                                .get_user_identity(self.own_user_id())
-                                .await?
-                                .and_then(|i| i.into_own());
-                            let identity = self.store.get_user_identity(event.sender()).await?;
+                            let identities = self.store.get_identities(device).await?;
 
-                            match Sas::from_start_event(
-                                flow_id,
-                                c,
-                                self.store.clone(),
-                                private_identity,
-                                device,
-                                own_identity,
-                                identity,
-                                None,
-                                false,
-                            ) {
+                            match Sas::from_start_event(flow_id, c, identities, None, false) {
                                 Ok(sas) => {
                                     self.verifications.insert_sas(sas);
                                 }
@@ -467,6 +427,18 @@ impl VerificationMachine {
 
                             if s.is_done() {
                                 self.mark_sas_as_done(s, content).await?;
+                            } else {
+                                // Even if we are not done (yet), there might be content to send
+                                // out, e.g. in the case where we are done with our side of the
+                                // verification process, but the other side has not yet sent their
+                                // "done".
+                                if let Some(content) = content {
+                                    self.queue_up_content(
+                                        s.other_user_id(),
+                                        s.other_device_id(),
+                                        content,
+                                    );
+                                }
                             }
                         } else {
                             flow_id_mismatch();
@@ -510,16 +482,12 @@ impl VerificationMachine {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
+    use std::{convert::TryFrom, sync::Arc, time::Duration};
 
-    use std::{
-        convert::TryFrom,
-        sync::Arc,
-        time::{Duration, Instant},
-    };
-
-    use matrix_sdk_common::locks::Mutex;
-    use ruma::{DeviceId, UserId};
+    use matrix_sdk_common::{instant::Instant, locks::Mutex};
+    use matrix_sdk_test::async_test;
+    use ruma::{device_id, user_id, DeviceId, TransactionId, UserId};
 
     use super::{Sas, VerificationMachine};
     use crate::{
@@ -527,31 +495,31 @@ mod test {
         store::MemoryStore,
         verification::{
             event_enums::{AcceptContent, KeyContent, MacContent, OutgoingContent},
-            test::wrap_any_to_device_content,
+            tests::wrap_any_to_device_content,
             VerificationStore,
         },
         ReadOnlyAccount, ReadOnlyDevice,
     };
 
-    fn alice_id() -> UserId {
-        UserId::try_from("@alice:example.org").unwrap()
+    fn alice_id() -> &'static UserId {
+        user_id!("@alice:example.org")
     }
 
-    fn alice_device_id() -> Box<DeviceId> {
-        "JLAFKJWSCS".into()
+    fn alice_device_id() -> &'static DeviceId {
+        device_id!("JLAFKJWSCS")
     }
 
-    fn bob_id() -> UserId {
-        UserId::try_from("@bob:example.org").unwrap()
+    fn bob_id() -> &'static UserId {
+        user_id!("@bob:example.org")
     }
 
-    fn bob_device_id() -> Box<DeviceId> {
-        "BOBDEVCIE".into()
+    fn bob_device_id() -> &'static DeviceId {
+        device_id!("BOBDEVCIE")
     }
 
     async fn setup_verification_machine() -> (VerificationMachine, Sas) {
-        let alice = ReadOnlyAccount::new(&alice_id(), &alice_device_id());
-        let bob = ReadOnlyAccount::new(&bob_id(), &bob_device_id());
+        let alice = ReadOnlyAccount::new(alice_id(), alice_device_id());
+        let bob = ReadOnlyAccount::new(bob_id(), bob_device_id());
         let store = MemoryStore::new();
         let bob_store = MemoryStore::new();
 
@@ -561,20 +529,20 @@ mod test {
         store.save_devices(vec![bob_device]).await;
         bob_store.save_devices(vec![alice_device.clone()]).await;
 
-        let bob_store = VerificationStore { account: bob, inner: Arc::new(bob_store) };
+        let identity = Arc::new(Mutex::new(PrivateCrossSigningIdentity::empty(bob_id())));
+        let bob_store = VerificationStore {
+            account: bob,
+            inner: Arc::new(bob_store),
+            private_identity: identity,
+        };
 
-        let identity = Arc::new(Mutex::new(PrivateCrossSigningIdentity::empty(alice_id())));
-        let machine = VerificationMachine::new(alice, identity, Arc::new(store));
-        let (bob_sas, start_content) = Sas::start(
-            PrivateCrossSigningIdentity::empty(bob_id()),
-            alice_device,
-            bob_store,
-            None,
-            None,
-            None,
-            true,
-            None,
+        let identities = bob_store.get_identities(alice_device).await.unwrap();
+        let machine = VerificationMachine::new(
+            alice,
+            Mutex::new(PrivateCrossSigningIdentity::empty(alice_id())).into(),
+            Arc::new(store),
         );
+        let (bob_sas, start_content) = Sas::start(identities, TransactionId::new(), true, None);
 
         machine
             .receive_any_event(&wrap_any_to_device_content(bob_sas.user_id(), start_content))
@@ -584,15 +552,15 @@ mod test {
         (machine, bob_sas)
     }
 
-    #[test]
-    fn create() {
-        let alice = ReadOnlyAccount::new(&alice_id(), &alice_device_id());
+    #[async_test]
+    async fn create() {
+        let alice = ReadOnlyAccount::new(alice_id(), alice_device_id());
         let identity = Arc::new(Mutex::new(PrivateCrossSigningIdentity::empty(alice_id())));
         let store = MemoryStore::new();
         let _ = VerificationMachine::new(alice, identity, Arc::new(store));
     }
 
-    #[tokio::test]
+    #[async_test]
     async fn full_flow() {
         let (alice_machine, bob) = setup_verification_machine().await;
 
@@ -612,7 +580,7 @@ mod test {
         assert!(!alice_machine.verifications.outgoing_requests().is_empty());
 
         let request = alice_machine.verifications.outgoing_requests().get(0).cloned().unwrap();
-        let txn_id = *request.request_id();
+        let txn_id = request.request_id().to_owned();
         let content = OutgoingContent::try_from(request).unwrap();
         let content = KeyContent::try_from(&content).unwrap().into();
 
@@ -624,12 +592,16 @@ mod test {
         assert!(bob.emoji().is_some());
         assert_eq!(alice.emoji(), bob.emoji());
 
-        let request = alice.confirm().await.unwrap().0.unwrap();
+        let mut requests = alice.confirm().await.unwrap().0;
+        assert!(requests.len() == 1);
+        let request = requests.pop().unwrap();
         let content = OutgoingContent::try_from(request).unwrap();
         let content = MacContent::try_from(&content).unwrap().into();
         bob.receive_any_event(alice.user_id(), &content);
 
-        let request = bob.confirm().await.unwrap().0.unwrap();
+        let mut requests = bob.confirm().await.unwrap().0;
+        assert!(requests.len() == 1);
+        let request = requests.pop().unwrap();
         let content = OutgoingContent::try_from(request).unwrap();
         let content = MacContent::try_from(&content).unwrap().into();
         alice.receive_any_event(bob.user_id(), &content);
@@ -638,8 +610,8 @@ mod test {
         assert!(bob.is_done());
     }
 
-    #[cfg(target_os = "linux")]
-    #[tokio::test]
+    #[cfg(not(target_os = "macos"))]
+    #[async_test]
     async fn timing_out() {
         let (alice_machine, bob) = setup_verification_machine().await;
         let alice = alice_machine.get_sas(bob.user_id(), bob.flow_id().as_str()).unwrap();
