@@ -29,20 +29,26 @@ use event_enums::OutgoingContent;
 pub use machine::VerificationMachine;
 use matrix_sdk_common::locks::Mutex;
 #[cfg(feature = "qrcode")]
-#[cfg_attr(feature = "docs", doc(cfg(qrcode)))]
-pub use qrcode::QrVerification;
+pub use qrcode::{QrVerification, ScanError};
 pub use requests::VerificationRequest;
+#[cfg(feature = "qrcode")]
+use ruma::events::key::verification::done::{
+    KeyVerificationDoneEventContent, ToDeviceKeyVerificationDoneEventContent,
+};
 use ruma::{
-    api::client::r0::keys::upload_signatures::Request as SignatureUploadRequest,
+    api::client::keys::upload_signatures::v3::Request as SignatureUploadRequest,
     events::{
         key::verification::{
-            cancel::{CancelCode, CancelEventContent, CancelToDeviceEventContent},
-            done::{DoneEventContent, DoneToDeviceEventContent},
+            cancel::{
+                CancelCode, KeyVerificationCancelEventContent,
+                ToDeviceKeyVerificationCancelEventContent,
+            },
             Relation,
         },
-        AnyMessageEventContent, AnyToDeviceEventContent,
+        AnyMessageLikeEventContent, AnyToDeviceEventContent,
     },
-    DeviceId, DeviceIdBox, DeviceKeyId, EventId, RoomId, UserId,
+    DeviceId, EventId, OwnedDeviceId, OwnedDeviceKeyId, OwnedEventId, OwnedRoomId,
+    OwnedTransactionId, OwnedUserId, RoomId, UserId,
 };
 pub use sas::{AcceptSettings, Sas};
 use tracing::{error, info, trace, warn};
@@ -52,15 +58,32 @@ use crate::{
     gossiping::{GossipMachine, GossipRequest},
     olm::{PrivateCrossSigningIdentity, ReadOnlyAccount, Session},
     store::{Changes, CryptoStore},
-    CryptoStoreError, LocalTrust, ReadOnlyDevice, ReadOnlyUserIdentities,
+    CryptoStoreError, LocalTrust, ReadOnlyDevice, ReadOnlyOwnUserIdentity, ReadOnlyUserIdentities,
 };
 
 #[derive(Clone, Debug)]
 pub(crate) struct VerificationStore {
     pub account: ReadOnlyAccount,
+    pub private_identity: Arc<Mutex<PrivateCrossSigningIdentity>>,
     inner: Arc<dyn CryptoStore>,
 }
 
+/// An emoji that is used for interactive verification using a short auth
+/// string.
+///
+/// This will contain a single emoji and description from the list of emojis
+/// from the [spec].
+///
+/// [spec]: https://spec.matrix.org/unstable/client-server-api/#sas-method-emoji
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd)]
+pub struct Emoji {
+    /// The emoji symbol that represents a part of the short auth string, for
+    /// example: 🐶
+    pub symbol: &'static str,
+    /// The description of the emoji, for example 'Dog'.
+    pub description: &'static str,
+}
+
 impl VerificationStore {
     pub async fn get_device(
         &self,
@@ -79,6 +102,25 @@ impl VerificationStore {
         self.inner.get_user_identity(user_id).await
     }
 
+    pub async fn get_identities(
+        &self,
+        device_being_verified: ReadOnlyDevice,
+    ) -> Result<IdentitiesBeingVerified, CryptoStoreError> {
+        let identity_being_verified =
+            self.get_user_identity(device_being_verified.user_id()).await?;
+
+        Ok(IdentitiesBeingVerified {
+            private_identity: self.private_identity.lock().await.clone(),
+            store: self.clone(),
+            device_being_verified,
+            own_identity: self
+                .get_user_identity(self.account.user_id())
+                .await?
+                .and_then(|i| i.into_own()),
+            identity_being_verified,
+        })
+    }
+
     pub async fn save_changes(&self, changes: Changes) -> Result<(), CryptoStoreError> {
         self.inner.save_changes(changes).await
     }
@@ -86,7 +128,7 @@ impl VerificationStore {
     pub async fn get_user_devices(
         &self,
         user_id: &UserId,
-    ) -> Result<HashMap<DeviceIdBox, ReadOnlyDevice>, CryptoStoreError> {
+    ) -> Result<HashMap<OwnedDeviceId, ReadOnlyDevice>, CryptoStoreError> {
         self.inner.get_user_devices(user_id).await
     }
 
@@ -100,7 +142,8 @@ impl VerificationStore {
     /// Get the signatures that have signed our own device.
     pub async fn device_signatures(
         &self,
-    ) -> Result<Option<BTreeMap<UserId, BTreeMap<DeviceKeyId, String>>>, CryptoStoreError> {
+    ) -> Result<Option<BTreeMap<OwnedUserId, BTreeMap<OwnedDeviceKeyId, String>>>, CryptoStoreError>
+    {
         Ok(self
             .inner
             .get_device(self.account.user_id(), self.account.device_id())
@@ -115,12 +158,12 @@ impl VerificationStore {
 
 /// An enum over the different verification types the SDK supports.
 #[derive(Clone, Debug)]
+#[non_exhaustive]
 pub enum Verification {
     /// The `m.sas.v1` verification variant.
     SasV1(Sas),
-    #[cfg(feature = "qrcode")]
-    #[cfg_attr(feature = "docs", doc(cfg(qrcode)))]
     /// The `m.qr_code.*.v1` verification variant.
+    #[cfg(feature = "qrcode")]
     QrV1(QrVerification),
 }
 
@@ -135,9 +178,8 @@ impl Verification {
         }
     }
 
-    #[cfg(feature = "qrcode")]
-    #[cfg_attr(feature = "docs", doc(cfg(qrcode)))]
     /// Try to deconstruct this verification enum into a QR code verification.
+    #[cfg(feature = "qrcode")]
     pub fn qr_v1(self) -> Option<QrVerification> {
         if let Verification::QrV1(qr) = self {
             Some(qr)
@@ -208,7 +250,6 @@ impl From<Sas> for Verification {
 }
 
 #[cfg(feature = "qrcode")]
-#[cfg_attr(feature = "docs", doc(cfg(qrcode)))]
 impl From<QrVerification> for Verification {
     fn from(qr: QrVerification) -> Self {
         Self::QrV1(qr)
@@ -220,24 +261,26 @@ impl From<QrVerification> for Verification {
 ///
 /// We can now mark the device in our verified devices list as verified and sign
 /// the master keys in the verified devices list.
+#[cfg(feature = "qrcode")]
 #[derive(Clone, Debug)]
 pub struct Done {
     verified_devices: Arc<[ReadOnlyDevice]>,
     verified_master_keys: Arc<[ReadOnlyUserIdentities]>,
 }
 
+#[cfg(feature = "qrcode")]
 impl Done {
     pub fn as_content(&self, flow_id: &FlowId) -> OutgoingContent {
         match flow_id {
             FlowId::ToDevice(t) => AnyToDeviceEventContent::KeyVerificationDone(
-                DoneToDeviceEventContent::new(t.to_owned()),
+                ToDeviceKeyVerificationDoneEventContent::new(t.to_owned()),
             )
             .into(),
             FlowId::InRoom(r, e) => (
                 r.to_owned(),
-                AnyMessageEventContent::KeyVerificationDone(DoneEventContent::new(Relation::new(
-                    e.to_owned(),
-                ))),
+                AnyMessageLikeEventContent::KeyVerificationDone(
+                    KeyVerificationDoneEventContent::new(Relation::new(e.to_owned())),
+                ),
             )
                 .into(),
         }
@@ -309,22 +352,24 @@ impl Cancelled {
 
     pub fn as_content(&self, flow_id: &FlowId) -> OutgoingContent {
         match flow_id {
-            FlowId::ToDevice(s) => {
-                AnyToDeviceEventContent::KeyVerificationCancel(CancelToDeviceEventContent::new(
+            FlowId::ToDevice(s) => AnyToDeviceEventContent::KeyVerificationCancel(
+                ToDeviceKeyVerificationCancelEventContent::new(
                     s.clone(),
-                    self.reason.to_string(),
+                    self.reason.to_owned(),
                     self.cancel_code.clone(),
-                ))
-                .into()
-            }
+                ),
+            )
+            .into(),
 
             FlowId::InRoom(r, e) => (
                 r.clone(),
-                AnyMessageEventContent::KeyVerificationCancel(CancelEventContent::new(
-                    self.reason.to_string(),
-                    self.cancel_code.clone(),
-                    Relation::new(e.clone()),
-                )),
+                AnyMessageLikeEventContent::KeyVerificationCancel(
+                    KeyVerificationCancelEventContent::new(
+                        self.reason.to_owned(),
+                        self.cancel_code.clone(),
+                        Relation::new(e.clone()),
+                    ),
+                ),
             )
                 .into(),
         }
@@ -333,8 +378,8 @@ impl Cancelled {
 
 #[derive(Clone, Debug, Hash, PartialEq, PartialOrd)]
 pub enum FlowId {
-    ToDevice(String),
-    InRoom(RoomId, EventId),
+    ToDevice(OwnedTransactionId),
+    InRoom(OwnedRoomId, OwnedEventId),
 }
 
 impl FlowId {
@@ -354,14 +399,14 @@ impl FlowId {
     }
 }
 
-impl From<String> for FlowId {
-    fn from(transaction_id: String) -> Self {
+impl From<OwnedTransactionId> for FlowId {
+    fn from(transaction_id: OwnedTransactionId) -> Self {
         FlowId::ToDevice(transaction_id)
     }
 }
 
-impl From<(RoomId, EventId)> for FlowId {
-    fn from(ids: (RoomId, EventId)) -> Self {
+impl From<(OwnedRoomId, OwnedEventId)> for FlowId {
+    fn from(ids: (OwnedRoomId, OwnedEventId)) -> Self {
         FlowId::InRoom(ids.0, ids.1)
     }
 }
@@ -388,6 +433,7 @@ pub struct IdentitiesBeingVerified {
     private_identity: PrivateCrossSigningIdentity,
     store: VerificationStore,
     device_being_verified: ReadOnlyDevice,
+    own_identity: Option<ReadOnlyOwnUserIdentity>,
     identity_being_verified: Option<ReadOnlyUserIdentities>,
 }
 
@@ -510,16 +556,15 @@ impl IdentitiesBeingVerified {
         let merged_request = if let Some(mut r) = signature_request {
             if let Some(user_request) = identity_signature_request {
                 r.signed_keys.extend(user_request.signed_keys);
-                Some(r)
-            } else {
-                Some(r)
             }
+
+            Some(r)
         } else {
             identity_signature_request
         };
 
         if should_request_secrets {
-            let secret_requests = self.request_missing_secrets().await;
+            let secret_requests = self.request_missing_secrets().await?;
             changes.key_requests = secret_requests;
         }
 
@@ -531,9 +576,16 @@ impl IdentitiesBeingVerified {
             .unwrap_or(VerificationResult::Ok))
     }
 
-    async fn request_missing_secrets(&self) -> Vec<GossipRequest> {
-        let secrets = self.private_identity.get_missing_secrets().await;
-        GossipMachine::request_missing_secrets(self.user_id(), secrets)
+    async fn request_missing_secrets(&self) -> Result<Vec<GossipRequest>, CryptoStoreError> {
+        #[allow(unused_mut)]
+        let mut secrets = self.private_identity.get_missing_secrets().await;
+
+        #[cfg(feature = "backups_v1")]
+        if self.store.inner.load_backup_keys().await?.recovery_key.is_none() {
+            secrets.push(ruma::events::secret::request::SecretName::RecoveryKey);
+        }
+
+        Ok(GossipMachine::request_missing_secrets(self.user_id(), secrets))
     }
 
     async fn mark_identity_as_verified(
@@ -650,7 +702,7 @@ impl IdentitiesBeingVerified {
 }
 
 #[cfg(test)]
-pub(crate) mod test {
+pub(crate) mod tests {
     use std::convert::TryInto;
 
     use ruma::{
@@ -688,31 +740,29 @@ pub(crate) mod test {
         content: OutgoingContent,
     ) -> AnyToDeviceEvent {
         let content = if let OutgoingContent::ToDevice(c) = content { c } else { unreachable!() };
+        let sender = sender.to_owned();
 
         match content {
+            AnyToDeviceEventContent::KeyVerificationRequest(c) => {
+                AnyToDeviceEvent::KeyVerificationRequest(ToDeviceEvent { sender, content: c })
+            }
+            AnyToDeviceEventContent::KeyVerificationReady(c) => {
+                AnyToDeviceEvent::KeyVerificationReady(ToDeviceEvent { sender, content: c })
+            }
             AnyToDeviceEventContent::KeyVerificationKey(c) => {
-                AnyToDeviceEvent::KeyVerificationKey(ToDeviceEvent {
-                    sender: sender.clone(),
-                    content: c,
-                })
+                AnyToDeviceEvent::KeyVerificationKey(ToDeviceEvent { sender, content: c })
             }
             AnyToDeviceEventContent::KeyVerificationStart(c) => {
-                AnyToDeviceEvent::KeyVerificationStart(ToDeviceEvent {
-                    sender: sender.clone(),
-                    content: c,
-                })
+                AnyToDeviceEvent::KeyVerificationStart(ToDeviceEvent { sender, content: c })
             }
             AnyToDeviceEventContent::KeyVerificationAccept(c) => {
-                AnyToDeviceEvent::KeyVerificationAccept(ToDeviceEvent {
-                    sender: sender.clone(),
-                    content: c,
-                })
+                AnyToDeviceEvent::KeyVerificationAccept(ToDeviceEvent { sender, content: c })
             }
             AnyToDeviceEventContent::KeyVerificationMac(c) => {
-                AnyToDeviceEvent::KeyVerificationMac(ToDeviceEvent {
-                    sender: sender.clone(),
-                    content: c,
-                })
+                AnyToDeviceEvent::KeyVerificationMac(ToDeviceEvent { sender, content: c })
+            }
+            AnyToDeviceEventContent::KeyVerificationDone(c) => {
+                AnyToDeviceEvent::KeyVerificationDone(ToDeviceEvent { sender, content: c })
             }
 
             _ => unreachable!(),
