@@ -14,29 +14,31 @@
 
 use std::sync::{Arc, Mutex};
 
-use matrix_qrcode::{
+use matrix_sdk_qrcode::{
     qrcode::QrCode, EncodingError, QrVerificationData, SelfVerificationData,
     SelfVerificationNoMasterKey, VerificationData,
 };
-use matrix_sdk_common::uuid::Uuid;
+use rand::{thread_rng, RngCore};
 use ruma::{
-    api::client::r0::keys::upload_signatures::Request as SignatureUploadRequest,
+    api::client::keys::upload_signatures::v3::Request as SignatureUploadRequest,
     events::{
         key::verification::{
             cancel::CancelCode,
-            done::{DoneEventContent, DoneToDeviceEventContent},
+            done::{KeyVerificationDoneEventContent, ToDeviceKeyVerificationDoneEventContent},
             start::{
-                self, ReciprocateV1Content, StartEventContent, StartMethod,
-                StartToDeviceEventContent,
+                self, KeyVerificationStartEventContent, ReciprocateV1Content, StartMethod,
+                ToDeviceKeyVerificationStartEventContent,
             },
             Relation,
         },
-        AnyMessageEventContent, AnyToDeviceEventContent,
+        AnyMessageLikeEventContent, AnyToDeviceEventContent,
     },
-    DeviceId, DeviceIdBox, DeviceKeyAlgorithm, RoomId, UserId,
+    serde::Base64,
+    DeviceId, OwnedDeviceId, OwnedUserId, RoomId, TransactionId, UserId,
 };
 use thiserror::Error;
 use tracing::trace;
+use vodozemac::Ed25519PublicKey;
 
 use super::{
     event_enums::{CancelContent, DoneContent, OutgoingContent, OwnedStartContent, StartContent},
@@ -45,8 +47,8 @@ use super::{
     VerificationStore,
 };
 use crate::{
-    olm::PrivateCrossSigningIdentity, CryptoStoreError, OutgoingVerificationRequest,
-    ReadOnlyDevice, ReadOnlyUserIdentities, RoomMessageRequest, ToDeviceRequest,
+    CryptoStoreError, OutgoingVerificationRequest, ReadOnlyDevice, ReadOnlyUserIdentities,
+    RoomMessageRequest, ToDeviceRequest,
 };
 
 const SECRET_SIZE: usize = 16;
@@ -61,19 +63,29 @@ pub enum ScanError {
     Store(#[from] CryptoStoreError),
     /// A key mismatch happened during the validation of the QR code scan.
     #[error("The keys that are being verified didn't match (expected {expected}, found {found})")]
-    KeyMismatch { expected: String, found: String },
+    KeyMismatch {
+        /// The expected ed25519 key.
+        expected: String,
+        /// The ed25519 key that we got.
+        found: String,
+    },
     /// One of the users that is participating in this verification doesn't have
     /// a valid cross signing identity.
     #[error("The user {0} is missing a valid cross signing identity")]
-    MissingCrossSigningIdentity(UserId),
+    MissingCrossSigningIdentity(OwnedUserId),
     /// The device of the user that is participating in this verification
     /// doesn't have a valid device key.
     #[error("The user's {0} device {1} is not E2E capable")]
-    MissingDeviceKeys(UserId, DeviceIdBox),
+    MissingDeviceKeys(OwnedUserId, OwnedDeviceId),
     /// The ID uniquely identifying this verification flow didn't match to the
     /// one that has been scanned.
     #[error("The unique verification flow id did not match (expected {expected}, found {found})")]
-    FlowIdMismatch { expected: String, found: String },
+    FlowIdMismatch {
+        /// The expected verification flow id.
+        expected: String,
+        /// The verification flow id that we instead got.
+        found: String,
+    },
 }
 
 /// An object controlling QR code style key verification flows.
@@ -206,7 +218,7 @@ impl QrVerification {
     ///
     /// **Note**: This method should generally not be used, the [`cancel()`]
     /// method should be preferred. The SDK will automatically cancel with the
-    /// approprate cancel code, user initiated cancellations should only cancel
+    /// appropriate cancel code, user initiated cancellations should only cancel
     /// with the `CancelCode::User`
     ///
     /// Returns None if the `Sas` object is already in a canceled state,
@@ -277,7 +289,7 @@ impl QrVerification {
     fn content_to_request(&self, content: OutgoingContent) -> OutgoingVerificationRequest {
         match content {
             OutgoingContent::Room(room_id, content) => {
-                RoomMessageRequest { room_id, txn_id: Uuid::new_v4(), content }.into()
+                RoomMessageRequest { room_id, txn_id: TransactionId::new(), content }.into()
             }
             OutgoingContent::ToDevice(c) => ToDeviceRequest::new(
                 self.identities.other_user_id(),
@@ -374,7 +386,7 @@ impl QrVerification {
 
     pub(crate) fn receive_reciprocation(
         &self,
-        content: &StartContent,
+        content: &StartContent<'_>,
     ) -> Option<OutgoingVerificationRequest> {
         let mut state = self.state.lock().unwrap();
 
@@ -420,17 +432,18 @@ impl QrVerification {
         }
     }
 
-    fn generate_secret() -> String {
-        let mut shared_secret = [0u8; SECRET_SIZE];
-        getrandom::getrandom(&mut shared_secret)
-            .expect("Can't generate randomness for the shared secret");
-        crate::utilities::encode(shared_secret)
+    fn generate_secret() -> Base64 {
+        let mut shared_secret = vec![0u8; SECRET_SIZE];
+        let mut rng = thread_rng();
+        rng.fill_bytes(&mut shared_secret);
+
+        Base64::new(shared_secret)
     }
 
     pub(crate) fn new_self(
         flow_id: FlowId,
-        own_master_key: String,
-        other_device_key: String,
+        own_master_key: Ed25519PublicKey,
+        other_device_key: Ed25519PublicKey,
         identities: IdentitiesBeingVerified,
         we_started: bool,
         request_handle: Option<RequestHandle>,
@@ -451,7 +464,7 @@ impl QrVerification {
     pub(crate) fn new_self_no_master(
         store: VerificationStore,
         flow_id: FlowId,
-        own_master_key: String,
+        own_master_key: Ed25519PublicKey,
         identities: IdentitiesBeingVerified,
         we_started: bool,
         request_handle: Option<RequestHandle>,
@@ -460,7 +473,7 @@ impl QrVerification {
 
         let inner: QrVerificationData = SelfVerificationNoMasterKey::new(
             flow_id.as_str().to_owned(),
-            store.account.identity_keys().ed25519().to_string(),
+            store.account.identity_keys().ed25519,
             own_master_key,
             secret,
         )
@@ -471,8 +484,8 @@ impl QrVerification {
 
     pub(crate) fn new_cross(
         flow_id: FlowId,
-        own_master_key: String,
-        other_master_key: String,
+        own_master_key: Ed25519PublicKey,
+        other_master_key: Ed25519PublicKey,
         identities: IdentitiesBeingVerified,
         we_started: bool,
         request_handle: Option<RequestHandle>,
@@ -491,12 +504,10 @@ impl QrVerification {
         Self::new_helper(flow_id, inner, identities, we_started, request_handle)
     }
 
-    #[allow(clippy::too_many_arguments)]
     pub(crate) async fn from_scan(
         store: VerificationStore,
-        private_identity: PrivateCrossSigningIdentity,
-        other_user_id: UserId,
-        other_device_id: DeviceIdBox,
+        other_user_id: OwnedUserId,
+        other_device_id: OwnedDeviceId,
         flow_id: FlowId,
         qr_code: QrVerificationData,
         we_started: bool,
@@ -509,75 +520,67 @@ impl QrVerification {
             });
         }
 
-        let own_identity =
-            store.get_user_identity(store.account.user_id()).await?.ok_or_else(|| {
-                ScanError::MissingCrossSigningIdentity(store.account.user_id().to_owned())
-            })?;
-        let other_identity = store
-            .get_user_identity(&other_user_id)
-            .await?
-            .ok_or_else(|| ScanError::MissingCrossSigningIdentity(other_user_id.clone()))?;
         let other_device =
             store.get_device(&other_user_id, &other_device_id).await?.ok_or_else(|| {
                 ScanError::MissingDeviceKeys(other_user_id.clone(), other_device_id.clone())
             })?;
 
+        let identities = store.get_identities(other_device).await?;
+
+        let own_identity = identities.own_identity.as_ref().ok_or_else(|| {
+            ScanError::MissingCrossSigningIdentity(store.account.user_id().to_owned())
+        })?;
+
+        let other_identity = identities
+            .identity_being_verified
+            .as_ref()
+            .ok_or_else(|| ScanError::MissingCrossSigningIdentity(other_user_id.clone()))?;
+
         let check_master_key = |key, identity: &ReadOnlyUserIdentities| {
             let master_key = identity.master_key().get_first_key().ok_or_else(|| {
-                ScanError::MissingCrossSigningIdentity(identity.user_id().clone())
+                ScanError::MissingCrossSigningIdentity(identity.user_id().to_owned())
             })?;
 
             if key != master_key {
                 Err(ScanError::KeyMismatch {
-                    expected: master_key.to_owned(),
-                    found: qr_code.first_key().to_owned(),
+                    expected: master_key.to_base64(),
+                    found: qr_code.first_key().to_base64(),
                 })
             } else {
                 Ok(())
             }
         };
 
-        let (device_being_verified, identity_being_verified) = match qr_code {
+        match qr_code {
             QrVerificationData::Verification(_) => {
-                check_master_key(qr_code.first_key(), &other_identity)?;
-                check_master_key(qr_code.second_key(), &own_identity)?;
-
-                (other_device, Some(other_identity))
+                check_master_key(qr_code.first_key(), other_identity)?;
+                check_master_key(qr_code.second_key(), &own_identity.to_owned().into())?;
             }
             QrVerificationData::SelfVerification(_) => {
-                check_master_key(qr_code.first_key(), &other_identity)?;
-                if qr_code.second_key() != store.account.identity_keys().ed25519() {
+                check_master_key(qr_code.first_key(), other_identity)?;
+                if qr_code.second_key() != store.account.identity_keys().ed25519 {
                     return Err(ScanError::KeyMismatch {
-                        expected: store.account.identity_keys().ed25519().to_owned(),
-                        found: qr_code.second_key().to_owned(),
+                        expected: store.account.identity_keys().ed25519.to_base64(),
+                        found: qr_code.second_key().to_base64(),
                     });
                 }
-
-                (other_device, Some(other_identity))
             }
             QrVerificationData::SelfVerificationNoMasterKey(_) => {
                 let device_key =
-                    other_device.get_key(DeviceKeyAlgorithm::Ed25519).ok_or_else(|| {
+                    identities.device_being_verified.ed25519_key().ok_or_else(|| {
                         ScanError::MissingDeviceKeys(other_user_id.clone(), other_device_id.clone())
                     })?;
+
                 if qr_code.first_key() != device_key {
                     return Err(ScanError::KeyMismatch {
-                        expected: device_key.to_owned(),
-                        found: qr_code.first_key().to_owned(),
+                        expected: device_key.to_base64(),
+                        found: qr_code.first_key().to_base64(),
                     });
                 }
-                check_master_key(qr_code.second_key(), &other_identity)?;
-                (other_device, Some(other_identity))
+                check_master_key(qr_code.second_key(), other_identity)?;
             }
         };
 
-        let identities = IdentitiesBeingVerified {
-            private_identity,
-            store: store.clone(),
-            device_being_verified,
-            identity_being_verified,
-        };
-
         let secret = qr_code.secret().to_owned();
         let own_device_id = store.account.device_id().to_owned();
 
@@ -637,7 +640,7 @@ impl<S: Clone> QrState<S> {
 
 #[derive(Clone, Debug)]
 struct Created {
-    secret: String,
+    secret: Base64,
 }
 
 #[derive(Clone, Debug)]
@@ -648,8 +651,8 @@ struct Confirmed {}
 
 #[derive(Clone, Debug)]
 struct Reciprocated {
-    own_device_id: DeviceIdBox,
-    secret: String,
+    own_device_id: OwnedDeviceId,
+    secret: Base64,
 }
 
 impl Reciprocated {
@@ -658,12 +661,15 @@ impl Reciprocated {
         let method = StartMethod::ReciprocateV1(content);
 
         let content: OwnedStartContent = match flow_id {
-            FlowId::ToDevice(t) => {
-                StartToDeviceEventContent::new(self.own_device_id.clone(), t.clone(), method).into()
-            }
+            FlowId::ToDevice(t) => ToDeviceKeyVerificationStartEventContent::new(
+                self.own_device_id.clone(),
+                t.clone(),
+                method,
+            )
+            .into(),
             FlowId::InRoom(r, e) => (
                 r.clone(),
-                StartEventContent::new(
+                KeyVerificationStartEventContent::new(
                     self.own_device_id.clone(),
                     method,
                     Relation::new(e.clone()),
@@ -695,7 +701,7 @@ impl QrState<Cancelled> {
 impl QrState<Created> {
     fn receive_reciprocate(
         self,
-        content: &StartContent,
+        content: &StartContent<'_>,
     ) -> Result<QrState<Scanned>, QrState<Cancelled>> {
         match content.method() {
             start::StartMethod::ReciprocateV1(m) => {
@@ -724,7 +730,7 @@ impl QrState<Done> {
 impl QrState<Confirmed> {
     fn into_done(
         self,
-        _: &DoneContent,
+        _: &DoneContent<'_>,
         verified_device: Option<&ReadOnlyDevice>,
         verified_identity: Option<&ReadOnlyUserIdentities>,
     ) -> QrState<Done> {
@@ -742,14 +748,14 @@ impl QrState<Confirmed> {
     fn as_content(&self, flow_id: &FlowId) -> OutgoingContent {
         match flow_id {
             FlowId::ToDevice(t) => AnyToDeviceEventContent::KeyVerificationDone(
-                DoneToDeviceEventContent::new(t.to_owned()),
+                ToDeviceKeyVerificationDoneEventContent::new(t.to_owned()),
             )
             .into(),
             FlowId::InRoom(r, e) => (
                 r.to_owned(),
-                AnyMessageEventContent::KeyVerificationDone(DoneEventContent::new(Relation::new(
-                    e.to_owned(),
-                ))),
+                AnyMessageLikeEventContent::KeyVerificationDone(
+                    KeyVerificationDoneEventContent::new(Relation::new(e.to_owned())),
+                ),
             )
                 .into(),
         }
@@ -763,7 +769,7 @@ impl QrState<Reciprocated> {
 
     fn into_done(
         self,
-        _: &DoneContent,
+        _: &DoneContent<'_>,
         verified_device: Option<&ReadOnlyDevice>,
         verified_identity: Option<&ReadOnlyUserIdentities>,
     ) -> QrState<Done> {
@@ -780,24 +786,25 @@ impl QrState<Reciprocated> {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use std::{convert::TryFrom, sync::Arc};
 
-    use matrix_qrcode::QrVerificationData;
+    use matrix_sdk_common::locks::Mutex;
+    use matrix_sdk_qrcode::QrVerificationData;
     use matrix_sdk_test::async_test;
-    use ruma::{event_id, room_id, user_id, DeviceIdBox, UserId};
+    use ruma::{device_id, event_id, room_id, user_id, DeviceId, UserId};
 
     use crate::{
         olm::{PrivateCrossSigningIdentity, ReadOnlyAccount},
         store::{Changes, CryptoStore, MemoryStore},
         verification::{
             event_enums::{DoneContent, OutgoingContent, StartContent},
-            FlowId, IdentitiesBeingVerified, VerificationStore,
+            FlowId, VerificationStore,
         },
         QrVerification, ReadOnlyDevice,
     };
 
-    fn user_id() -> UserId {
+    fn user_id() -> &'static UserId {
         user_id!("@example:localhost")
     }
 
@@ -805,87 +812,89 @@ mod test {
         Arc::new(MemoryStore::new())
     }
 
-    fn device_id() -> DeviceIdBox {
-        "DEVICEID".into()
+    fn device_id() -> &'static DeviceId {
+        device_id!("DEVICEID")
     }
 
     #[async_test]
     async fn test_verification_creation() {
         let store = memory_store();
-        let account = ReadOnlyAccount::new(&user_id(), &device_id());
+        let account = ReadOnlyAccount::new(user_id(), device_id());
 
-        let store = VerificationStore { account: account.clone(), inner: store };
-
-        let private_identity = PrivateCrossSigningIdentity::new(user_id()).await;
-        let flow_id = FlowId::ToDevice("test_transaction".to_owned());
-
-        let device_key = account.identity_keys().ed25519().to_owned();
+        let private_identity = PrivateCrossSigningIdentity::new(user_id().to_owned()).await;
         let master_key = private_identity.master_public_key().await.unwrap();
         let master_key = master_key.get_first_key().unwrap().to_owned();
 
+        let store = VerificationStore {
+            account: account.clone(),
+            inner: store,
+            private_identity: Mutex::new(private_identity).into(),
+        };
+
+        let flow_id = FlowId::ToDevice("test_transaction".into());
+
+        let device_key = account.identity_keys.ed25519;
         let alice_device = ReadOnlyDevice::from_account(&account).await;
 
-        let identities = IdentitiesBeingVerified {
-            private_identity,
-            store: store.clone(),
-            device_being_verified: alice_device,
-            identity_being_verified: None,
-        };
+        let identities = store.get_identities(alice_device).await.unwrap();
 
         let verification = QrVerification::new_self_no_master(
             store.clone(),
             flow_id.clone(),
-            master_key.clone(),
+            master_key,
             identities.clone(),
             false,
             None,
         );
 
-        assert_eq!(verification.inner.first_key(), &device_key);
-        assert_eq!(verification.inner.second_key(), &master_key);
+        assert_eq!(verification.inner.first_key(), device_key);
+        assert_eq!(verification.inner.second_key(), master_key);
 
         let verification = QrVerification::new_self(
             flow_id,
-            master_key.clone(),
-            device_key.clone(),
+            master_key,
+            device_key,
             identities.clone(),
             false,
             None,
         );
 
-        assert_eq!(verification.inner.first_key(), &master_key);
-        assert_eq!(verification.inner.second_key(), &device_key);
+        assert_eq!(verification.inner.first_key(), master_key);
+        assert_eq!(verification.inner.second_key(), device_key);
 
-        let bob_identity = PrivateCrossSigningIdentity::new(user_id!("@bob:example")).await;
+        let bob_identity =
+            PrivateCrossSigningIdentity::new(user_id!("@bob:example").to_owned()).await;
         let bob_master_key = bob_identity.master_public_key().await.unwrap();
         let bob_master_key = bob_master_key.get_first_key().unwrap().to_owned();
 
-        let flow_id = FlowId::InRoom(room_id!("!test:example"), event_id!("$EVENTID"));
+        let flow_id =
+            FlowId::InRoom(room_id!("!test:example").to_owned(), event_id!("$EVENTID").to_owned());
 
-        let verification = QrVerification::new_cross(
-            flow_id,
-            master_key.clone(),
-            bob_master_key.clone(),
-            identities,
-            false,
-            None,
-        );
+        let verification =
+            QrVerification::new_cross(flow_id, master_key, bob_master_key, identities, false, None);
 
-        assert_eq!(verification.inner.first_key(), &master_key);
-        assert_eq!(verification.inner.second_key(), &bob_master_key);
+        assert_eq!(verification.inner.first_key(), master_key);
+        assert_eq!(verification.inner.second_key(), bob_master_key);
     }
 
     #[async_test]
     async fn test_reciprocate_receival() {
         let test = |flow_id: FlowId| async move {
-            let alice_account = ReadOnlyAccount::new(&user_id(), &device_id());
+            let alice_account = ReadOnlyAccount::new(user_id(), device_id());
             let store = memory_store();
 
-            let store = VerificationStore { account: alice_account.clone(), inner: store };
+            let private_identity = PrivateCrossSigningIdentity::new(user_id().to_owned()).await;
 
-            let bob_account = ReadOnlyAccount::new(alice_account.user_id(), "BOBDEVICE".into());
+            let store = VerificationStore {
+                account: alice_account.clone(),
+                inner: store,
+                private_identity: Mutex::new(private_identity).into(),
+            };
 
-            let private_identity = PrivateCrossSigningIdentity::new(user_id()).await;
+            let bob_account =
+                ReadOnlyAccount::new(alice_account.user_id(), device_id!("BOBDEVICE"));
+
+            let private_identity = PrivateCrossSigningIdentity::new(user_id().to_owned()).await;
             let identity = private_identity.to_public_identity().await.unwrap();
 
             let master_key = private_identity.master_public_key().await.unwrap();
@@ -899,19 +908,12 @@ mod test {
             changes.devices.new.push(bob_device.clone());
             store.save_changes(changes).await.unwrap();
 
-            let identities = IdentitiesBeingVerified {
-                private_identity: PrivateCrossSigningIdentity::empty(
-                    alice_account.user_id().to_owned(),
-                ),
-                store: store.clone(),
-                device_being_verified: alice_device.clone(),
-                identity_being_verified: Some(identity.clone().into()),
-            };
+            let identities = store.get_identities(alice_device.clone()).await.unwrap();
 
             let alice_verification = QrVerification::new_self_no_master(
                 store,
                 flow_id.clone(),
-                master_key.clone(),
+                master_key,
                 identities,
                 false,
                 None,
@@ -919,7 +921,12 @@ mod test {
 
             let bob_store = memory_store();
 
-            let bob_store = VerificationStore { account: bob_account.clone(), inner: bob_store };
+            let private_identity = PrivateCrossSigningIdentity::new(user_id().to_owned()).await;
+            let bob_store = VerificationStore {
+                account: bob_account.clone(),
+                inner: bob_store,
+                private_identity: Mutex::new(private_identity).into(),
+            };
 
             let mut changes = Changes::default();
             changes.identities.new.push(identity.into());
@@ -931,7 +938,6 @@ mod test {
 
             let bob_verification = QrVerification::from_scan(
                 bob_store,
-                private_identity,
                 alice_account.user_id().to_owned(),
                 alice_account.device_id().to_owned(),
                 flow_id,
@@ -978,10 +984,11 @@ mod test {
             assert!(identity.is_verified());
         };
 
-        let flow_id = FlowId::ToDevice("test_transaction".to_owned());
+        let flow_id = FlowId::ToDevice("test_transaction".into());
         test(flow_id).await;
 
-        let flow_id = FlowId::InRoom(room_id!("!test:example"), event_id!("$EVENTID"));
+        let flow_id =
+            FlowId::InRoom(room_id!("!test:example").to_owned(), event_id!("$EVENTID").to_owned());
         test(flow_id).await;
     }
 }
