@@ -13,18 +13,18 @@
 // limitations under the License.
 
 use std::sync::Arc;
-#[cfg(test)]
-use std::time::Instant;
 
+#[cfg(test)]
+use matrix_sdk_common::instant::Instant;
 use ruma::{
     events::key::verification::{cancel::CancelCode, ShortAuthenticationString},
-    EventId, RoomId, UserId,
+    UserId,
 };
 
 use super::{
     sas_state::{
-        Accepted, Confirmed, Created, KeyReceived, MacReceived, SasState, Started, WaitingForDone,
-        WeAccepted,
+        Accepted, Confirmed, Created, Done, KeyReceived, MacReceived, SasState, Started,
+        WaitingForDone, WeAccepted,
     },
     FlowId,
 };
@@ -32,9 +32,9 @@ use crate::{
     identities::{ReadOnlyDevice, ReadOnlyUserIdentities},
     verification::{
         event_enums::{AnyVerificationContent, OutgoingContent, OwnedAcceptContent, StartContent},
-        Cancelled, Done,
+        Cancelled,
     },
-    ReadOnlyAccount, ReadOnlyOwnUserIdentity,
+    Emoji, ReadOnlyAccount, ReadOnlyOwnUserIdentity,
 };
 
 #[derive(Clone, Debug)]
@@ -57,7 +57,8 @@ impl InnerSas {
         other_device: ReadOnlyDevice,
         own_identity: Option<ReadOnlyOwnUserIdentity>,
         other_identity: Option<ReadOnlyUserIdentities>,
-        transaction_id: Option<String>,
+        transaction_id: FlowId,
+        started_from_request: bool,
     ) -> (InnerSas, OutgoingContent) {
         let sas = SasState::<Created>::new(
             account,
@@ -65,6 +66,7 @@ impl InnerSas {
             own_identity,
             other_identity,
             transaction_id,
+            started_from_request,
         );
         let content = sas.as_content();
         (InnerSas::Created(sas), content.into())
@@ -133,31 +135,11 @@ impl InnerSas {
         }
     }
 
-    pub fn start_in_room(
-        event_id: EventId,
-        room_id: RoomId,
-        account: ReadOnlyAccount,
-        other_device: ReadOnlyDevice,
-        own_identity: Option<ReadOnlyOwnUserIdentity>,
-        other_identity: Option<ReadOnlyUserIdentities>,
-    ) -> (InnerSas, OutgoingContent) {
-        let sas = SasState::<Created>::new_in_room(
-            room_id,
-            event_id,
-            account,
-            other_device,
-            own_identity,
-            other_identity,
-        );
-        let content = sas.as_content();
-        (InnerSas::Created(sas), content.into())
-    }
-
     pub fn from_start_event(
         account: ReadOnlyAccount,
         other_device: ReadOnlyDevice,
         flow_id: FlowId,
-        content: &StartContent,
+        content: &StartContent<'_>,
         own_identity: Option<ReadOnlyOwnUserIdentity>,
         other_identity: Option<ReadOnlyUserIdentities>,
         started_from_request: bool,
@@ -181,7 +163,7 @@ impl InnerSas {
         methods: Vec<ShortAuthenticationString>,
     ) -> Option<(InnerSas, OwnedAcceptContent)> {
         if let InnerSas::Started(s) = self {
-            let sas = s.into_accepted(methods);
+            let sas = s.into_we_accepted(methods);
             let content = sas.as_content();
             Some((InnerSas::WeAccepted(sas), content))
         } else {
@@ -228,52 +210,59 @@ impl InnerSas {
         (InnerSas::Cancelled(sas), Some(content))
     }
 
-    pub fn confirm(self) -> (InnerSas, Option<OutgoingContent>) {
+    pub fn confirm(self) -> (InnerSas, Vec<OutgoingContent>) {
         match self {
             InnerSas::KeyReceived(s) => {
                 let sas = s.confirm();
                 let content = sas.as_content();
-                (InnerSas::Confirmed(sas), Some(content))
+                (InnerSas::Confirmed(sas), vec![content])
             }
             InnerSas::MacReceived(s) => {
                 if s.started_from_request {
                     let sas = s.confirm_and_wait_for_done();
-                    let content = sas.as_content();
+                    let contents = vec![sas.as_content(), sas.done_content()];
 
-                    (InnerSas::WaitingForDone(sas), Some(content))
+                    (InnerSas::WaitingForDone(sas), contents)
                 } else {
                     let sas = s.confirm();
                     let content = sas.as_content();
 
-                    (InnerSas::Done(sas), Some(content))
+                    (InnerSas::Done(sas), vec![content])
                 }
             }
-            _ => (self, None),
+            _ => (self, Vec::new()),
         }
     }
 
     pub fn receive_any_event(
         self,
         sender: &UserId,
-        content: &AnyVerificationContent,
+        content: &AnyVerificationContent<'_>,
     ) -> (Self, Option<OutgoingContent>) {
         match content {
-            AnyVerificationContent::Accept(c) => {
-                if let InnerSas::Created(s) = self {
-                    match s.into_accepted(sender, c) {
-                        Ok(s) => {
-                            let content = s.as_content();
-                            (InnerSas::Accepted(s), Some(content))
-                        }
-                        Err(s) => {
-                            let content = s.as_content();
-                            (InnerSas::Cancelled(s), Some(content))
-                        }
+            AnyVerificationContent::Accept(c) => match self {
+                InnerSas::Created(s) => match s.into_accepted(sender, c) {
+                    Ok(s) => {
+                        let content = s.as_content();
+                        (InnerSas::Accepted(s), Some(content))
                     }
-                } else {
-                    (self, None)
-                }
-            }
+                    Err(s) => {
+                        let content = s.as_content();
+                        (InnerSas::Cancelled(s), Some(content))
+                    }
+                },
+                InnerSas::Started(s) => match s.into_accepted(sender, c) {
+                    Ok(s) => {
+                        let content = s.as_content();
+                        (InnerSas::Accepted(s), Some(content))
+                    }
+                    Err(s) => {
+                        let content = s.as_content();
+                        (InnerSas::Cancelled(s), Some(content))
+                    }
+                },
+                _ => (self, None),
+            },
             AnyVerificationContent::Cancel(c) => {
                 let (sas, _) = self.cancel(false, c.cancel_code().to_owned());
                 (sas, None)
@@ -328,10 +317,7 @@ impl InnerSas {
             },
             AnyVerificationContent::Done(c) => match self {
                 InnerSas::WaitingForDone(s) => match s.into_done(sender, c) {
-                    Ok(s) => {
-                        let content = s.done_content();
-                        (InnerSas::Done(s), Some(content))
-                    }
+                    Ok(s) => (InnerSas::Done(s), None),
                     Err(s) => {
                         let content = s.as_content();
                         (InnerSas::Cancelled(s), Some(content))
@@ -376,22 +362,7 @@ impl InnerSas {
         }
     }
 
-    pub fn verification_flow_id(&self) -> Arc<FlowId> {
-        match self {
-            InnerSas::Created(s) => s.verification_flow_id.clone(),
-            InnerSas::Started(s) => s.verification_flow_id.clone(),
-            InnerSas::Cancelled(s) => s.verification_flow_id.clone(),
-            InnerSas::Accepted(s) => s.verification_flow_id.clone(),
-            InnerSas::KeyReceived(s) => s.verification_flow_id.clone(),
-            InnerSas::Confirmed(s) => s.verification_flow_id.clone(),
-            InnerSas::MacReceived(s) => s.verification_flow_id.clone(),
-            InnerSas::WaitingForDone(s) => s.verification_flow_id.clone(),
-            InnerSas::Done(s) => s.verification_flow_id.clone(),
-            InnerSas::WeAccepted(s) => s.verification_flow_id.clone(),
-        }
-    }
-
-    pub fn emoji(&self) -> Option<[(&'static str, &'static str); 7]> {
+    pub fn emoji(&self) -> Option<[Emoji; 7]> {
         match self {
             InnerSas::KeyReceived(s) => Some(s.get_emoji()),
             InnerSas::MacReceived(s) => Some(s.get_emoji()),
