@@ -17,32 +17,30 @@ mod inner_sas;
 mod sas_state;
 
 use std::sync::{Arc, Mutex};
-#[cfg(test)]
-use std::time::Instant;
 
 use inner_sas::InnerSas;
-use matrix_sdk_common::uuid::Uuid;
+#[cfg(test)]
+use matrix_sdk_common::instant::Instant;
 use ruma::{
-    api::client::r0::keys::upload_signatures::Request as SignatureUploadRequest,
+    api::client::keys::upload_signatures::v3::Request as SignatureUploadRequest,
     events::{
         key::verification::{cancel::CancelCode, ShortAuthenticationString},
-        AnyMessageEventContent, AnyToDeviceEventContent,
+        AnyMessageLikeEventContent, AnyToDeviceEventContent,
     },
-    DeviceId, EventId, RoomId, UserId,
+    DeviceId, OwnedEventId, OwnedRoomId, OwnedTransactionId, RoomId, TransactionId, UserId,
 };
 use tracing::trace;
 
 use super::{
     event_enums::{AnyVerificationContent, OutgoingContent, OwnedAcceptContent, StartContent},
     requests::RequestHandle,
-    CancelInfo, FlowId, IdentitiesBeingVerified, VerificationResult, VerificationStore,
+    CancelInfo, FlowId, IdentitiesBeingVerified, VerificationResult,
 };
 use crate::{
     identities::{ReadOnlyDevice, ReadOnlyUserIdentities},
-    olm::PrivateCrossSigningIdentity,
     requests::{OutgoingVerificationRequest, RoomMessageRequest},
     store::CryptoStoreError,
-    ReadOnlyAccount, ReadOnlyOwnUserIdentity, ToDeviceRequest,
+    Emoji, ReadOnlyAccount, ToDeviceRequest,
 };
 
 /// Short authentication string object.
@@ -144,33 +142,33 @@ impl Sas {
     }
 
     fn start_helper(
-        inner_sas: InnerSas,
-        private_identity: PrivateCrossSigningIdentity,
-        other_device: ReadOnlyDevice,
-        store: VerificationStore,
-        other_identity: Option<ReadOnlyUserIdentities>,
+        flow_id: FlowId,
+        identities: IdentitiesBeingVerified,
         we_started: bool,
         request_handle: Option<RequestHandle>,
-    ) -> Sas {
-        let flow_id = inner_sas.verification_flow_id();
+    ) -> (Sas, OutgoingContent) {
+        let (inner, content) = InnerSas::start(
+            identities.store.account.clone(),
+            identities.device_being_verified.clone(),
+            identities.own_identity.clone(),
+            identities.identity_being_verified.clone(),
+            flow_id.clone(),
+            request_handle.is_some(),
+        );
 
-        let account = store.account.clone();
+        let account = identities.store.account.clone();
 
-        let identities = IdentitiesBeingVerified {
-            private_identity,
-            store,
-            device_being_verified: other_device,
-            identity_being_verified: other_identity,
-        };
-
-        Sas {
-            inner: Arc::new(Mutex::new(inner_sas)),
-            account,
-            identities_being_verified: identities,
-            flow_id,
-            we_started,
-            request_handle,
-        }
+        (
+            Sas {
+                inner: Arc::new(Mutex::new(inner)),
+                account,
+                identities_being_verified: identities,
+                flow_id: flow_id.into(),
+                we_started,
+                request_handle,
+            },
+            content,
+        )
     }
 
     /// Start a new SAS auth flow with the given device.
@@ -183,37 +181,15 @@ impl Sas {
     ///
     /// Returns the new `Sas` object and a `StartEventContent` that needs to be
     /// sent out through the server to the other device.
-    #[allow(clippy::too_many_arguments)]
     pub(crate) fn start(
-        private_identity: PrivateCrossSigningIdentity,
-        other_device: ReadOnlyDevice,
-        store: VerificationStore,
-        own_identity: Option<ReadOnlyOwnUserIdentity>,
-        other_identity: Option<ReadOnlyUserIdentities>,
-        transaction_id: Option<String>,
+        identities: IdentitiesBeingVerified,
+        transaction_id: OwnedTransactionId,
         we_started: bool,
         request_handle: Option<RequestHandle>,
     ) -> (Sas, OutgoingContent) {
-        let (inner, content) = InnerSas::start(
-            store.account.clone(),
-            other_device.clone(),
-            own_identity,
-            other_identity.clone(),
-            transaction_id,
-        );
+        let flow_id = FlowId::ToDevice(transaction_id);
 
-        (
-            Self::start_helper(
-                inner,
-                private_identity,
-                other_device,
-                store,
-                other_identity,
-                we_started,
-                request_handle,
-            ),
-            content,
-        )
+        Self::start_helper(flow_id, identities, we_started, request_handle)
     }
 
     /// Start a new SAS auth flow with the given device inside the given room.
@@ -228,37 +204,14 @@ impl Sas {
     /// sent out through the server to the other device.
     #[allow(clippy::too_many_arguments)]
     pub(crate) fn start_in_room(
-        flow_id: EventId,
-        room_id: RoomId,
-        private_identity: PrivateCrossSigningIdentity,
-        other_device: ReadOnlyDevice,
-        store: VerificationStore,
-        own_identity: Option<ReadOnlyOwnUserIdentity>,
-        other_identity: Option<ReadOnlyUserIdentities>,
+        flow_id: OwnedEventId,
+        room_id: OwnedRoomId,
+        identities: IdentitiesBeingVerified,
         we_started: bool,
         request_handle: RequestHandle,
     ) -> (Sas, OutgoingContent) {
-        let (inner, content) = InnerSas::start_in_room(
-            flow_id,
-            room_id,
-            store.account.clone(),
-            other_device.clone(),
-            own_identity,
-            other_identity.clone(),
-        );
-
-        (
-            Self::start_helper(
-                inner,
-                private_identity,
-                other_device,
-                store,
-                other_identity,
-                we_started,
-                Some(request_handle),
-            ),
-            content,
-        )
+        let flow_id = FlowId::InRoom(room_id, flow_id);
+        Self::start_helper(flow_id, identities, we_started, Some(request_handle))
     }
 
     /// Create a new Sas object from a m.key.verification.start request.
@@ -271,37 +224,33 @@ impl Sas {
     ///
     /// * `event` - The m.key.verification.start event that was sent to us by
     /// the other side.
-    #[allow(clippy::too_many_arguments)]
     pub(crate) fn from_start_event(
         flow_id: FlowId,
-        content: &StartContent,
-        store: VerificationStore,
-        private_identity: PrivateCrossSigningIdentity,
-        other_device: ReadOnlyDevice,
-        own_identity: Option<ReadOnlyOwnUserIdentity>,
-        other_identity: Option<ReadOnlyUserIdentities>,
+        content: &StartContent<'_>,
+        identities: IdentitiesBeingVerified,
         request_handle: Option<RequestHandle>,
         we_started: bool,
     ) -> Result<Sas, OutgoingContent> {
         let inner = InnerSas::from_start_event(
-            store.account.clone(),
-            other_device.clone(),
-            flow_id,
+            identities.store.account.clone(),
+            identities.device_being_verified.clone(),
+            flow_id.clone(),
             content,
-            own_identity,
-            other_identity.clone(),
+            identities.own_identity.clone(),
+            identities.identity_being_verified.clone(),
             request_handle.is_some(),
         )?;
 
-        Ok(Self::start_helper(
-            inner,
-            private_identity,
-            other_device,
-            store,
-            other_identity,
+        let account = identities.store.account.clone();
+
+        Ok(Sas {
+            inner: Arc::new(Mutex::new(inner)),
+            account,
+            identities_being_verified: identities,
+            flow_id: flow_id.into(),
             we_started,
             request_handle,
-        ))
+        })
     }
 
     /// Accept the SAS verification.
@@ -336,8 +285,8 @@ impl Sas {
                 }
                 OwnedAcceptContent::Room(room_id, content) => RoomMessageRequest {
                     room_id,
-                    txn_id: Uuid::new_v4(),
-                    content: AnyMessageEventContent::KeyVerificationAccept(content),
+                    txn_id: TransactionId::new(),
+                    content: AnyMessageLikeEventContent::KeyVerificationAccept(content),
                 }
                 .into(),
             })
@@ -355,27 +304,29 @@ impl Sas {
     /// the server.
     pub async fn confirm(
         &self,
-    ) -> Result<
-        (Option<OutgoingVerificationRequest>, Option<SignatureUploadRequest>),
-        CryptoStoreError,
-    > {
-        let (content, done) = {
+    ) -> Result<(Vec<OutgoingVerificationRequest>, Option<SignatureUploadRequest>), CryptoStoreError>
+    {
+        let (contents, done) = {
             let mut guard = self.inner.lock().unwrap();
             let sas: InnerSas = (*guard).clone();
-            let (sas, content) = sas.confirm();
+            let (sas, contents) = sas.confirm();
 
             *guard = sas;
-            (content, guard.is_done())
+            (contents, guard.is_done())
         };
 
-        let mac_request = content.map(|c| match c {
-            OutgoingContent::ToDevice(c) => self.content_to_request(c).into(),
-            OutgoingContent::Room(r, c) => {
-                RoomMessageRequest { room_id: r, txn_id: Uuid::new_v4(), content: c }.into()
-            }
-        });
+        let mac_requests = contents
+            .into_iter()
+            .map(|c| match c {
+                OutgoingContent::ToDevice(c) => self.content_to_request(c).into(),
+                OutgoingContent::Room(r, c) => {
+                    RoomMessageRequest { room_id: r, txn_id: TransactionId::new(), content: c }
+                        .into()
+                }
+            })
+            .collect::<Vec<_>>();
 
-        if mac_request.is_some() {
+        if !mac_requests.is_empty() {
             trace!(
                 user_id = self.other_user_id().as_str(),
                 device_id = self.other_device_id().as_str(),
@@ -385,12 +336,14 @@ impl Sas {
 
         if done {
             match self.mark_as_done().await? {
-                VerificationResult::Cancel(c) => Ok((self.cancel_with_code(c), None)),
-                VerificationResult::Ok => Ok((mac_request, None)),
-                VerificationResult::SignatureUpload(r) => Ok((mac_request, Some(r))),
+                VerificationResult::Cancel(c) => {
+                    Ok((self.cancel_with_code(c).into_iter().collect(), None))
+                }
+                VerificationResult::Ok => Ok((mac_requests, None)),
+                VerificationResult::SignatureUpload(r) => Ok((mac_requests, Some(r))),
             }
         } else {
-            Ok((mac_request, None))
+            Ok((mac_requests, None))
         }
     }
 
@@ -416,7 +369,7 @@ impl Sas {
     ///
     /// **Note**: This method should generally not be used, the [`cancel()`]
     /// method should be preferred. The SDK will automatically cancel with the
-    /// approprate cancel code, user initiated cancellations should only cancel
+    /// appropriate cancel code, user initiated cancellations should only cancel
     /// with the `CancelCode::User`
     ///
     /// Returns None if the `Sas` object is already in a canceled state,
@@ -427,7 +380,7 @@ impl Sas {
         let mut guard = self.inner.lock().unwrap();
 
         if let Some(request) = &self.request_handle {
-            request.cancel_with_code(&code)
+            request.cancel_with_code(&code);
         }
 
         let sas: InnerSas = (*guard).clone();
@@ -435,7 +388,7 @@ impl Sas {
         *guard = sas;
         content.map(|c| match c {
             OutgoingContent::Room(room_id, content) => {
-                RoomMessageRequest { room_id, txn_id: Uuid::new_v4(), content }.into()
+                RoomMessageRequest { room_id, txn_id: TransactionId::new(), content }.into()
             }
             OutgoingContent::ToDevice(c) => self.content_to_request(c).into(),
         })
@@ -475,7 +428,7 @@ impl Sas {
     ///
     /// Returns None if we can't yet present the short auth string, otherwise
     /// seven tuples containing the emoji and description.
-    pub fn emoji(&self) -> Option<[(&'static str, &'static str); 7]> {
+    pub fn emoji(&self) -> Option<[Emoji; 7]> {
         self.inner.lock().unwrap().emoji()
     }
 
@@ -501,7 +454,7 @@ impl Sas {
     pub(crate) fn receive_any_event(
         &self,
         sender: &UserId,
-        content: &AnyVerificationContent,
+        content: &AnyVerificationContent<'_>,
     ) -> Option<OutgoingContent> {
         let mut guard = self.inner.lock().unwrap();
         let sas: InnerSas = (*guard).clone();
@@ -554,10 +507,12 @@ impl AcceptSettings {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use std::{convert::TryFrom, sync::Arc};
 
-    use ruma::{DeviceId, UserId};
+    use matrix_sdk_common::locks::Mutex;
+    use matrix_sdk_test::async_test;
+    use ruma::{device_id, user_id, DeviceId, TransactionId, UserId};
 
     use super::Sas;
     use crate::{
@@ -570,64 +525,54 @@ mod test {
         ReadOnlyAccount, ReadOnlyDevice,
     };
 
-    fn alice_id() -> UserId {
-        UserId::try_from("@alice:example.org").unwrap()
+    fn alice_id() -> &'static UserId {
+        user_id!("@alice:example.org")
     }
 
-    fn alice_device_id() -> Box<DeviceId> {
-        "JLAFKJWSCS".into()
+    fn alice_device_id() -> &'static DeviceId {
+        device_id!("JLAFKJWSCS")
     }
 
-    fn bob_id() -> UserId {
-        UserId::try_from("@bob:example.org").unwrap()
+    fn bob_id() -> &'static UserId {
+        user_id!("@bob:example.org")
     }
 
-    fn bob_device_id() -> Box<DeviceId> {
-        "BOBDEVCIE".into()
+    fn bob_device_id() -> &'static DeviceId {
+        device_id!("BOBDEVCIE")
     }
 
-    #[tokio::test]
+    #[async_test]
     async fn sas_wrapper_full() {
-        let alice = ReadOnlyAccount::new(&alice_id(), &alice_device_id());
+        let alice = ReadOnlyAccount::new(alice_id(), alice_device_id());
         let alice_device = ReadOnlyDevice::from_account(&alice).await;
 
-        let bob = ReadOnlyAccount::new(&bob_id(), &bob_device_id());
+        let bob = ReadOnlyAccount::new(bob_id(), bob_device_id());
         let bob_device = ReadOnlyDevice::from_account(&bob).await;
 
-        let alice_store =
-            VerificationStore { account: alice.clone(), inner: Arc::new(MemoryStore::new()) };
+        let alice_store = VerificationStore {
+            account: alice.clone(),
+            inner: Arc::new(MemoryStore::new()),
+            private_identity: Mutex::new(PrivateCrossSigningIdentity::empty(alice_id())).into(),
+        };
 
         let bob_store = MemoryStore::new();
         bob_store.save_devices(vec![alice_device.clone()]).await;
 
-        let bob_store = VerificationStore { account: bob.clone(), inner: Arc::new(bob_store) };
+        let bob_store = VerificationStore {
+            account: bob.clone(),
+            inner: Arc::new(bob_store),
+            private_identity: Mutex::new(PrivateCrossSigningIdentity::empty(bob_id())).into(),
+        };
 
-        let (alice, content) = Sas::start(
-            PrivateCrossSigningIdentity::empty(alice_id()),
-            bob_device,
-            alice_store,
-            None,
-            None,
-            None,
-            true,
-            None,
-        );
+        let identities = alice_store.get_identities(bob_device).await.unwrap();
+
+        let (alice, content) = Sas::start(identities, TransactionId::new(), true, None);
 
         let flow_id = alice.flow_id().to_owned();
         let content = StartContent::try_from(&content).unwrap();
 
-        let bob = Sas::from_start_event(
-            flow_id,
-            &content,
-            bob_store,
-            PrivateCrossSigningIdentity::empty(bob_id()),
-            alice_device,
-            None,
-            None,
-            None,
-            false,
-        )
-        .unwrap();
+        let identities = bob_store.get_identities(alice_device).await.unwrap();
+        let bob = Sas::from_start_event(flow_id, &content, identities, None, false).unwrap();
 
         let request = bob.accept().unwrap();
         let content = OutgoingContent::try_from(request).unwrap();
@@ -650,12 +595,16 @@ mod test {
         assert_eq!(alice.emoji().unwrap(), bob.emoji().unwrap());
         assert_eq!(alice.decimals().unwrap(), bob.decimals().unwrap());
 
-        let request = alice.confirm().await.unwrap().0.unwrap();
+        let mut requests = alice.confirm().await.unwrap().0;
+        assert!(requests.len() == 1);
+        let request = requests.pop().unwrap();
         let content = OutgoingContent::try_from(request).unwrap();
         let content = MacContent::try_from(&content).unwrap();
         bob.receive_any_event(alice.user_id(), &content.into());
 
-        let request = bob.confirm().await.unwrap().0.unwrap();
+        let mut requests = bob.confirm().await.unwrap().0;
+        assert!(requests.len() == 1);
+        let request = requests.pop().unwrap();
         let content = OutgoingContent::try_from(request).unwrap();
         let content = MacContent::try_from(&content).unwrap();
         alice.receive_any_event(bob.user_id(), &content.into());
